/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2005, 2007                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20041001  Initial Version                                         */
/* 20060403  F14109  Add tpf_stpoc to API5 driver.                   */
/* 20060626  F14591  fix the API driver dump when displaying help    */
/* 20060929  F14494  Add usergroup to API5 driver.                   */
/* 20070205  cpacfdrv CPACF driver                                   */
/* 20081024  D14942  Fix the CTL-4 dump for ztest api5 api-all       */
/* 20081125  D14942_1  Fix the CTL-3 dump for ztest api5 api-all     */
/*                                                                   */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name:  qmac00.cpp                                         */
/*                                                                   */
/* Abstract:      Member of the QMAC DLM which comprises the         */
/*                z/TPF API driver.                                  */
/*                                                                   */
/* Description:   This segment is the main entry point for the API   */
/*                driver.  It will perform parsing and invoke the    */
/*                appropriate routine.                               */
/*                                                                   */
/*                This segment works with a table-driven design.     */
/*                A table entry exists for each macro/C-function,    */
/*                API or group of APIs that is supported by this     */
/*                driver.                                            */
/*                The table entry consists of the following:         */
/*                                                                   */
/*                      - name of the API                            */
/*                      - pointer to routine to assembler routine    */
/*                      - pointer to C++ function                    */
/*                                                                   */
/* ECB fields                                                        */
/* ebx000(1)  error case indicator                                   */
/* ebx001(1)  last iteration indicator                               */
/* ebx002(4)  thread count                                           */
/* ebcm01(1)  verbose indicator                                      */
/*                                                                   */
/*********************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <tpf/tpfapi.h>
#include <tpf/tpfio.h>
#include <tpf/tpfparse.h>
#include <tpf/i_decb.h>
#include <tpf/c_stck.h>
#include <c_api5.h>




#define QMAC_GRAMMAR "{API-caa+++ [(NO)ERR] [(NO)Verbose] {[CNT-d++++] | [TIme-d+++]} [CASE-d+] [(NO)BSO] [(NO)CSO] [PName-caaa] [LSET-a+++++++] [SNAP-d++] [THR-d+++]}"                                                        //F14494


/*-------------------------------------------------------------------*/
/* make all function extern C to simplify the linkage                */
/*-------------------------------------------------------------------*/

extern "C" {

/*-------------------------------------------------------------------*/
/* function prototype                                                */
/*-------------------------------------------------------------------*/
boolean getapi(char *api, unsigned char *errcase, int *count,
               int *seconds, unsigned char *verbose, api5parms *,
               int *threads);                                 //F14494

void attac_macro(void);
void attac_func(void);
void caloc_macro(void);
void caloc_func(void);
void cfisvc_macro(void);
void cfisvc_func(void);
void cifrc_func(void);
void clinkc_macro(void);
void corhc_macro(void);
void corhc_func(void);
void coruc_macro(void);
void coruc_func(void);
void cratc_macro(void);
void cratc_func(void);
void cresvc_macro(void);
void cresvc_func(void);
void crexc_macro(void);
void crexc_func(void);
void crosc_macro(void);
void crosc_func(void);
void cryptc_func(void);
void debugc_macro(void);
void debugc_func(void);
void defbc_macro(void);
void detac_macro(void);
void dstoc_macro(void);
void dstoc_func(void);
void ecblc_macro(void);
void ecblc_func(void);
void errnoc_macro(void);
void entrc_func(void);
void fiwhc_macro(void);
void fiwhc_func(void);
void flipc_macro(void);
void flipc_func(void);
void freec_macro(void);
void freec_func(void);
void fsysc_macro(void);
void fsysc_func(void);
void genlc_func(void);
void gevac_macro(void);
void gevac_func(void);
void gflmc_macro(void);
void gflmc_func(void);
void gstoc_macro(void);
void gstoc_func(void);
void gsvac_macro(void);
void gsvac_func(void);
void gswbc_macro(void);
void gswbc_func(void);
void gsysc_macro(void);
void gsysc_func(void);
void linku_macro(void);
void linku_func(void);
void lodic_macro(void);
void lodic_func(void);
void maloc_macro(void);
void maloc_func(void);
void maxbc_macro(void);
void maxbc_func(void);
void movec_macro(void);
void movec_func(void);
void movec_ret_func(void);
void numbc_macro(void);
void numbc_func(void);
void pausc_func(void);                                        //f15945
void phybc_macro(void);
void phybc_func(void);
void pnamc_macro(void);
void pnamc_func(void);
void raloc_macro(void);
void raloc_func(void);
void recvc_macro(void);
void recvc_func(void);
void rehka_macro(void);
void rehka_func(void);
void ritid_macro(void);
void ritid_func(void);
void rlfmc_macro(void);
void rlfmc_func(void);
void rstoc_macro(void);
void rstoc_func(void);
void rsysc_macro(void);
void rsysc_func(void);
void serrc_macro(void);
void serrc_func(void);
void snapc_macro(void);
void snapc_func(void);
void sonic_macro(void);
void sonic_func(void);
void stpoc_func(void);                                                          //D14109
void swisc_macro(void);
void tamcc_macro(void);
void tamcc_func(void);
void tmslc_macro(void);
void tmslc_func(void);
void uatbc_macro(void);
void uatbc_func(void);
void unhka_macro(void);
void unhka_func(void);
void uxcmc_macro(void);
void uxcmc_func(void);
void uxrit_func(void);                                                         //f10789
void valbc_macro(void);
void valbc_func(void);
void wgtac_macro(void);
void wgtac_func(void);
void wtopc_macro(void);
void wtopc_func(void);
void tsgrp_func(void);
void getpc_func(void);
void getpc_macro(void);
void progc_func(void);
void progc_macro(void);
void usergroup_func(void);                                    //F14494
void drv_zdcrt_func(void);                                    //f15945


/*-------------------------------------------------------------------*/
/* structure definition                                              */
/*-------------------------------------------------------------------*/

typedef struct api_tbl {
            char           api_name[8];
            void           (*api_asm)(void); /* ptr to assem func    */
            void           (*api_c)(void);   /* ptr to C func        */
            } API_TBL;

const API_TBL  api_tbl[] =
{
/*    api   name           assembler rtn    c function               */
   {  "PNAMC",             pnamc_macro,     NULL       },     //D14942_1 Don't move this test case to other place
   {  "ATTAC",             attac_macro,     attac_func },
   {  "CALOC",             caloc_macro,     caloc_func },
   {  "CFISVC",            cfisvc_macro,    NULL       },
   {  "CIFRC",             NULL,            cifrc_func },
   {  "CLINKC",            clinkc_macro,    NULL       },
   {  "CORHC",             corhc_macro,     corhc_func },
   {  "CORUC",             coruc_macro,     NULL       },
   {  "CRATC",             cratc_macro,     cratc_func },
   {  "CRESVC",            NULL,            NULL       },
   {  "CREXC",             crexc_macro,     crexc_func },
   {  "CROSC",             NULL,            crosc_func },
   {  "CRYPC",             NULL,            cryptc_func},
   {  "DEBUGC",            NULL,            NULL       },
   {  "DEFBC",             defbc_macro,     NULL       },
   {  "DETAC",             detac_macro,     NULL       },
   {  "DSTOC",             NULL,            NULL       },
   {  "ECBLC",             ecblc_macro,     NULL       },
   {  "ENTRC",             NULL,            entrc_func },
   {  "ERRNOC",            errnoc_macro,    NULL       },
   {  "FIWHC",             NULL,            fiwhc_func },
   {  "FLIPC",             flipc_macro,     flipc_func },
   {  "FREEC",             freec_macro,     freec_func },
   {  "FSYSC",             fsysc_macro,     NULL       },
   {  "GENLC",             NULL,            genlc_func },
   {  "GEVAC",             NULL,            NULL       },
   {  "GFLMC",             NULL,            NULL       },
   {  "GSTOC",             NULL,            NULL       },
   {  "GSVAC",             gsvac_macro,     NULL       },
   {  "GSWBC",             gswbc_macro,     NULL       },
   {  "GSYSC",             gsysc_macro,     gsysc_func },
   {  "LINKU",             linku_macro,     NULL       },
   {  "LODIC",             NULL,            lodic_func },
   {  "MALOC",             maloc_macro,     maloc_func },
   {  "MAXBC",             maxbc_macro,     NULL       },
   {  "MOVEC",             NULL,            movec_func },
   {  "MOVCR",             NULL,        movec_ret_func },
   {  "NUMBC",             numbc_macro,     numbc_func },
   {  "PAUSC",             NULL,            pausc_func },                      //f15945
   {  "PHYBC",             phybc_macro,     NULL       },
   {  "RALOC",             raloc_macro,     raloc_func },
   {  "RECVC",             recvc_macro,     NULL       },
   {  "REHKA",             NULL,            rehka_func },
   {  "RITID",             ritid_macro,     NULL       },
   {  "RLFMC",             NULL,            NULL       },
   {  "RSTOC",             NULL,            NULL       },
   {  "RSYSC",             rsysc_macro,     rsysc_func },
   {  "SERRC",             NULL,            serrc_func },
   {  "SNAPC",             NULL,            snapc_func },
   {  "SONIC",             sonic_macro,     sonic_func },
   {  "STPOC",             NULL,            stpoc_func },                       //D14109
   {  "SWISC",             swisc_macro,     NULL       },
   {  "TAMCC",             tamcc_macro,     NULL       },
   {  "TMSLC",             NULL,            tmslc_func },
   {  "TSGRP",             NULL,            tsgrp_func },
   {  "UATBC",             uatbc_macro,     uatbc_func },
   {  "UNHKA",             NULL,            unhka_func },
   {  "UXCMC",             NULL,            NULL       },
   {  "UXRIT",             NULL,            uxrit_func },                      //f10789
   {  "VALBC",             valbc_macro,     NULL       },
   {  "WGTAC",             NULL,            wgtac_func },
   {  "GETPC",             getpc_macro,     getpc_func },
   {  "PROGC",             progc_macro,     progc_func },
   {  "USRGRP",            NULL,            usergroup_func },
   {  "CRET",			   NULL,            drv_zdcrt_func },                  //f15945
   {  "", NULL, NULL }  };

/*********************************************************************/
/*                                                                   */
/* QMAC( )                                                           */
/*                                                                   */
/* Entry point to the ZTEST API5  functional message.                */
/*                                                                   */
/*********************************************************************/
void QMAC(void)
{
   int                  i;
   int                  rc;
   char                 api[8] = { 0 };
   char                 help[255] = { 0 };
   char                 spaces[5] = { 0x40 };
   char                *buffer_ptr;
   boolean              found = FALSE;
   int                  count = 1;
   int                  seconds = 0;
   tpf_TOD_type         timeNow = {0};
   int                  startTime = 0;
   int                  currentTime = 0;
   int                  threads = 5;      /* default is 5 */      //F14494

   ecbptr()->ebrout = 0x010000;
   ecbptr()->ebx000 = FALSE;              /*default:  error cases = no    */
   ecbptr()->ebx001 = FALSE;              /*default:  last iteration = no */
   ecbptr()->ebcm01 = TRUE;               /*default:  verbose = yes       */
   memcpy (&ecbptr()->ebx002, &threads, 4);                       //F14494
	
   api5parms *api5parms_p;

   api5parms_p = (api5parms *)malloc(sizeof(api5parms));
   memcpy(&(ecbptr()->ebx008),&api5parms_p,sizeof(api5parms_p));
   api5parms_p->icase = 0;
   api5parms_p->bso = FALSE;
   api5parms_p->cso = FALSE;
   api5parms_p->snap = 0;
   memcpy(api5parms_p->pname,"CVZZ",sizeof(api5parms_p->pname));
   memset(api5parms_p->lset,' ',sizeof(api5parms_p->lset));

   if ( getapi(api, &(ecbptr()->ebx000), &count, &seconds, &(ecbptr()->ebcm01),api5parms_p, (int*)&(ecbptr()->ebx002)) == FALSE )                //F14494
   {
/*********************************************************************************/
/*                           help display                                        */
      sprintf(help, "\nFORMAT ZTEST API5 API-name \n"
              " (NO)ERR (NO)Verbose CNT-count TIme-seconds (NO)BSO (NO)CSO \n\n"
              "API-name is required, specify ALL or a specific API \n\n"
              "The following parameters are optional:");
      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);
      memset (help, 0x00, sizeof(help));
      sprintf(help, " ERR executes error conditions, the default is NOERR \n"
              " NOVerbose suppresses messages, the default is Verbose \n"
              " CNT or TIme may be specified to repeat the request");

      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);
      memset (help, 0x00, sizeof(help));
      sprintf(help, " NOBSO skips the BSO tests, the default is BSO \n"
              " NOCSO skips CSO tests, the default is CSO \n\n"
              "The following parameters are limited to specific APIs:");

      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);
      memset (help, 0x00, sizeof(help));
      sprintf(help, " LSET-loadset for those APIs requiring a loadset name\n"
              " PName-prog   designates a specific program name \n"
              " SNAP-bytes   to dump to screen up to 999 bytes\n"
              " CASE-xx      designates a specific test case \n"
              " THR-xx       number of threads");                //F14494

      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);

      memset (help, 0x00, sizeof(help));
      sprintf(help, "Macros and functions supported by API5: \n");

      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);
      memset (spaces,' ', sizeof(spaces));
      count = 0;
      memset (help, 0x00, sizeof(help));
      buffer_ptr = help;
      for ( i = 0; (memcmp(api_tbl[i].api_name, "", 1)!=0); i++ )
      {
         if ( buffer_ptr >= (help + sizeof(help) - sizeof(api) - sizeof(spaces) -count - 2 ))
         {
            rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
            if ( rc == WTOPC_PAGE_STOP )
               exit(0);
            memset (help, 0x00, sizeof(help));
            count = 0;
            buffer_ptr = help;
         }
         buffer_ptr += sprintf (buffer_ptr, "%s%.5s",
                                api_tbl[i].api_name,
                                spaces);
         ++count;
         if ( count == 4 )
         {
            memcpy(buffer_ptr, "\n", 1);
            ++buffer_ptr;
            count = 0;
         }
      }
      rc = wtopc(help, WTOPC_EBROUT, WTOPC_PAGE, WTOPC_NO_HEADER);
      if ( rc == WTOPC_PAGE_STOP )
         exit(0);
      wtopc("End of API5 help display", WTOPC_EBROUT, WTOPC_PAGE_END, WTOPC_NO_HEADER);
      exit(0);
   }
/*********************************************************************************/

   tpf_STCK (&timeNow); /* get the value of TOD clock */
   if ( seconds )
      count = 2;

   /* use left most 4 bytes of TOD clock as an approximation of      */
   /* a "seconds" counter                                            */
   memcpy (&startTime, &timeNow, sizeof(startTime));

   if ( memcmp(api, "ALL", 3) == 0 )    /* execute cases for all api */
   {
       if (ecbptr()->ebx000)
          prtmsg("executing error cases for all API \n");
       else
          prtmsg("executing normal cases for all API \n");

       while (count > 0)
       {
           /*  if this is the only or last iteration, set the indicator */
           if ( count == 1 )
              ecbptr()->ebx001 = TRUE;
           for ( i = 0; (memcmp(api_tbl[i].api_name, "", 1)!=0); i++ )
           {
               /* The following if is required so that PAUSC test cases do not */
               /* get called in continuous mode run. F15945                    */
               if (memcmp(api_tbl[i].api_name, "PAUSC", 5))
               {
                  defrc();
                  if ( api_tbl[i].api_asm )
                  {
                     (*api_tbl[i].api_asm)();
                  }
                  if ( api_tbl[i].api_c )
                  {
                     (*api_tbl[i].api_c)();
                  }
               }
           }

           /* if the user specified an amount of time that the       */
           /* driver should run for, calculate if that time has      */
           /* expired yet.  If it has, decrement the count variable  */
           /* so we will be forced out of the while loop; else,      */
           /* continue in the loop.                                  */
           if (seconds > 0)
           {
               tpf_STCK (&timeNow);
               memcpy (&currentTime, &timeNow, sizeof(currentTime));
               if (currentTime - startTime >= seconds)
               {
                  /* time is up, force the next iteration to be the last */
                  seconds = 0;
                  count = 1;
               }
           }
           /* if the number of seconds to run was not specified,     */
           /* just decrement the count variable so we know whether   */
           /* or not to continue in the loop                         */
           else
               count--;

       }

       found = TRUE;
    }
    else
    {                           /* execute specific API test cases   */
       for ( i = 0; (memcmp(api_tbl[i].api_name, "", 1)!=0); i++ )
       {
          if ( memcmp(api, api_tbl[i].api_name, strlen(api)) == 0)
          {
              found = TRUE;

              while (count > 0)
              {
                  /*  if this is the only or last iteration, set the indicator */
                  if ( count == 1 )
                     ecbptr()->ebx001 = TRUE;
                  defrc();
                   if ( ((api_tbl[i].api_asm ) && api5parms_p->bso == TRUE) || ( (api_tbl[i].api_asm) && api5parms_p->bso == FALSE && api5parms_p->cso == FALSE))
                  {
                     (*api_tbl[i].api_asm)();
                  }
                  if ( (( api_tbl[i].api_c ) && api5parms_p->cso == TRUE) || ( ( api_tbl[i].api_c ) && api5parms_p->bso == FALSE && api5parms_p->cso == FALSE))
                  {
                     (*api_tbl[i].api_c)();
                  }
                  /* if the user specified an amount of time that*/
                  /* the driver should run for, calculate if that*/
                  /* time has expired yet.  If it has, decrement */
                  /* the count variable so we will be forced out */
                  /* of the while loop; else, just release all   */
                  /* DECBs and continue in the loop.             */
                  if (seconds > 0)
                  {
                      tpf_STCK (&timeNow);
                      memcpy (&currentTime, &timeNow,
                              sizeof(currentTime));
                      if (currentTime - startTime >= seconds)
                      {
                         /* time is up, force the next iteration to be the last */
                         seconds = 0;
                         count = 1;
                      }
                  }
                  /* if the number of seconds to run was not     */
                  /* specified, just decrement the count variable*/
                  /* so we know whether or not to continue in the*/
                  /* loop                                        */
                  else
                      count--;

              }
              break;

          } /* end if the matching API is found */
       }  /* end for loop that searches specific API */

       if ( ! found )
       {
          prtmsg(api);
          prtmsg(":  is not in the API table, no test cased executed \n");
       }
    } /* end executing specific API test cases */

   printf("API5 driver exited \n");

   exit(0);
}


/*********************************************************************/
/*                                                                   */
/* getapi( )                                                         */
/*                                                                   */
/* Routine to parse the input functional message and extract the     */
/* name of the api   which is to be executed and the error case      */
/* number if supplied.                                               */
/*                                                                   */
/*********************************************************************/
boolean getapi(char *api, unsigned char *errcase, int *count, int *seconds, unsigned char *verbose, api5parms *api5parms_p, int *threads)     //F14494
{
   int    i;
   struct IPRSE_output  parsed_data, *pparm;
   struct mi0mi        *blk_ptr;
   char                *input_ptr, *eom_ptr;
   int                  parser_return;

   blk_ptr = (struct mi0mi *)ecbptr()->ce1cr0;
   input_ptr = blk_ptr->mi0acc+strlen("Zxxxx");
   eom_ptr = (char *)&blk_ptr->mi0ln0 + blk_ptr->mi0cct - 1;
   *eom_ptr = '\0';

   parser_return = IPRSE_parse(input_ptr,
                              QMAC_GRAMMAR,
                              &parsed_data,
                              IPRSE_ALLOC+IPRSE_PRINT,
                              "QMAC");

   /******************************************************************/
   /* return -1 indicates HELP or ?                                  */
   /* return  0 indicates message did not meet grammar requirements  */
   /******************************************************************/

   if ( parser_return == -1 )
   {
      return(FALSE);
   }
   if ( parser_return == 0 )
   {
      return(FALSE);
   }

   /******************************************************************/
   /* Else, determine which parameters were input and set the        */
   /* appropriate values to return to the caller so the corresponding*/
   /* APIs can be tested.                                            */
   /******************************************************************/
   for ( i = 1, pparm = &parsed_data;
         i <= parser_return;
         i++ , pparm = pparm->IPRSE_next )
   {
      if ( memcmp(pparm->IPRSE_parameter, "API", 3) == 0 )
      {
         memcpy(api, pparm->IPRSE_value, strlen(pparm->IPRSE_value));
      }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)ERR") == 0 )
           {
               *errcase = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
           }
      else if ( memcmp(pparm->IPRSE_parameter, "CNT", 3) == 0 )
           {
               *count = atoi(pparm->IPRSE_value);
           }
      else if ( memcmp(pparm->IPRSE_parameter, "TIme", 4) == 0 )
           {
               *seconds = atoi(pparm->IPRSE_value);
           }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)Verbose") == 0 )
           {
               *verbose = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
           }
      else if ( memcmp(pparm->IPRSE_parameter, "CASE",4) == 0 )
           {
               api5parms_p->icase = atoi(pparm->IPRSE_value);
           }
      else if ( memcmp(pparm->IPRSE_parameter, "PName", 5) == 0 )
           {
              memcpy(api5parms_p->pname, pparm->IPRSE_value, strlen(pparm->IPRSE_value));
           }
      else if ( memcmp(pparm->IPRSE_parameter, "LSET", 4) == 0 )
           {
              memcpy(api5parms_p->lset, pparm->IPRSE_value, strlen(pparm->IPRSE_value));
           }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)BSO") == 0 )
           {
                api5parms_p->bso = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
           }
      else if ( strcmp(pparm->IPRSE_parameter, "(NO)CSO") == 0 )
           {
                api5parms_p->cso = *(pparm->IPRSE_value) == 'Y' ? TRUE : FALSE;
           }
      else if ( memcmp(pparm->IPRSE_parameter, "SNAP", 4) == 0 )
           {
              api5parms_p->snap = (short int)(atoi(pparm->IPRSE_value));
           }
      else if ( memcmp(pparm->IPRSE_parameter, "THR", 3) == 0 )      //F14494
           {                                                         //F14494
              *threads =  atoi(pparm->IPRSE_value);                  //F14494
           }                                                         //F14494

   }


   return(TRUE);
}


} /* end of extern "C"                                               */
