/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2005                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20041001  Initial Version                                         */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name:  qmac10.cpp                                         */
/*                                                                   */
/* Abstract:      This driver tests the use of the following         */
/*                functions:                                         */
/*                   attac   -   attac_func()                        */
/*                   flipc   -   flipc_func()                        */
/*                   crosc   -   crosc_func()                        */
/*                   caloc   -   caloc_func()                        */
/*                   maloc   -   maloc_func()                        */
/*                   raloc   -   raloc_func()                        */
/*                   freec   -   freec_func()                        */
/*                   gsysc   -   gsysc_func()                        */
/*                   rsysc   -   rsysc_func()                        */
/*                   rehka   -   rehka_func()                        */
/*                   unhka   -   unhka_func()                        */
/*                   fiwhc   -   fiwhc_func()                        */
/*                                                                   */
/* Description:   The driver to run this program is qmac00.cpp.  To  */
/*                enter the flipc_func to test the flipc() function, */
/*                the user would enter:  ZTEST API-FLIPC after having*/
/*                defined a Z message:  ZFMSG add ZTEST prog-QMAC.   */
/*                The following is the grammar to this driver:       */
/*                   {API-caa+++ [(NO)ERR] [(NO)Verbose]             */
/*                            {[CNT-d++++] | [TIme-d+++]}}           */
/*********************************************************************/
#include <c_api5.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <tpf/tpfapi.h>
#include <tpf/c_eb0eb.h>
#include <tpf/tpfio.h>
#include <unistd.h>                                                             //C64CF
#include <tpf/sysapi.h>
#include <tpf/c_uatbc.h>
#include <tpf/c_ms0ut.h>

#define  REC_ID     "OM"          /* Record ID for fiwhc_func() "OM" */
long  TWO_GIG_BAR = 0x7FFFFFFF;   /* Used to compare Above/Below Bar */

extern "C"{
/*********************************************************************/
/*********************************************************************/
/* Function Name:    attac_func()                                    */
/*                                                                   */
/* Number of Cases:  2 regular exit test cases                       */
/*                   5 system exit test cases                        */
/*                                                                   */
/*********************************************************************/
void attac_func(void)
{
  int MAX_SIZE = 100;
  char parm1[3] = { '0', '1' , '0' }, parm2[3] = { '0', '2' , '0' },
       parm3[3] = { '0', '3' , '0' }, parm4[3] = { '0', '4' , '0' },
       parm5[3] = { '0', '5' , '0' };
  prtmsg("attac: entering attac_func\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {

    crusa(2,D1,D2);           /* releases the core blocks if occupied */   //d12221

    /*****************************************************************/
    /* Test Case 1:       Attach a working storage block that has    */
    /*                    been previously detached.                  */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("attac: 1-Attach D1 to previously Detached D1..\n");
    getcc(D1, (enum t_getfmt)(GETCC_TYPE), L2);

    if(levtest(D1))                /* if the DataLevel is occupied.. */
    {
      /***************************************************************/
      /* Place a footprint in the DataLevel before detaching, then   */
      /* attach the Datalevel and make sure the correct DataLevel    */
      /* was attached.                                               */
      /***************************************************************/
      ecbptr()->ce1fx1[0] = (unsigned char) 'A';

      detac(D1);
      ecbptr()->ce1fx1[0] = NULL;

      if(!levtest(D1))
      {
        attac(D1);

        if(ecbptr()->ce1fx1[0] == 'A')
        {
          if(levtest(D1))
            prtmsg("attac: 1-Successfully Completed.\n");
          else
            prterr("attac: 1-Unable to attac D1.");
        }
        else
          prterr("attac: 1-Incorrect DataLevel was attached.");
      }
      else
        prterr("attac: 1-Unable to detac D1 - not an attac error.");

      /* Release the working storage block on Data Level D1.         */
      relcc(D1);
    }
    else
      prterr("attac: 1-getcc() failed - not an attac error.");

    /*****************************************************************/
    /* Test Case 2:       Testing use of multiple DETAC's and        */
    /*                    ATTAC's.                                   */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("attac: 2-Attach and Detach D2 multiple times.\n");
    getcc(D2, (enum t_getfmt)(GETCC_TYPE), L2);

    if(levtest(D2))                /* if the DataLevel is occupied.. */
    {
      for(int i=0; i < MAX_SIZE; i = i+1)
      {
        detac(D2);
        if(!levtest(D2))
        {
          attac(D2);
        }
      }/*end for loop                                                */

      if(levtest(D2))
        prtmsg("attac: 2-Successfully Completed.\n");
      else
        prterr("attac: 2-Unable to detach and attac D1.");

      /* Release the working storage block on Data Level D2.         */
      relcc(D2);
    }
    else
      prterr("attac: 2-getcc() failed - not an attac error.");
  }/*end if                                                          */

  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
  {
    prtmsg("attac: processing attac_func error cases.\n");

    parm1[2] = ecbptr()->ebcm01;               /* set the print byte */
    parm2[2] = ecbptr()->ebcm01;               /* set the print byte */
    parm3[2] = ecbptr()->ebcm01;               /* set the print byte */
    parm4[2] = ecbptr()->ebcm01;               /* set the print byte */
    parm5[2] = ecbptr()->ebcm01;               /* set the print byte */

    /*****************************************************************/
    /* Create an independent ECB for the QMAD program.  The second   */
    /* parameter is the string to pass (which consists of the        */
    /* function to test, the case number and the prtmsg bit), and    */
    /* first parameter is the length of that string.                 */
    /*****************************************************************/
    cremc(3, parm1, QMAD);
    sleep(10);
    cremc(3, parm2, QMAD);
    sleep(10);
    cremc(3, parm3, QMAD);
    sleep(10);
    cremc(3, parm4, QMAD);
    sleep(10);
    cremc(3, parm5, QMAD);
    sleep(10);
  }/*end else                                                        */

  prtmsg("attac: completed attac_func.\n");
  return;
}/*end of attac_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    flipc_func()                                    */
/*                                                                   */
/* Number of Cases:  3 regular exit test cases                       */
/*                   0 system exit test cases                        */
/*                                                                   */
/*********************************************************************/
void flipc_func(void)
{
  int MAX_SIZE = 99;     // defined for looping test (must be an odd #)
  int count = 0;                   // defined for looping test.
  unsigned int address = NULL;    //defined for 4byte file address.

  struct DL_struct {
    void *CBRW;                    // 4bytes stores CBRW info.  //gcc
    unsigned int FARW;             // 4bytes stores FARW info.
    unsigned char DEB;             // 1byte stores DEB info.
    unsigned char FARW_ext[8];     // 8bytes stores FARW ext info.
  };

  struct saved_DL_struct {
    unsigned char DEB;             // 1byte stores DEB info.
    unsigned char FARW_ext[8];     // 8bytes stores FARW ext info.
  };

  /*****************************************************************/
  /* Holds original and flipped values for each DataLevel.         */
  /*****************************************************************/
  struct DL_struct D1_o, D1_f, D2_o, D2_f, D3_o,
                   D3_f, D4_o, D4_f, D5_o, D5_f;
  /*****************************************************************/
  /* Stores DEB and FARW_ext info for each DataLevel, so the info  */
  /* on the DataLevels is not changed.  The values are set back at */
  /* the end of each test to the stored values.                    */
  /*****************************************************************/
  struct saved_DL_struct D1_s, D2_s, D3_s, D4_s, D5_s;

  prtmsg("flipc: entering flipc_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    /*****************************************************************/
    /* Test Case 1:       Normal Completion Test.                    */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("flipc: 1-Flipping DataLevels D1 and D2.\n");

    /*****************************************************************/
    /* Setup of DataLevel D1.  If the DataLevel exists, release it   */
    /* and get a file address and a working storage block..          */
    /*****************************************************************/
    if(levtest(D1))
      relcc(D1);

    /*****************************************************************/
    /* Get the FARW and CBRW.                                        */
    /*****************************************************************/
    getcc(D1, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L2, 0xC1);

    address = getfc(D1,GETFC_TYPE1,"OM",GETFC_NOBLOCK,GETFC_NOSERRC);

    /*****************************************************************/
    /* getfc returns 0 if it has an error obtaining an address.      */
    /*****************************************************************/
    if(address == 0)
    {
      prterr("flipc: getfc() failed - Not a flipc error.");
      goto testcase1;
    }

    /*****************************************************************/
    /* Save values of original D1 in order to compare the flip.      */
    /*****************************************************************/
    memcpy(D1_s.FARW_ext, ecbptr()->ce1fx1, 8);
    D1_s.DEB = ecbptr()->ebcsd1;

    /*****************************************************************/
    /* After the DEB & FARW_ext have been saved off, set their values*/
    /* and save them as the original D1 DEB & FARW_ext values.       */
    /*****************************************************************/
    ecbptr()->ce1fx1[0] = (unsigned char) '1';
    ecbptr()->ce1fx1[1] = (unsigned char) '0';
    ecbptr()->ce1fx1[2] = (unsigned char) '0';
    ecbptr()->ce1fx1[3] = (unsigned char) '0';
    ecbptr()->ce1fx1[4] = (unsigned char) '0';
    ecbptr()->ce1fx1[5] = (unsigned char) '0';
    ecbptr()->ce1fx1[6] = (unsigned char) '0';
    ecbptr()->ce1fx1[7] = (unsigned char) '0';
    ecbptr()->ebcsd1 = (unsigned char) '1';

    D1_o.FARW = ecbptr()->ce1fa1;
    D1_o.CBRW = ecbptr()->ce1cr1;
    memcpy(D1_o.FARW_ext, ecbptr()->ce1fx1, 8);
    D1_o.DEB = ecbptr()->ebcsd1;

    /*****************************************************************/
    /* Setup of DataLevel D2.  If the DataLevel exists, release it   */
    /* and get a file address and a working storage block..          */
    /*****************************************************************/
    if(levtest(D2))
      relcc(D2);

    /*****************************************************************/
    /* Get the FARW and CBRW.                                        */
    /*****************************************************************/
    getcc(D2, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L2, 0xC2);

    address = NULL;
    address = getfc(D2,GETFC_TYPE1,"OM",GETFC_NOBLOCK,GETFC_NOSERRC);

    /*****************************************************************/
    /* getfc returns 0 if it has an error obtaining an address.      */
    /*****************************************************************/
    if(address == 0)
    {
      prterr("flipc: getfc() failed - Not a flipc error.");

      //release D1 CBRW
      relfc(D1);
      goto testcase1;
    }

    /*****************************************************************/
    /* Save values of original D2 in order to compare the flip.      */
    /*****************************************************************/
    memcpy(D2_s.FARW_ext, ecbptr()->ce1fx2, 8);
    D2_s.DEB = ecbptr()->ebcsd2;

    /*****************************************************************/
    /* After the DEB & FARW_ext have been saved off, set their values*/
    /* and save them as the original D1 DEB & FARW_ext values.       */
    /*****************************************************************/
    ecbptr()->ce1fx2[0] = (unsigned char) '0';
    ecbptr()->ce1fx2[1] = (unsigned char) '1';
    ecbptr()->ce1fx2[2] = (unsigned char) '0';
    ecbptr()->ce1fx2[3] = (unsigned char) '0';
    ecbptr()->ce1fx2[4] = (unsigned char) '0';
    ecbptr()->ce1fx2[5] = (unsigned char) '0';
    ecbptr()->ce1fx2[6] = (unsigned char) '0';
    ecbptr()->ce1fx2[7] = (unsigned char) '0';
    ecbptr()->ebcsd2 = (unsigned char) '0';

    D2_o.FARW = ecbptr()->ce1fa2;
    D2_o.CBRW = ecbptr()->ce1cr2;
    memcpy(D2_o.FARW_ext, ecbptr()->ce1fx2, 8);
    D2_o.DEB = ecbptr()->ebcsd2;

    /*****************************************************************/
    /* Flip the contents of D1 and D2                                */
    /*****************************************************************/
    flipc(D1, D2);

    /*****************************************************************/
    /* Save values of flipped D1 & D2 in order to compare the flip.  */
    /*****************************************************************/
    D1_f.FARW = ecbptr()->ce1fa1;
    D1_f.CBRW = ecbptr()->ce1cr1;
    memcpy(D1_f.FARW_ext, ecbptr()->ce1fx1, 8);
    D1_f.DEB = ecbptr()->ebcsd1;

    D2_f.FARW = ecbptr()->ce1fa2;
    D2_f.CBRW = ecbptr()->ce1cr2;
    memcpy(D2_f.FARW_ext, ecbptr()->ce1fx2, 8);
    D2_f.DEB = ecbptr()->ebcsd2;

    /*****************************************************************/
    /* Compare the flipped DataLevels to the original DataLevels.    */
    /*****************************************************************/
    if((memcmp(&D1_o, &D2_f, 9) == 0) &&
       (memcmp(&D2_o, &D1_f, 9) == 0) &&
       (memcmp(&D1_o.FARW_ext, &D2_f.FARW_ext, 8) == 0) &&
       (memcmp(&D2_o.FARW_ext, &D1_f.FARW_ext, 8) == 0))
      prtmsg("flipc: 1-Successfully Completed.\n");
    else
      prterr("flipc: 1-Not all fields in D1 & D2 have been flipped.");

    /*****************************************************************/
    /* Return FARW_ext[8] and DEB back to their original saved values*/
    /*****************************************************************/
    memcpy(ecbptr()->ce1fx1, D1_s.FARW_ext, 8);
    ecbptr()->ebcsd1 = D1_s.DEB;
    memcpy(ecbptr()->ce1fx2, D2_s.FARW_ext, 8);
    ecbptr()->ebcsd2 = D2_s.DEB;

    /*****************************************************************/
    /* Release the D1 & D2 Working Storage Blocks & File Pool Storage*/
    /*****************************************************************/
    relfc(D1);
    relfc(D2);

testcase1:
    if(levtest(D1))
      relcc(D1);
    if(levtest(D2))
      relcc(D2);
    /*****************************************************************/
    /* Test Case 2:       Specifying the same data level for both    */
    /*                    parameters results in no action being      */
    /*                    taken.                                     */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("flipc: 2-Flipping DataLevel D3 with itself.\n");

    /*****************************************************************/
    /* Setup of DataLevel D3.  If the DataLevel exists, release it   */
    /* and get a file address and a working storage block..          */
    /*****************************************************************/
    if(levtest(D3))
      relcc(D3);

    /*****************************************************************/
    /* Get the FARW and CBRW.                                        */
    /*****************************************************************/
    getcc(D3, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L2, 0xC3);

    address = NULL;
    address = getfc(D3,GETFC_TYPE1,"OM",GETFC_NOBLOCK,GETFC_NOSERRC);

    /*****************************************************************/
    /* getfc returns 0 if it has an error obtaining an address.      */
    /*****************************************************************/
    if(address == 0)
    {
      prterr("flipc: getfc() failed - Not a flipc error.");
      goto testcase2;
    }

    /*****************************************************************/
    /* Save values of original D3 in order to compare the flip.      */
    /*****************************************************************/
    memcpy(D3_s.FARW_ext, ecbptr()->ce1fx3, 8);
    D3_s.DEB = ecbptr()->ebcsd3;

    /*****************************************************************/
    /* After the DEB & FARW_ext have been saved off, set their values*/
    /* and save them as the original D1 DEB & FARW_ext values.       */
    /*****************************************************************/
    ecbptr()->ce1fx3[0] = (unsigned char) '0';
    ecbptr()->ce1fx3[1] = (unsigned char) '0';
    ecbptr()->ce1fx3[2] = (unsigned char) '1';
    ecbptr()->ce1fx3[3] = (unsigned char) '0';
    ecbptr()->ce1fx3[4] = (unsigned char) '0';
    ecbptr()->ce1fx3[5] = (unsigned char) '0';
    ecbptr()->ce1fx3[6] = (unsigned char) '0';
    ecbptr()->ce1fx3[7] = (unsigned char) '0';
    ecbptr()->ebcsd3 = (unsigned char) '1';

    D3_o.FARW = ecbptr()->ce1fa3;
    D3_o.CBRW = ecbptr()->ce1cr3;
    memcpy(D3_o.FARW_ext, ecbptr()->ce1fx3, 8);
    D3_o.DEB = ecbptr()->ebcsd3;

    /*****************************************************************/
    /* Flip the contents of D3 and D3                                */
    /*****************************************************************/
    flipc(D3, D3);

    /*****************************************************************/
    /* Save values of flipped D3 in order to compare the flip.       */
    /*****************************************************************/
    D3_f.FARW = ecbptr()->ce1fa3;
    D3_f.CBRW = ecbptr()->ce1cr3;
    memcpy(D3_f.FARW_ext, ecbptr()->ce1fx3, 8);
    D3_f.DEB = ecbptr()->ebcsd3;

    /*****************************************************************/
    /* Compare the flipped DataLevel to the original DataLevel.      */
    /*****************************************************************/
    if((memcmp(&D3_o, &D3_f, 9) == 0) &&
       (memcmp(&D3_o.FARW_ext, &D3_f.FARW_ext, 8) == 0))
      prtmsg("flipc: 2-Successfully Completed.\n");
    else
      prterr("flipc: 2-Not all fields in D3 & D3 have been flipped.");

    /*****************************************************************/
    /* Return FARW_ext[8] and DEB back to their original saved values*/
    /*****************************************************************/
    memcpy(ecbptr()->ce1fx3, D3_s.FARW_ext, 8);
    ecbptr()->ebcsd3 = D3_s.DEB;

    /*****************************************************************/
    /* Release the D3 Working Storage Block and File Pool Storage.   */
    /*****************************************************************/
    relfc(D3);

testcase2:
    if(levtest(D3))
      relcc(D3);
    /*****************************************************************/
    /* Test Case 3:       Muliple interchanges of the contents of    */
    /*                    Data Levels.                               */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("flipc: 3-Flipping DataLevels D4 and D5.\n");

    /*****************************************************************/
    /* Setup of DataLevel D4.  If the DataLevel exists, release it   */
    /* and get a file address and a working storage block..          */
    /*****************************************************************/
    if(levtest(D4))
      relcc(D4);

    /*****************************************************************/
    /* Get the FARW and CBRW.                                        */
    /*****************************************************************/
    getcc(D4, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L2, 0xC4);

    address = NULL;
    address = getfc(D4,GETFC_TYPE1,"OM",GETFC_NOBLOCK,GETFC_NOSERRC);

    /*****************************************************************/
    /* getfc returns 0 if it has an error obtaining an address.      */
    /*****************************************************************/
    if(address == 0)
    {
      prterr("flipc: getfc() failed - Not a flipc error.");
      goto done;
    }

    /*****************************************************************/
    /* Save values of original D4 in order to compare the flip.      */
    /*****************************************************************/
    memcpy(D4_s.FARW_ext, ecbptr()->ce1fx4, 8);
    D4_s.DEB = ecbptr()->ebcsd4;

    /*****************************************************************/
    /* After the DEB & FARW_ext have been saved off, set their values*/
    /* and save them as the original D1 DEB & FARW_ext values.       */
    /*****************************************************************/
    ecbptr()->ce1fx4[0] = (unsigned char) '0';
    ecbptr()->ce1fx4[1] = (unsigned char) '0';
    ecbptr()->ce1fx4[2] = (unsigned char) '0';
    ecbptr()->ce1fx4[3] = (unsigned char) '1';
    ecbptr()->ce1fx4[4] = (unsigned char) '0';
    ecbptr()->ce1fx4[5] = (unsigned char) '0';
    ecbptr()->ce1fx4[6] = (unsigned char) '0';
    ecbptr()->ce1fx4[7] = (unsigned char) '0';
    ecbptr()->ebcsd4 = (unsigned char) '1';

    D4_o.FARW = ecbptr()->ce1fa4;
    D4_o.CBRW = ecbptr()->ce1cr4;
    memcpy(D4_o.FARW_ext, ecbptr()->ce1fx4, 8);
    D4_o.DEB = ecbptr()->ebcsd4;

    /*****************************************************************/
    /* Setup of DataLevel D5.  If the DataLevel exists, release it   */
    /* and get a file address and a working storage block..          */
    /*****************************************************************/
    if(levtest(D5))
      relcc(D5);

    /*****************************************************************/
    /* Get the FARW and CBRW.                                        */
    /*****************************************************************/
    getcc(D5, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L2, 0xC5);

    address = NULL;
    address = getfc(D5,GETFC_TYPE1,"OM",GETFC_NOBLOCK,GETFC_NOSERRC);

    /*****************************************************************/
    /* getfc returns 0 if it has an error obtaining an address.      */
    /*****************************************************************/
    if(address == 0)
    {
      prterr("flipc: getfc() failed - Not a flipc error.");

      //release D1 CBRW
      relfc(D5);
      goto done;
    }

    /*****************************************************************/
    /* Save values of original D5 in order to compare the flip.      */
    /*****************************************************************/
    memcpy(D5_s.FARW_ext, ecbptr()->ce1fx5, 8);
    D5_s.DEB = ecbptr()->ebcsd5;

    /*****************************************************************/
    /* After the DEB & FARW_ext have been saved off, set their values*/
    /* and save them as the original D1 DEB & FARW_ext values.       */
    /*****************************************************************/
    ecbptr()->ce1fx5[0] = (unsigned char) '0';
    ecbptr()->ce1fx5[1] = (unsigned char) '0';
    ecbptr()->ce1fx5[2] = (unsigned char) '0';
    ecbptr()->ce1fx5[3] = (unsigned char) '0';
    ecbptr()->ce1fx5[4] = (unsigned char) '1';
    ecbptr()->ce1fx5[5] = (unsigned char) '0';
    ecbptr()->ce1fx5[6] = (unsigned char) '0';
    ecbptr()->ce1fx5[7] = (unsigned char) '0';
    ecbptr()->ebcsd5 = (unsigned char) '0';

    D5_o.FARW = ecbptr()->ce1fa5;
    D5_o.CBRW = ecbptr()->ce1cr5;
    memcpy(D5_o.FARW_ext, ecbptr()->ce1fx5, 8);
    D5_o.DEB = ecbptr()->ebcsd5;

    /*****************************************************************/
    /* Do multiple flips, but end up doing an odd # so you flip the  */
    /* contents of D4 and D5.                                        */
    /*****************************************************************/
    for(count=0; count < MAX_SIZE; count = count+1)
    {
      flipc(D4, D5);
      flipc(D5, D4);
      flipc(D4, D4);
      flipc(D5, D5);
      flipc(D5, D4);
    }

    /*****************************************************************/
    /* Save values of flipped D4 & D5 in order to compare the flip.  */
    /*****************************************************************/
    D4_f.FARW = ecbptr()->ce1fa4;
    D4_f.CBRW = ecbptr()->ce1cr4;
    memcpy(D4_f.FARW_ext, ecbptr()->ce1fx4, 8);
    D4_f.DEB = ecbptr()->ebcsd4;

    D5_f.FARW = ecbptr()->ce1fa5;
    D5_f.CBRW = ecbptr()->ce1cr5;
    memcpy(D5_f.FARW_ext, ecbptr()->ce1fx5, 8);
    D5_f.DEB = ecbptr()->ebcsd5;

    /*****************************************************************/
    /* Compare the flipped DataLevels to the original DataLevels.    */
    /*****************************************************************/
    if((memcmp(&D4_o, &D5_f, 9) == 0) &&
       (memcmp(&D5_o, &D4_f, 9) == 0) &&
       (memcmp(&D4_o.FARW_ext, &D5_f.FARW_ext, 8) == 0) &&
       (memcmp(&D5_o.FARW_ext, &D4_f.FARW_ext, 8) == 0))
      prtmsg("flipc: 3-Successfully Completed.\n");
    else
      prterr("flipc: 3-Not all fields in D4 & D5 have been flipped.");

    /*****************************************************************/
    /* Return FARW_ext[8] and DEB back to their original saved values*/
    /*****************************************************************/
    memcpy(ecbptr()->ce1fx4, D4_s.FARW_ext, 8);
    ecbptr()->ebcsd4 = D4_s.DEB;

    memcpy(ecbptr()->ce1fx5, D5_s.FARW_ext, 8);
    ecbptr()->ebcsd5 = D5_s.DEB;

    /*****************************************************************/
    /* Release the D4 & D5 Working Storage Blocks & File Pool Storage*/
    /*****************************************************************/
    relfc(D4);
    relfc(D5);

done:
    if(levtest(D4))
      relcc(D4);
    if(levtest(D5))
      relcc(D5);
  }/*end if                                                          */

  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
    prtmsg("flipc: no flipc_func error cases to process.\n");

  prtmsg("flipc: completed flipc_func.\n");
  return;
}/*end of flipc_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    crosc_func()                                    */
/*                                                                   */
/* Number of Cases:  3 regular exit test cases                       */
/*                   4 system exit test cases                        */
/*                                                                   */
/*********************************************************************/
void crosc_func(void)
{
  struct TPF_regs regs;
  long   answer,
         INT_ARG = 6,           /* INT_ARG is passed into QMAF & QMAG*/
         CORRECT_ANS = 12,      /* CORRECT_ANS must be twice INT_ARG */
         INCORRECT_ANS = 1;     /* helps to display specific err msg */

  char*  CHAR_ARG = "test";     /* CHAR_ARG is passed into QMAG      */
  unsigned short    save_dbi;   /* variable saves ECB dbi setting    */
  unsigned short    save_ssu;   /* variable saves ECB ssu setting    */
  unsigned short    new_dbi;
  struct   uatbc_area uatbc_parm;
  struct   ms0ut    *ssu_slot_ptr;
  unsigned int     ss_ord = 0;/* SS oridnal number                  */

  char parm1[3] = { '1', '1' , '0' }, parm2[3] = { '1', '2' , '0' };

  prtmsg("crosc: entering crosc_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    /*****************************************************************/
    /* Save original DBI and SSU.  Even though the DBI and SSU are   */
    /* saved, you will still need to go through cebic_save() so the  */
    /* reference to globals will be correct.                        */
    /*****************************************************************/
    save_dbi = ecbptr()->ce1dbi;
    save_ssu = ecbptr()->ce1ssu;

    uatbc_parm.uat_call = UAT_RSSI;
    uatbc_parm.uat_parm[0] = (char)ss_ord;

    if (uatbc(&uatbc_parm) == UAT_SUCCESS)
    {
      ss_ord++;                          /* go to the next subsystem */
      uatbc_parm.uat_parm[0] = (char)ss_ord;

      /***************************************************************/
      /* If there is more than one subsystem running, alert the user */
      /* that crosc_entrc() will cross subsystems to enter the       */
      /* program... otherwise, crosc_entrc() will simply enter the   */
      /* program in the current subsystem.                           */
      /***************************************************************/
      if(uatbc(&uatbc_parm) == UAT_SUCCESS)
      {
        prtmsg("crosc: There are at least 2 subsystems running.\n");
        prtmsg("crosc: crosc_entrc() will enter program in a different subsystem.\n");
      }
      else
      {
        prtmsg("crosc: There is only 1 subsystem running.\n");
        prtmsg("crosc: crosc_entrc() will enter program in the current subsystem\n");
      }

      /* pointer to subsystem user slot                              */
      ssu_slot_ptr = (ms0ut*) uatbc_parm.uat_ssu_ptr;
      new_dbi = *(unsigned short *) ssu_slot_ptr->mu0dpi;

      /***************************************************************/
      /* Test Case 1:       Cross subsystem to enter the QMAE program*/
      /*                    in a different subsystem with no         */
      /*                    arguments, and then return.              */
      /* Test Result:       Normal exit with no errors.              */
      /* Error Msg Output:  None                                     */
      /***************************************************************/
      if(uatbc(&uatbc_parm) == UAT_SUCCESS)
        prtmsg("crosc: 1-Enter/Return from QMAE program in different SS (with no arguments).\n");
      else
        prtmsg("crosc: 1-Enter/Return from QMAE program in current SS (with no arguments).\n");

      answer = 0;                             /* Set 'answer' to 0   */

      /***************************************************************/
      /* crosc to QMAE program results in returning the number 12    */
      /***************************************************************/
      cebic_save();
      cebic_goto_dbi(new_dbi);
      answer = crosc_entrc("QMAE", CROSC_DBI, NULL);
      cebic_restore();

      if(answer == CORRECT_ANS)
        prtmsg("crosc: 1-Successfully Completed.\n");
      else
        prterr("crosc: 1-Unable to Enter/Return from QMAE.");


      /***************************************************************/
      /* Test Case 2:       Cross subsystem to enter the QMAF program*/
      /*                    in a different subsystem with one        */
      /*                    argument and then return.                */
      /* Test Result:       Normal exit with no errors.              */
      /* Error Msg Output:  None                                     */
      /***************************************************************/
      if(uatbc(&uatbc_parm) == UAT_SUCCESS)
        prtmsg("crosc: 2-Enter/Return from QMAF program in different SS (with one argument).\n");
      else
        prtmsg("crosc: 2-Enter/Return from QMAF program in current SS (with one argument).\n");

      answer = 0;                             /* Reset 'answer' to 0 */
      regs.r1 = INT_ARG;

      /***************************************************************/
      /* crosc to QMAF program results in multiplying r1 by 2        */
      /***************************************************************/
      cebic_save();
      cebic_goto_dbi(new_dbi);
      answer = crosc_entrc("QMAF", CROSC_DBI, &regs);
      cebic_restore();

      if(answer == CORRECT_ANS)
        prtmsg("crosc: 2-Successfully Completed.\n");
      else
        prterr("crosc: 2-Unable to Enter/Return from QMAF.");


      /***************************************************************/
      /* Test Case 3:       Cross subsystem to enter the QMAG program*/
      /*                    in a different subsystem with arguments  */
      /*                    of different types, and then return.     */
      /* Test Result:       Normal exit with no errors.              */
      /* Error Msg Output:  None                                     */
      /***************************************************************/
      if(uatbc(&uatbc_parm) == UAT_SUCCESS)
        prtmsg("crosc: 3-Enter/Return from QMAG program in different SS (with different args).\n");
      else
        prtmsg("crosc: 3-Enter/Return from QMAG program in current SS (with different args).\n");

      answer = 0;                             /* Reset 'answer' to 0 */
      regs.r1 = INT_ARG;
      regs.r2 = (long) CHAR_ARG;

      /***************************************************************/
      /* crosc to QMAG program results in multiplying r1 by 2 if the */
      /* string (CHAR_ARG) is passed successfully to QMAG ...        */
      /* otherwise the number 1 is returned.                         */
      /***************************************************************/
      cebic_save();
      cebic_goto_dbi(new_dbi);
      answer = crosc_entrc("QMAG", CROSC_DBI, &regs);
      cebic_restore();

      if(answer == CORRECT_ANS)
        prtmsg("crosc: 3-Successfully Completed.\n");
      else if(answer == INCORRECT_ANS)
        prterr("crosc: 3-Unable to pass parameters correctly.");
      else
        prterr("crosc: 3-Unable to Enter/Return from QMAG.");
    }
    else
      prterr("crosc: There are no subsystems active.");

    /*****************************************************************/
    /* Restore original DBI and SSU                                  */
    /*****************************************************************/
    ecbptr()->ce1dbi = save_dbi;
    ecbptr()->ce1ssu = save_ssu;
  }/*end if                                                          */

  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
  {
    prtmsg("crosc: processing crosc_func error cases.\n");

    parm1[2] = ecbptr()->ebcm01;               /* set the print byte */
    parm2[2] = ecbptr()->ebcm01;               /* set the print byte */

    /*****************************************************************/
    /* Create an independent ECB for the QMAD program.  The second   */
    /* parameter is the string to pass (which consists of the        */
    /* function to test, the case number and the prtmsg bit), and    */
    /* first parameter is the length of that string.                 */
    /*****************************************************************/
    cremc(3, parm1, QMAD);
    sleep(10);
    cremc(3, parm2, QMAD);
    sleep(10);
  }/*end else                                                        */

  prtmsg("crosc: completed crosc_func.\n");
  return;
}/*end of crosc_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    caloc_func()                                    */
/*                                                                   */
/* Number of Cases:  1 regular exit test case                        */
/*                                                                   */
/*********************************************************************/
void caloc_func(void)
{
  int  NUM = 10;               /* Number of elements to be allocated */
  int  * rc = NULL;            /* Initialize the return code to NULL */
  char buffer[80];             /* will contain the address returned  */

  prtmsg("caloc: entering caloc_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    /*****************************************************************/
    /* Test Case 1:       Attempt to reserve and initialize heap     */
    /*                    storage in 64BIT mode.                     */
    /*                                                               */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("caloc: 1-Attempting to reserve and initialize heap storage using tpf_calloc64()\n");

    /*****************************************************************/
    /* z/TPF UPDATE - caloc_func - Update this function for z/TPF    */
    /* tpf_calloc64() function not supported yet...                  */
    /* replace line below with the following line once supported:    */
    /*     rc = (int *) tpf_calloc64(NUM, sizeof(int) );             */
    /*****************************************************************/
    rc = (int *) calloc(NUM , sizeof(int));

    if(rc != NULL)
    {
      /***************************************************************/
      /* The storage of a calloc() is initialized to "0".  Check to  */
      /* make sure that the storage was initialized and obtained     */
      /* correctly.                                                  */
      /***************************************************************/
      if(*rc == 0)
      {
        if(TWO_GIG_BAR >= (long)&rc)
          sprintf(buffer, "caloc: 1-Address below 2-Gig Bar.  Address returned: %p.\n", &rc);
        else
          sprintf(buffer, "caloc: 1-Address above 2-Gig Bar.  Address returned: %p.\n", &rc);

        prtmsg(buffer);
      }
      else
        prterr("caloc: 1-calloc() initialization unsuccessful.");
    }
    else
      prterr("caloc: 1-Insufficient Memory available.");

    free(rc); /* deallocates the space - does nothing if rc is NULL. */
  }/*end if                                                          */
  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
    prtmsg("caloc: no caloc_func error cases to process.\n");

  prtmsg("caloc: completed caloc_func.\n");
  return;
}/*end of caloc_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    maloc_func()                                    */
/*                                                                   */
/* Number of Cases:  1 regular exit test case                        */
/*                                                                   */
/* Note:             The storage of a malloc() is uninitialized, so  */
/*                   there is no need to check the contents of the   */
/*                   storage area.                                   */
/*                                                                   */
/*********************************************************************/
void maloc_func(void)
{
  int  * rc = NULL;            /* Initialize the return code to NULL */
  char buffer[80];             /* will contain the address returned  */

  prtmsg("maloc: entering maloc_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    /*****************************************************************/
    /* Test Case 1:       Attempt to reserve heap storage in 64BIT   */
    /*                    mode.                                      */
    /*                                                               */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("maloc: 1-Attempting to reserve heap storage using tpf_malloc64()\n");

    /*****************************************************************/
    /* z/TPF UPDATE - maloc_func - Update this function for z/TPF    */
    /* tpf_malloc64() function not supported yet...                  */
    /* replace line below with the following line once supported:    */
    /*     rc = (int *) tpf_malloc64( sizeof(int) );                 */
    /*****************************************************************/
    rc = (int *) malloc(sizeof(int));

    if(rc != NULL)
    {
      if(TWO_GIG_BAR >= (long)&rc)
        sprintf(buffer, "maloc: 1-Address below 2-Gig Bar.  Address returned: %p.\n", &rc);
      else
        sprintf(buffer, "maloc: 1-Address above 2-Gig Bar.  Address returned: %p.\n", &rc);

      prtmsg(buffer);
    }
    else
      prterr("maloc: 1-Insufficient Memory available.");

    free(rc); /* deallocates the space - does nothing if rc is NULL. */
  }/*end if                                                          */
  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
    prtmsg("maloc: no maloc_func error cases to process.\n");

  prtmsg("maloc: completed maloc_func.\n");
  return;
}/*end of maloc_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    raloc_func()                                    */
/*                                                                   */
/* Number of Cases:  1 regular exit test case                        */
/*                                                                   */
/* Note:             The storage of a malloc() and realloc() are     */
/*                   uninitialized, so there is no need to check the */
/*                   contents of the storage area.                   */
/*                                                                   */
/*********************************************************************/
void raloc_func(void)
{
  int  NUM1 = 10, NUM2 = 20;   /* Number of elements to be allocated */
  int  * rc = NULL;            /* Initialize the return code to NULL */
  char buffer[80];             /* will contain the address returned  */

  prtmsg("raloc: entering raloc_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    /*****************************************************************/
    /* Test Case 1:       Attempt to change reserved storage block   */
    /*                    size in 64BIT mode.                        */
    /*                                                               */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("raloc: 1-Attempting to change reserved storage block size using tpf_realloc64()\n");

    /*****************************************************************/
    /* z/TPF UPDATE - maloc_func - Update this function for z/TPF    */
    /* tpf_malloc64() function not supported yet...                  */
    /* replace line below with the following line once supported:    */
    /*     rc = (int *) tpf_malloc64( NUM1 * sizeof(int) );          */
    /*****************************************************************/
    /* Initialize the size of the storage */
    rc = (int *) malloc(NUM1 * sizeof(int));

    if(rc != NULL)
    {
      /***************************************************************/
      /* z/TPF UPDATE - raloc_func - Update this function for z/TPF  */
      /* tpf_realloc64() function not supported yet...               */
      /* replace line below with the following line once supported:  */
      /*   rc = (int *) tpf_realloc64( rc, NUM2 * sizeof( int )) );  */
      /***************************************************************/
      /* Change the size of the storage */
      rc = (int *) realloc(rc, NUM2 * sizeof(int));

      if(rc != NULL)
      {
        if(TWO_GIG_BAR >= (long)&rc)
          sprintf(buffer, "raloc: 1-Address below 2-Gig Bar.  Address returned: %p.\n", &rc);
        else
          sprintf(buffer, "raloc: 1-Address above 2-Gig Bar.  Address returned: %p.\n", &rc);

        prtmsg(buffer);
      }
      else                                        /*  realloc error  */
        prterr("raloc: 1-Insufficient Memory available.");
    }/*end else                                                      */
    else                                          /*  malloc error   */
      prterr("raloc: 1-call to tpf_malloc64() failed.");

    free(rc); /* deallocates the space - does nothing if rc is NULL. */
  }/*end if                                                          */
  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
    prtmsg("raloc: no raloc_func error cases to process.\n");

  prtmsg("raloc: completed raloc_func.\n");
  return;
}/*end of raloc_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    freec_func()                                    */
/*                                                                   */
/* Number of Cases:  1 error test case                               */
/*                                                                   */
/*********************************************************************/
void freec_func(void)
{
  char parm1[3] = { '2', '1' , '0' };

  prtmsg("freec: entering freec_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    /* freec() regular cases run with calloc(), malloc() & realloc().*/
    prtmsg("freec: no freec_func regular cases to process.\n");
  }
  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
  {
    /*****************************************************************/
    /* Test Case 1:       Attempt to release the storage blocks of   */
    /*                    an area that is not in the ECB heap.       */
    /*                                                               */
    /* Test Result:       Ctl-74F dump                               */
    /* Error Msg Output:  FREEC - BLOCK ADDRESS NOT IN HEAP AREA     */
    /*****************************************************************/
    /* Error Description for FREEC macro:                            */
    /* The general register specified by BLOCK must contain the      */
    /* address of a block of storage returned from the last MALOC,   */
    /* CALOC, or RALOC operation against it. Otherwise, this function*/
    /* may conclude that there is corruption in the heap, causing    */
    /* the immediate exit of the ECB, unless the address is zero.    */
    /*****************************************************************/
    prtmsg("freec: processing freec_func error cases.\n");

    parm1[2] = ecbptr()->ebcm01;               /* set the print byte */

    /*****************************************************************/
    /* Create an independent ECB for the QMAD program.  The second   */
    /* parameter is the string to pass (which consists of the        */
    /* function to test, the case number and the prtmsg bit), and    */
    /* first parameter is the length of that string.                 */
    /*****************************************************************/
    cremc(3, parm1, QMAD);
    sleep(10);
  }

  prtmsg("freec: completed freec_func.\n");
  return;
}/*end of freec_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    gsysc_func()                                    */
/*                                                                   */
/* Number of Cases:  1 regular exit test case                        */
/*                                                                   */
/*********************************************************************/
void gsysc_func(void)
{
  int  frames = 3;            /* 3 frames to be allocated (4KB each) */
  int  rc = NULL;             /* Initialize the return code to NULL  */
  char * token = "GSYSCTST";  /* identifies storage being allocated  */
  char buffer[80];            /* will contain the address returned   */
  long temp_var = 0;           /* holds address of tbl_ptr            */

  struct table
  {
    char  *name;
    int   code;
  } *tbl_ptr;

  prtmsg("gsysc: entering gsysc_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    /*****************************************************************/
    /* Test Case 1:       Attempt to get 31-bit system heap storage. */
    /*                                                               */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("gsysc: 1-Attempting to get 31-bit storage from the system heap.\n");

    /*****************************************************************/
    /* This was the previous call to the 31-bit gsysc():             */
    /* tbl_ptr = (table*) gsysc(frames, token);                      */
    /*****************************************************************/
    tbl_ptr = (table*) tpf_gsysc(frames, token, NULL, NULL);

    if (tbl_ptr == 0)
      prterr("gsysc: 1-Error allocating table.");
    else
    {
      prtmsg("gsysc: 1-Storage obtained from the system heap.\n");

      temp_var = (long) tbl_ptr;

      if(TWO_GIG_BAR >= temp_var)
        sprintf(buffer, "gsysc: 1-Address below 2-Gig Bar.  Address returned: %p.\n", tbl_ptr);
      else
        sprintf(buffer, "gsysc: 1-Address above 2-Gig Bar.  Address returned: %p.\n", tbl_ptr);

        prtmsg(buffer);
    }

    /*****************************************************************/
    /* This was the previous call to the 31-bit rsysc():             */
    /* rc = rsysc((void *)tbl_ptr, frames, token);                   */
    /*****************************************************************/
    rc = tpf_rsysc((void *)tbl_ptr, frames, token);

    if (rc == RSYSC_ERROR)
      prterr("gsysc: 1-Error releasing storage - tpf_rsysc() error.");
    else
      prtmsg("gsysc: 1-Storage released from the system heap.\n");
  }/*end if                                                          */
  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
    prtmsg("gsysc: no gsysc_func error cases to process.\n");

  prtmsg("gsysc: completed gsysc_func.\n");
  return;
}/*end of gsysc_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    rsysc_func()                                    */
/*                                                                   */
/* Number of Cases:  1 regular exit error test case                  */
/*                                                                   */
/*********************************************************************/
void rsysc_func(void)
{
//  int frames = 4;          /* 4 frames to be allocated (4KB each)    */
  int rc = NULL;           /* Initialize the return code to NULL     */
  prtmsg("rsysc: entering rsysc_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    /* regular rsysc_func test case handled in gsysc_func test case. */
    prtmsg("rsysc: no rsysc_func regular cases to process.\n");
  }/*end if                                                          */
  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
  {
    /*****************************************************************/
    /* Test Case 1:       Attempt to release system heap storage when*/
    /*                    passed an incorrect token.                 */
    /*                                                               */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("rsysc: 1-Attempting to release storage from the system heap.\n");

    /*****************************************************************/
    /* The last parameter does not match a gsysc()'s token parameter */
    /* and so it is unable to release the heap storage.              */
    /*****************************************************************/
    rc = rsysc(NULL, 0, "XXXXXXXX");
    if (rc == RSYSC_ERROR)
      prtmsg("rsysc: 1-Error releasing storage - (expected results).\n");
    else
      prterr("rsysc: 1-Storage released from the system heap.");
  }

  prtmsg("rsysc: completed rsysc_func.\n");
  return;
}/*end of rsysc_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    rehka_func()                                    */
/*                                                                   */
/* Number of Cases:  1 regular exit test case                        */
/*                                                                   */
/*********************************************************************/
void rehka_func(void)
{
  char     save_area[]= "GARBAGE ";
  char buffer[80];                     /* used to print out messages */
  prtmsg("rehka: entering rehka_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    /*****************************************************************/
    /* Test Case 1:       Attempt to rehook core block.              */
    /*                                                               */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("rehka: 1-Attempting to rehook core block.\n");

    /*****************************************************************/
    /* If DataLevel D4 is occupied, release the core block.          */
    /*****************************************************************/
    if(levtest(D4))
      relcc(D4);

    /*****************************************************************/
    /* Get a common block on DataLevel D4. (L2: 1055 bytes)          */
    /*****************************************************************/
    getcc(D4, (enum t_getfmt) (GETCC_TYPE), L2);

    if(levtest(D4))                 /* if DataLevel D4 is occupied.. */
    {
      prtmsg("rehka: 1-Data level D4 now has a core block.\n");

      sprintf((char*)ecbptr()->ce1cr4, "rehka: 1-8-byte field before unhka() of D4 cbrw : %s\n", save_area);
      prtmsg((const char*)ecbptr()->ce1cr4);
      sprintf((char*)ecbptr()->ce1cr4, "rehka: 1-Address of cbrw on data level D4: %i\n", (int)ecbptr()->ce1cr4);
      prtmsg((const char*)ecbptr()->ce1cr4);
    }
    else
      prterr("rehka: 1-Data level D4 was unable to get a core block.\n");

    /*****************************************************************/
    /* Next, unhook the common block into an 8-byte field.           */
    /*****************************************************************/
    unhka(D4, UNHKA_UNPROTECTED, save_area);
    prtmsg("rehka: 1-8-byte field now contains the info from the D4 cbrw.\n");

    if(levtest(D4))                 /* if DataLevel D4 is occupied.. */
      prterr("rehka: 1-Data level D4 should have had its core block unhooked.\n");
    else
      prtmsg("rehka: 1-Data level D4 is now unhooked and does not contain a core block.\n");

    /*****************************************************************/
    /* If DataLevel D5 is occupied, release the core block.  Then    */
    /* print out a message that states whether D5 is occupied or not.*/
    /*****************************************************************/
    if(levtest(D5))
      relcc(D5);

    if(levtest(D5))                 /* if DataLevel D5 is occupied.. */
      prterr("rehka: 1-Data level D5 should not have a core block.\n");
    else
    {
      prtmsg("rehka: 1-Data level D5 does not have a core block.\n");

      sprintf(buffer, "rehka: 1-Address of cbrw on data level D5 before rehka(): %i\n", (int)ecbptr()->ce1cr5);
      prtmsg(buffer);
    }

    /*****************************************************************/
    /* Rehook Common Block that was previously unhooked from data    */
    /* level D4 to data level D5.                                    */
    /*****************************************************************/
    rehka(D5, REHKA_UNPROTECTED, save_area);

    if(levtest(D5))                 /* if DataLevel D5 is occupied.. */
    {
      prtmsg("rehka: 1-Data level D5 is now rehooked to the D4 core block.\n");

      sprintf((char*)ecbptr()->ce1cr5, "rehka: 1-Address of cbrw on data level D5 after rehka(): %i\n", (int)ecbptr()->ce1cr5);
      prtmsg((const char*)ecbptr()->ce1cr5);
    }
    else
      prterr("rehka: 1-Data level D5 was unable to be rehooked.\n");

    /*****************************************************************/
    /* Rehook transfers the information from save_area (an 8-byte    */
    /* field) to a specified cbrw in the ECB, and zeroes the 8-byte  */
    /* field.  Below, we are checking the original address given to  */
    /* data level D4 for the common block with the address given to  */
    /* data level D5 after the rehook.                               */
    /*****************************************************************/
    if(memcmp(ecbptr()->ce1cr4, ecbptr()->ce1cr5, 8) == 0)
      prtmsg("rehka: 1-Common Block successfully rehooked to data level D5.\n");
    else
      prterr("rehka: 1-Address of cbrw in D4 and D5 do not match.");

    if(strcmp(save_area, "") == 0 )
    {
      prtmsg("rehka: 1-8byte field successfully zeroed out.\n");
      prtmsg("rehka: 1-Successfully Completed.\n");
    }
    else
      prterr("rehka: 1-8byte field has not been zeroed out.");

    /*****************************************************************/
    /* Release the working storage block. relcc(D4) gives you a      */
    /* CTL-0000I7 dump because an unhka() was done on D4, so I have  */
    /* used crusa(1, D4).                                            */
    /*****************************************************************/
    crusa(1, D4);
    relcc(D5);
  }/*end if                                                          */
  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
    prtmsg("rehka: no rehka_func error cases to process.\n");

  prtmsg("rehka: completed rehka_func.\n");
  return;
}/*end of rehka_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    unhka_func()                                    */
/*                                                                   */
/* Number of Cases:  1 error test case                               */
/*                                                                   */
/*********************************************************************/
void unhka_func(void)
{
  char parm1[3] = { '5', '1' , '0' };
  prtmsg("unhka: entering unhka_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    /* unhka() regular case run with rehka().                        */
    prtmsg("unhka: no unhka_func regular cases to process.\n");
  }/*end if                                                          */
  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
  {
    prtmsg("unhka: processing unhka_func error cases.\n");
    parm1[2] = ecbptr()->ebcm01;               /* set the print byte */

    /*****************************************************************/
    /* Create an independent ECB for the QMAD program.  The second   */
    /* parameter is the string to pass (which consists of the        */
    /* function to test, the case number and the prtmsg bit), and    */
    /* the first parameter is the length of that string.             */
    /*****************************************************************/
    cremc(3, parm1, QMAD);
    sleep(10);
  }/*end else                                                        */

  prtmsg("unhka: completed unhka_func.\n");
  return;
}/*end of unhka_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    fiwhc_func()                                    */
/*                                                                   */
/* Number of Cases:  1 regular exit test case                        */
/*                                                                   */
/*********************************************************************/
void fiwhc_func(void)
{
  const char *  recid = REC_ID;        /* REC_ID = "OM"              */
  char          buffer[80];            /* used to print out messages */

  prtmsg("fiwhc: entering fiwhc_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    /*****************************************************************/
    /* Test Case 1:       Find and hold a file record and wait with  */
    /*                    extended options.                          */
    /* Test Result:       Normal exit with no errors.                */
    /* Error Msg Output:  None                                       */
    /*****************************************************************/
    prtmsg("fiwhc: 1-Attempting to find and hold a file record.\n");

    /*****************************************************************/
    /* If DataLevel D2 is occupied, release the core block.          */
    /*****************************************************************/
    if(levtest(D2))
      relcc(D2);

    /*****************************************************************/
    /* Put text in a record and write it out to file.  CBRW MUST     */
    /* start off with the recordID as the first 2 bytes.             */
    /*****************************************************************/
    getfc(D2,GETFC_TYPE1,recid,GETFC_BLOCK+GETFC_FILL,GETFC_NOSERRC, 0xF0);

    sprintf((char*)ecbptr()->ce1cr2, recid);
    sprintf((char*)ecbptr()->ce1cr2+25, "VERIFY");
    sprintf(buffer, "fiwhc: 1-Text going into record to be filed: %s\n", (const char*)ecbptr()->ce1cr2+25);
    prtmsg(buffer);

    ecbptr()->ebcrc2 = 0;                       /* record code check */
    memcpy(&(ecbptr()->ebcid2),recid,2);        /* record id         */

    if(levtest(D2))
    {
      sprintf(buffer, "fiwhc: 1-D2 has obtained a Working Storage Block at: %x\n", (int)ecbptr()->ce1cr2);
      prtmsg(buffer);
    }
    else
      prterr("fiwhc: 1-D2 has erroneously not obtained a Working Storage Block.");

    /*****************************************************************/
    /* If DataLevel D3 is occupied, release the core block.          */
    /*****************************************************************/
    if(levtest(D3))
      relcc(D3);

    /*****************************************************************/
    /* Copy D2 CBRW into D3 CBRW, so that once we have performed the */
    /* fiwhc_ext() on D2, we can compare the results to the original */
    /* values.                                                       */
    /*****************************************************************/
    getfc(D3,GETFC_TYPE1,recid,GETFC_BLOCK+GETFC_FILL,GETFC_NOSERRC, 0xF0);
    memcpy(ecbptr()->ce1cr3, ecbptr()->ce1cr2, 50);

    /*****************************************************************/
    /* Write the data in the working storage block on D2 to file     */
    /*****************************************************************/
    filnc(D2);
    if (waitc())
       prterr("fiwhc: Error return from waitc()");

    /*****************************************************************/
    /* Assure that a Working Storage Block has been kept on D2.      */
    /*****************************************************************/
    if(levtest(D2))                 /* if DataLevel D2 is occupied.. */
      prtmsg("fiwhc: 1-D2 has kept its Working Storage Block.\n");
    else
      prterr("fiwhc: 1-D2 has erroneously released its Working Storage Block.");

    /*****************************************************************/
    /* Clear out the area where the text was held, so we can be      */
    /* assured that the fiwhc_ext() actually placed the "VERIFY"     */
    /* text in the CBRW and releases the block.                      */
    /*****************************************************************/
    sprintf((char*)ecbptr()->ce1cr2+25, "xxxxxx");
    relcc(D2);

    /*****************************************************************/
    /* Find/Wait/Hold the record onto D2                             */
    /*   - FIND_DEFEXT accesses the record on an online database     */
    /*****************************************************************/
    fiwhc_ext(D2,FIND_DEFEXT);

    if(levtest(D2))
    {
      sprintf(buffer, "fiwhc: 1-D2 has obtained a Working Storage Block at: %x by fiwhc_ext()\n", (int)ecbptr()->ce1cr2);
      prtmsg(buffer);
    }
    else
      prterr("fiwhc: 1-D2 has erroneously not obtained a Working Storage Block.");

    /*****************************************************************/
    /* Check the text in the record with the original text 'VERIFY'  */
    /* that was stored in D3.                                        */
    /*****************************************************************/
    if(strcmp((const char*)ecbptr()->ce1cr2+25,(const char*)ecbptr()->ce1cr3+25) == 0)
    {
      sprintf(buffer, "fiwhc: 1-The text in the record matches the original text: %s.\n", (const char*)ecbptr()->ce1cr2+25);
      prtmsg(buffer);
    }
    else
      prterr("fiwhc: 1-The text in the record and the original text do not match.");

    /*****************************************************************/
    /* The defined Record ID and the Record ID on the data level     */
    /* should match up.                                              */
    /*****************************************************************/
    if(strcmp((const char*)&ecbptr()->ebcid2, recid) == 0)
    {
      sprintf(buffer, "fiwhc: 1-The Record ID has been verified. The Record ID is: %s\n", recid);
      prtmsg(buffer);
    }
    else
      prterr("fiwhc: 1-The Record ID of the held record does not match Record ID filed out to D2.");

    /*****************************************************************/
    /* Check the error bits                                          */
    /*****************************************************************/
    if (ecbptr()->ce1sud[2] != 0)
      prterr("fiwhc: 1- An error has occured during fiwhc_ext().\n");
    else
      prtmsg("fiwhc: 1-Successfully Completed.\n");
    /*****************************************************************/
    /* rcunc(D2) - Release core block and unhold file record. In     */
    /* effect, this does a relfc(D2), and unfrc_ext(D2, FIND_DEFEXT) */
    /*****************************************************************/
    rcunc(D2);
    relfc(D3);
  }/*end if                                                          */
  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
    prtmsg("fiwhc: no fiwhc_func error cases to process.\n");

  prtmsg("fiwhc: completed fiwhc_func.\n");
  return;
}/*end of fiwhc_func()                                               */
}/*end of extern "C"                                                 */
