/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2005                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20041001  Initial Version                                         */
/* 20070509  jkharms  d14760 change routecd variable size            */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/* Load module name: QMAC                                            */
/* Segment Name:     qmac20.cpp                                      */
/* Abstract:         This driver tests the use of the following      */
/*                   functions:                                      */
/*                   cratc   -   cratc_func()                        */
/*                   uatbc   -   uatbc_func()                        */
/*                   wgtac   -   wgtac_func()                        */
/*                   lodic   -   lodic_func()                        */
/*                   tmslc   -   tmslc_func()                        */
/*                   crexc   -   crexc_func()                        */
/*                   corhc   -   corhc_func()                        */
/*                   movec   -   movec_func()                        */
/*********************************************************************/
/*********************************************************************/
/* Function Name:    cratc_func                                      */
/* Description: function test driver for CRATC.                      */
/* Information: There is no header file for CR0AT. If the DSECT of   */
/*              CR0AT(32 way) changes, the offsets used in this      */
/*              driver may need to be changed too.                   */
/*                                                                   */
/*              If you used index in type CRATC_LOCATE, you are      */
/*              searching the specified alternate CRAS slot(Axx).    */
/*              If you used index in type CRATC_FSC, you are         */
/*              searching the index of CRAS slot address of the      */
/*              functional support consoles. In these two cases,     */
/*              cpuid is of no use. CRATC doesn't use or check the   */
/*              cpuid field.                                         */
/*                                                                   */
/*              Cratc generates system error only for invalid type.  */
/*                                                                   */
/*              Cratc_func sets up the valid cpuids for testing.     */
/*              Cratc_func sets up the variables numOfCrasEntries    */
/*              and numOfAlterEntries according to the Cras Table.   */
/*              If there is no entries for Alternate Cras slot, the  */
/*              following two entries will be added before testing   */
/*              and will be removed before cratc_func returns.       */
/*                                                                   */
/*              A01  520000    B    PRT-3284                         */
/*              A02  4E0000    B    CRT-3277                         */
/*                                                                   */
/*              ZDCRS-Display CRAS Status                            */
/*              Use this command to display information from the     */
/*              CRAS status table.                                   */
/*********************************************************************/
#define _POSIX_SOURCE
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <tpf/tpfio.h>
#include <tpf/tpfapi.h>
#include <tpf/sysapi.h>
#include <tpf/tpfeq.h>
#include <tpf/c_pi1dt.h>
#include <tpf/c_ms0ut.h>
#include <tpf/c_ms0at.h>
#include <tpf/c_uatbc.h>
#include <tpf/c_mk0ck.h>
#include <tpf/c_eb0eb.h>
#include <tpf/tpfglbl.h>
#include <tpf/c_globz.h>
#include <c_api5.h>

extern "C" void cratc_func() {

   struct cratc_slot   *crat_slot_ptr;
   struct cratc_iparms  cratc_input;

   /*********************************************************/
   /*  MAXCRAS: Max allowable Cras entries,                 */
   /*  MAXALT:  Max allowable Alternate Cras entries.       */
   /*  OFFSET_1ST_CRAS: Offset of the first Cras slot       */
   /*  OFFSET_1ST_ALTER: Offset of the first Alternate slot */
   /*********************************************************/

   const int MAXCRAS = 64,
             MAXALT = 99,
             OFFSET_1ST_CRAS = 516,
             OFFSET_1ST_ALTER = 1156,
             SIZE_ENTRY = 10,
             NUM_ENTRY_ADD = 2;

   boolean needRestore = FALSE;
   
   unsigned char lniata[3] = {01,00,00},
                 zero[NUM_ENTRY_ADD * SIZE_ENTRY] = {0};
   
   short int i;
   unsigned int rc;
   unsigned int routecd = 0;
   unsigned int * route_ptr = &routecd;
   //points to the least significant three bytes assuming 8 bytes for an int
   route_ptr += 5;  
   
   char index,
        numOfCrasEntries = 0,
        numOfAlterEntries = 0,
        cpuid[32] = { 0 },
        invalid_cpuid = '?',   

        /**************************************************************/
        /*  Initializes table with two valid entries for possible use */
        /**************************************************************/

        *table = "\x52\x00\x00\xD0\xA5\xC2\x00\x00\x00\xC2"\
                 "\x4E\x00\x00\xC0\x15\xC2\x00\x00\x00\xC2";

   void *cras_ptr = cinfc_fast(CINFC_CMCRAS);
   void *temp_ptr = (char*) cras_ptr + OFFSET_1ST_CRAS;
   struct pi1dt* pid = (struct pi1dt *) cinfc_fast(CINFC_CMMPID);

   /*********************************************/
   /*  Collects the valid cpuids for test cases */
   /*********************************************/

   short int numOfProcessors = pid->pi1pids.pi1nocp;
   for ( i = 0; i < numOfProcessors; i++)
       cpuid[i] = pid->pi1lnn[i].pi1spt;

   /****************************************************/
   /*  Uses 6 valid cpuids in the following test cases */
   /*  In the case of Base only system or less cpuids  */
   /*  genned. Needs to change some cpuids to cpuid[0] */
   /****************************************************/

   for ( i = 1; i < 6; i++)
      if (!cpuid[i])
         cpuid[i] = cpuid[0];

   /******************************************************/
   /*  Counts the number of entries for Processor Slots  */
   /******************************************************/

   do
   {
      numOfCrasEntries++;
      temp_ptr = (char*) temp_ptr + SIZE_ENTRY;
   }while (memcmp(temp_ptr, zero, SIZE_ENTRY) && numOfCrasEntries <= MAXCRAS );

   /*************************************************************/
   /*  Check whether there are entries for Alternate Slots      */
   /*  If there are, counts the number of entries for Alternate */
   /*  Slots. If there are not, add two entries for test cases  */
   /*************************************************************/

   void *alter_ptr = temp_ptr = (char*) cras_ptr + OFFSET_1ST_ALTER;
   if (!memcmp(temp_ptr, zero, SIZE_ENTRY) )
   {
      corhc(cras_ptr);
      cinfc(CINFC_WRITE,CINFC_CMCRAS);
      memcpy(temp_ptr, table, NUM_ENTRY_ADD * SIZE_ENTRY);
      keyrc_okey();
      numOfAlterEntries = NUM_ENTRY_ADD;
      needRestore = TRUE;
   }
   else
   {
      do
      {
         numOfAlterEntries++;
         temp_ptr = (char*) temp_ptr + SIZE_ENTRY;
      }while (memcmp(temp_ptr, zero, SIZE_ENTRY) && numOfAlterEntries <= MAXALT);
   }

   if (ecbptr()->ebx000 == FALSE)
   {

   /***************************************/
   /* Testing NORMAL case of CRATC macro  */
   /***************************************/

      prtmsg("Entering CRATC_func - Testing NORMAL case.\n");

      /*************************************************************/
      /* type CRATC_VERIFY- VALID Lniata 010000, NULL cpuid        */
      /*               testing whether default using CE1CPD works? */
      /*************************************************************/

      memset(&cratc_input,0x00,sizeof(struct cratc_iparms));
      memcpy(cratc_input.cr_in.cratc_i_lniata, lniata, sizeof(lniata));
      prtmsg("cratc: 1- Tests CRATC_VERIFY using VALID lniata 010000, NULL cpuid\n");
      rc = cratc(CRATC_VERIFY, &cratc_input, &crat_slot_ptr);
      if (rc)
      {
         prtmsg("cratc: 1- request FOUND in the CRAS table\n");
         if (memcmp(&(ecbptr()->ce1cpd), &(crat_slot_ptr->cratc_cpuid),sizeof(char)))
            prterr("cratc: 1- NOT using cpuid in CE1CPD");
      }
      else
         prterr("cratc: 1-  request NOT found in the CRAS table");

      /***********************************************/
      /* CRATC_VERIFY- VALID Lniata 010000 and cpuid */
      /***********************************************/

      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[1], sizeof(char));
      prtmsg("cratc: 2- Tests CRATC_VERIFY using VALID lniata and cpuid\n");
      rc = cratc(CRATC_VERIFY, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 2- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 2- request NOT found in the CRAS table");

      /*********************************************************/
      /* CRATC_LOCATE- Routing code: WTOPC_EBROUT, VALID cpuid */
      /*********************************************************/

      *route_ptr = WTOPC_EBROUT;
      memset(&cratc_input,0x00,sizeof(struct cratc_iparms));
      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[3],sizeof(char));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0], route_ptr, 3);
      prtmsg("cratc: 3- Tests CRATC_LOCATE using WTOPC_EBROUT, VALID cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 3- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 3- request NOT found in the CRAS table\n");

      /********************************************************/
      /* CRATC_LOCATE- Routing code: WTOPC_UNSOL, VALID cpuid */
      /********************************************************/

      *route_ptr = WTOPC_UNSOL;
      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[5],sizeof(cpuid[5]));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0], route_ptr, 3);
      prtmsg("cratc: 4- Tests CRATC_LOCATE using WTOPC_UNSOL, VALID cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 4- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 4- request NOT found in the CRAS table\n");

      /**********************************************************/
      /* CRATC_LOCATE- Routing code: WTOPC_RO, NULL cpuid       */
      /*               test whether default using CE1CPD works? */
      /**********************************************************/

      *route_ptr = WTOPC_RO;
      memset(&cratc_input,0x00,sizeof(struct cratc_iparms));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0], route_ptr, 3);
      prtmsg("cratc: 5- Tests CRATC_LOCATE using WTOPC_RO, cpuid in CE1CPD\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
      {
         prtmsg("cratc: 5- request FOUND in the CRAS table\n");
         if (memcmp(&(ecbptr()->ce1cpd), &(crat_slot_ptr->cratc_cpuid), sizeof(cpuid[0])))
            prterr("cratc: 5- NOT using cpuid in CE1CPD");
      }
      else
         prterr("cratc: 5- request NOT found in the CRAS table");

      /*****************************************************/
      /* CRATC_LOCATE- Routing code: WTOPC_RO, VALID cpuid */
      /*****************************************************/

      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[0],sizeof(cpuid[0]));
      prtmsg("cratc: 6- Tests CRATC_LOCATE using WTOPC_RO, VALID cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 6- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 6- request NOT found in the CRAS table");

      /******************************************************/
      /* CRATC_LOCATE- Routing code: WTOPC_PRC, VALID cpuid */
      /******************************************************/

      *route_ptr = WTOPC_PRC;
      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[2],sizeof(cpuid[2]));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0], route_ptr, 3);
      prtmsg("cratc: 7- Tests CRATC_LOCATE using WTOPC_PRC, VALID cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 7- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 7- request NOT found in the CRAS table");

      /*******************************************************/
      /* CRATC_LOCATE- Routing code: WTOPC_TAPE, VALID cpuid */
      /*******************************************************/

      *route_ptr = WTOPC_TAPE;
      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[4],sizeof(cpuid[4]));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0], route_ptr, 3);
      prtmsg("cratc: 8- Tests CRATC_LOCATE using WTOPC_TAPE, VALID cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 8- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 8- request NOT found in the CRAS table");

      /*******************************************************/
      /* CRATC_LOCATE- Routing code: WTOPC_DASD, VALID cpuid */
      /*******************************************************/

      *route_ptr = WTOPC_DASD;
      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[1],sizeof(cpuid[1]));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0], route_ptr, 3);
      prtmsg("cratc: 9- Tests CRATC_LOCATE using WTOPC_DASD, VALID cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 9- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 9- request NOT found in the CRAS table");

      /*******************************************************/
      /* CRATC_LOCATE- Routing code: WTOPC_COMM, VALID cpuid */
      /*******************************************************/

      *route_ptr = WTOPC_COMM;
      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[3],sizeof(cpuid[3]));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0], route_ptr, 3);
      prtmsg("cratc: 10- Tests CRATC_LOCATE using WTOPC_COMM, VALID cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 10- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 10- request NOT found in the CRAS table");

      /*******************************************************/
      /* CRATC_LOCATE- Routing code: WTOPC_AUDT, VALID cpuid */
      /*******************************************************/

      *route_ptr = WTOPC_AUDT;
      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[5],sizeof(cpuid[5]));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0], route_ptr, 3);
      prtmsg("cratc: 11- Tests CRATC_LOCATE using WTOPC_AUDT, VALID cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 11- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 11- request NOT found in the CRAS table");

      /*******************************************************/
      /* CRATC_LOCATE- Routing code: WTOPC_RDBS, VALID cpuid */
      /*******************************************************/

      *route_ptr = WTOPC_RDBS;
      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[4],sizeof(cpuid[4]));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0], route_ptr, 3);
      prtmsg("cratc: 12- Tests CRATC_LOCATE using WTOPC_RDBS, VALID cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 12- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 12- request NOT found in the CRAS table");

      /********************************************************/
      /* CRATC_LOCATE- VALID CRAS table index 1, NULL cpuid   */
      /********************************************************/

      index = 1;
      memset(&cratc_input,0x00,sizeof(struct cratc_iparms));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0],&index,1);
      prtmsg("cratc: 13- Tests CRATC_LOCATE using VALID CRAS table index 1, NULL cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 13- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 13- request NOT found in the CRAS table");

      /*********************************************************/
      /* CRATC_LOCATE- VALID CRAS table index 2, INvalid cpuid */
      /*********************************************************/

      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0],&numOfAlterEntries,1);
      memcpy(&cratc_input.cratc_i_cpuid, &invalid_cpuid,sizeof(invalid_cpuid));
      prtmsg("cratc: 14- Tests CRATC_LOCATE using VALID CRAS table index,INvalid cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 14- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 14- request NOT found in the CRAS table\n");

      /*******************************************************/
      /* CRATC_FSC- VALID CRAS table index 1,  NULL cpuid    */
      /*            test whether default using CE1CPD works? */
      /*******************************************************/

      index = 1;
      memset(&cratc_input,0x00,sizeof(struct cratc_iparms));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0],&index,1);
      prtmsg("cratc: 15- Tests CRATE_FSC using VALID CRAS table index 1,NULL cpuid\n");
      rc = cratc(CRATC_FSC, &cratc_input, &crat_slot_ptr);
      if (rc)
         prtmsg("cratc: 15- request FOUND in the CRAS table\n");
      else
         prterr("cratc: 15- request NOT found in the CRAS table");

      /*******************************************************************/
      /* CRATC_FSC- VALID CRAS index 1 - numOfCrasEntries, Invalid cpuid */
      /*******************************************************************/

      index = 1;
      memcpy(&cratc_input.cratc_i_cpuid, &invalid_cpuid,sizeof(invalid_cpuid));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0],&index,1);
      prtmsg("cratc: 16- Tests CRATE_FSC using VALID CRAS table index, INvalid cpuid\n");
      char printout[50] = "cratc: 16- index 1 FOUND in the CRAS table\n";

      while (index <= numOfCrasEntries )
      {
         rc = cratc(CRATC_FSC, &cratc_input, &crat_slot_ptr);
         if (rc)
            prtmsg(printout);
         else
            prterr("cratc: 16- VALID CRAS table index NOT found in the CRAS table");
         index += 1;
         memcpy(&cratc_input.cr_in.cratc_i_rtcd[0],&index,1);
         sprintf(printout,"cratc: 16- index %d FOUND in the CRAS table\n", index);
      }
   }
   else
   {

   /*********************************************************/
   /* Testing ERROR case of CRATC macro                     */
   /*********************************************************/

      prtmsg("Entering CRATC_func - testing ERROR case.\n");

      /******************************************************/
      /* type CRATC_VERIFY- NULL INPUT                      */
      /******************************************************/

      memset(&cratc_input,0x00,sizeof(struct cratc_iparms));
      prtmsg("cratc: 1- Tests CRATC_VERIFY using NULL input\n");
      rc = cratc(CRATC_VERIFY, &cratc_input, &crat_slot_ptr);
      if (rc)
         prterr("Cratc: 1- NULL input FOUND in the CRAS table");
      else
         prtmsg("Cratc: 1- request NOT FOUND in the CRAS table\n");

      /****************************************************/
      /* CRATC_VERIFY- INvalid cpuid, VALID Lniata 010000 */
      /****************************************************/

      memcpy(&cratc_input.cratc_i_cpuid, &invalid_cpuid,sizeof(invalid_cpuid));
      memcpy(cratc_input.cr_in.cratc_i_lniata,lniata,sizeof(lniata));
      prtmsg("cratc: 2- Tests CRATC_VERIFY using VALID lniata 010000, INvalid cpuid\n");
      rc = cratc(CRATC_VERIFY, &cratc_input, &crat_slot_ptr);
      if (rc)
         prterr("cratc: 2- INvalid cpuid FOUND in the CRAS table");
      else
         prtmsg("cratc: 2- request NOT FOUND in the CRAS table\n");

      /****************************************************/
      /* CRATC_VERIFY- INvalid Lniata 012345, VALID cpuid */
      /****************************************************/

      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[0],sizeof(cpuid[0]));
      memcpy(cratc_input.cr_in.cratc_i_lniata +1, "\x23\x45",2);
      prtmsg("cratc: 3-  Tests CRATC_VERIFY using INvalid lniata 012345, VALID cpuid\n");
      rc = cratc(CRATC_VERIFY, &cratc_input, &crat_slot_ptr);
      if (rc)
         prterr("cratc: 3- INvalid lniata 012345 FOUND in the CRAS table");
      else
         prtmsg("cratc: 3- request NOT FOUND in the CRAS table\n");

      /*******************************************************/
      /* CRATC_LOCATE- Routing code: WTOPC_RO, INvalid cpuid */
      /*******************************************************/

      *route_ptr = WTOPC_RO;
      memset(&cratc_input,0x00,sizeof(struct cratc_iparms));
      memcpy(&cratc_input.cratc_i_cpuid, &invalid_cpuid, sizeof(invalid_cpuid));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0], route_ptr, 3);
      prtmsg("cratc: 4- Tests CRATC_LOCATE using WTOPC_RO, INvalid cpuid\n");
      rc = cratc(CRATC_LOCATE, &cratc_input, &crat_slot_ptr);
      if (rc)
         prterr("cratc: 4- INvalid cpuid FOUND in the CRAS table");
      else
         prtmsg("cratc: 4- request NOT FOUND in the CRAS table\n");

      /******************************************************/
      /* CRATC_FSC- INvalid CRAS table index 0, VALID cpuid */
      /******************************************************/

      index = 0;
      memset(&cratc_input,0x00,sizeof(struct cratc_iparms));
      memcpy(&cratc_input.cratc_i_cpuid, &cpuid[5],sizeof(cpuid[5]));
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0],&index,1);
      prtmsg("cratc: 5- Tests CRATE_FSC using CRAS table index 0, VALID cpuid\n");
      rc = cratc(CRATC_FSC, &cratc_input, &crat_slot_ptr);
      if (rc)
         prterr("cratc: 5- INvalid table index 0 FOUND in the CRAS table");
      else
         prtmsg("cratc: 5- request NOT FOUND in the CRAS table\n");

      /*******************************************************************/
      /* CRATC_FSC- INvalid CRAS index numOfCrasEntries + 1, VALID cpuid */
      /*******************************************************************/

      //Skip the two entries added if numOfCrasEntries == 64.
      if (numOfCrasEntries == 64 && needRestore)
         index = numOfCrasEntries + 2 + 1;
      else
         index = numOfCrasEntries + 1;
      memcpy(&cratc_input.cr_in.cratc_i_rtcd[0],&index,1);
      memcpy(&cratc_input.cratc_i_cpuid,&cpuid[0],sizeof(cpuid[0]));
      prtmsg("cratc: 6- Tests CRATE_FSC using INvalid table index 13, VALID cpuid\n");
      rc = cratc(CRATC_FSC, &cratc_input, &crat_slot_ptr);
      if (rc)
         prterr("cratc: 6- INvallid table index FOUND in the CRAS table");
      else
         prtmsg("cratc: 6- request NOT FOUND in the CRAS table\n");
   }

   /*********************************************************/
   /* Restores original Cras Table: deletes added entries   */
   /*********************************************************/

   if (needRestore)  {
      cinfc(CINFC_WRITE,CINFC_CMCRAS);
      memcpy(alter_ptr, zero, numOfAlterEntries * SIZE_ENTRY);
      keyrc_okey();
      coruc(cras_ptr);
   }

   /*********************************************************/
   /* CRATC testing completed                               */
   /*********************************************************/

   prtmsg("Completes cratc_func successfully.\n");
   return;
}


/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/* Load module name: QMAC                                            */
/* Segment Name:     qmac20.cpp                                      */
/* Function Name:    uatbc_func.cpp                                  */
/* Abstract:                                                         */
/* Description: function test driver for UATBC                       */
/*                                                                   */
/* Information: Error return:                                        */
/*              Return code 8 (UAT_EXCD)                             */
/*              The SS or SSU ordinal number supplied exceeds the    */
/*              number of SSs/SSUs included in the last IPL.         */
/*                                                                   */
/*              Return code 16 (UAT_INVLID)                          */
/*              The SS or SSU name supplied cannot be found in       */
/*              subsystem attribute table or the subsystem user      */
/*              table, respectively.                                 */
/*                                                                   */
/*              Return code 32 (UAT_NOTAVL)                          */
/*              1).The specified SS is indicated as inactive in CTKM.*/
/*              2).The parent SS of the specified SSU is indicated   */
/*              as inactive in CTKM, or, the parent SS of the        */
/*              specified SSU is active but the specified SSU is     */
/*              indicated as dormant in MS0UT.                       */
/*                                                                   */
/*********************************************************************/
extern "C" void uatbc_func() {

   struct uatbc_area uatbc_parm;
   char printout[50];
   int rc,
       numOfEntry,
       numOfSubsystem;
   struct ms0ut *ssu_slot_ptr = (struct ms0ut*) cinfc_fast(CINFC_CMMSSUT);
   struct ms0at * ssat = (struct ms0at*) cinfc_fast(CINFC_CMMSSAT);
   const int SIZE_ENTRY = sizeof(ms0ut) - sizeof(ssu_slot_ptr->mu0cnt);

   numOfEntry = ssu_slot_ptr->mu0cnt;
   numOfSubsystem = ssat->ma0cnt;
   char SSName[256] = { 0 };

   /************************************************/
   /*  Collects all subsystem names for latter use */
   /************************************************/

   for ( int i = 0; i < numOfSubsystem; i++)
      memcpy(SSName + sizeof(ssat->entry[i].ma0nam) * i, ssat->entry[i].ma0nam,
             sizeof(ssat->entry[i].ma0nam));

   if (ecbptr()->ebx000 == FALSE)
   {

   /*********************************************************/
   /*  Testing NORMAL case of UATBC macro                   */
   /*********************************************************/

      prtmsg("Entering UATBC_func - Testing NORMAL case.\n");

      /******************************************************/
      /*  UAT_ESSI- input is the SS ID in the ECB, ce1dbi   */
      /*            UATBC should ignore uatbc_parm.uat_parm */
      /*            which contains invalid value.           */
      /******************************************************/

      prtmsg("uatbc: 1- Test using subsystem ID in ce1dbi\n");
      uatbc_parm.uat_call = UAT_ESSI;
      memcpy(uatbc_parm.uat_parm,"VOID",sizeof(uatbc_parm.uat_parm));
      if ((rc = uatbc(&uatbc_parm)) == 0)
         prtmsg("uatbc: 1- Request FOUND in the MS0UT table\n");
      else
         prterr("uatbc: 1- Request NOT found");

      /******************************************************/
      /*  UAT_ESSU- input is the SSU ID in the ECB, ce1ssu  */
      /*            UATBC should ignore uatbc_parm.uat_parm */
      /*            which contains invalid value.           */
      /******************************************************/

      prtmsg("uatbc: 2- Test using subsystem user ID in ce1ssu\n");
      uatbc_parm.uat_call = UAT_ESSU;
      memcpy(uatbc_parm.uat_parm,"VOID",sizeof(uatbc_parm.uat_parm));
      if ((rc = uatbc(&uatbc_parm)) == 0)
         prtmsg("uatbc: 2- Request FOUND in the MS0UT table\n");
      else
         prterr("uatbc: 2- Request NOT found");

      /****************************************************************/
      /*  UAT_RSSI- input is the SS ordinal number in the uatbc_area, */
      /*            The left-most byte of uat_parm.                   */
      /****************************************************************/

      prtmsg("uatbc: 3- Tests using every subsystem ordinal number\n");
      uatbc_parm.uat_call = UAT_RSSI;
      BYTE indexSS = 0x00;
      uatbc_parm.uat_parm[0] = indexSS;
      sprintf(printout,"uatbc: 3- Subsystem entry %2.2X FOUND\n", indexSS);
      while ( indexSS <= numOfSubsystem-1 )
      {
         if ((rc = uatbc(&uatbc_parm)) == 0)
            prtmsg(printout);
         else
            prterr("uatbc: 3- Valid ordinal number of subsystem entry NOT found");

         indexSS += 1;
         uatbc_parm.uat_parm[0] = indexSS;
         sprintf(printout,"uatbc: 3- Subsystem entry %2.2X FOUND\n", indexSS);
      }

      /*****************************************************************/
      /*  UAT_RSSU- input is the SSU ordinal number in the uatbc_area, */
      /*            The left-most byte of uat_parm.                    */
      /*****************************************************************/

      prtmsg("uatbc: 4- Tests using every subsystem user ordinal number\n");
      uatbc_parm.uat_call = UAT_RSSU;

      BYTE indexSSU = 0x00;
      uatbc_parm.uat_parm[0] = indexSSU;
      sprintf(printout,"uatbc: 4- Subsystem user entry %2.2X FOUND\n", indexSSU);
      while ( indexSSU <= numOfEntry-1 )
      {
         if ((rc = uatbc(&uatbc_parm)) == 0)
            prtmsg(printout);
         else
            prterr("uatbc: 4- Valid ordinal number of subsystem user entry NOT found");
         indexSSU += 1;
         uatbc_parm.uat_parm[0] = indexSSU;
         sprintf(printout,"uatbc: 4- Subsystem user entry %2.2X FOUND\n", indexSSU);
      }

      /**************************************************/
      /*  UAT_NSSI- inputs the SS name in the uat_parm  */
      /**************************************************/

      prtmsg("uatbc: 5- Tests using every subsystem name\n");
      uatbc_parm.uat_call = UAT_NSSI;
      indexSS = 0;
      memcpy(uatbc_parm.uat_parm,SSName,sizeof(uatbc_parm.uat_parm));
      sprintf(printout,"uatbc: 5- Subsystem name %4.4s FOUND\n", uatbc_parm.uat_parm);
      while (indexSS <= numOfSubsystem - 1)
      {
         if ((rc = uatbc(&uatbc_parm)) == 0)
            prtmsg(printout);
         else
            prterr("uatbc: 5- Subsystem name NOT found");
         indexSS += 1;
         memcpy(uatbc_parm.uat_parm, SSName + sizeof(uatbc_parm.uat_parm) * indexSS,
                sizeof(uatbc_parm.uat_parm));
         sprintf(printout,"uatbc: 5- Subsystem name %4.4s FOUND\n", uatbc_parm.uat_parm);
      }

      /****************************************************/
      /*  UAT_NSSU- inputs the SSU names in the uat_parm  */
      /****************************************************/

      prtmsg("uatbc: 6- Tests using every subsystem user name\n");
      uatbc_parm.uat_call = UAT_NSSU;
      indexSSU = 0x00;
      memcpy(uatbc_parm.uat_parm, ((char*) ssu_slot_ptr + sizeof(ssu_slot_ptr->mu0cnt)),
             sizeof(uatbc_parm.uat_parm));
      sprintf(printout,"uatbc: 6- Subsystem user entry %4.4s FOUND\n",uatbc_parm.uat_parm);
      while ( indexSSU <= numOfEntry-1 )
      {
         if ((rc = uatbc(&uatbc_parm)) == 0)
            prtmsg(printout);
         else
            prterr("uatbc: 6- Valid subsystem user NOT found");
         indexSSU += 1;
         memcpy(uatbc_parm.uat_parm, ((char*)ssu_slot_ptr + sizeof(ssu_slot_ptr->mu0cnt) +
                indexSSU * SIZE_ENTRY),sizeof(uatbc_parm.uat_parm));
         sprintf(printout,"uatbc: 6- Subsystem user entry %4.4s FOUND\n",
                 uatbc_parm.uat_parm);
      }

   }
   else
   {

   /*********************************************************/
   /*  Testing ERROR case of UATBC macro                    */
   /*********************************************************/

   prtmsg("Entering UATBC_func - testing ERROR case.\n");

      /****************************************/
      /*  UAT_RSSI- INvalid SS ordinal number */
      /****************************************/

      prtmsg("uatbc: 1- Test using exceed subsystem ordinal number\n");
      uatbc_parm.uat_call = UAT_RSSI;
      uatbc_parm.uat_parm[0] = numOfSubsystem;
      if ((rc = uatbc(&uatbc_parm)))
      {
         switch (rc)
         {
            case UAT_EXCD:   prtmsg("uatbc: 1- Ordinal number exceeded DETECTed\n");
                             break;
            case UAT_INVLID:
            case UAT_NOTAVL: prterr("uatbc: 1- INcorrect return code");
                             break;
         }
      }
      else
         prterr("uatbc: 1- Ordinal number excess NOT detected");

      /*********************************************/
      /*  UAT_RSSU- INvalid SSU ordinal number     */
      /*********************************************/

      prtmsg("uatbc: 2- Test using exceed subsystem user ordinal number\n");
      uatbc_parm.uat_call = UAT_RSSU;
      uatbc_parm.uat_parm[0] = numOfEntry;
      if ((rc = uatbc(&uatbc_parm)))
      {
         switch (rc)
         {
            case UAT_EXCD:   prtmsg("uatbc: 2- Ordinal number excess DETECTed\n");
                             break;
            case UAT_INVLID:
            case UAT_NOTAVL: prterr("uatbc: 2- INcorrect return code");
                             break;
         }
      }
      else
         prterr("uatbc: 2- Ordinal number excess NOT detected");

      /*************************************/
      /*  UAT_NSSI- INvalid SS name  VOID  */
      /*************************************/

      prtmsg("uatbc: 3- Test using INvalid subsystem name VOID\n");
      uatbc_parm.uat_call = UAT_NSSI;
      memcpy(uatbc_parm.uat_parm,"VOID",sizeof(uatbc_parm.uat_parm));
      if ((rc = uatbc(&uatbc_parm)))
      {
         switch (rc)
         {
            case UAT_INVLID:  prtmsg("uatbc: 3- INvalid SS name DETECTed\n");
                              break;
            case   UAT_EXCD:
            case UAT_NOTAVL:  prterr("uatbc: 3- INcorrect return code");
                              break;
         }
      }
      else
         prterr("uatbc: 3- INvalid SS name (VOID) NOT detected");

      /*************************************/
      /*  UAT_NSSU- INvalid SSU name       */
      /*************************************/

      prtmsg("uatbc: 4- Test using INvalid subsystem user name VOID\n");
      uatbc_parm.uat_call = UAT_NSSU;
      memcpy(uatbc_parm.uat_parm,"VOID",sizeof(uatbc_parm.uat_parm));
      if ((rc = uatbc(&uatbc_parm)))
      {
         switch (rc)
         {
            case UAT_INVLID:  prtmsg("uatbc: 4- INvalid SSU name DETECTed\n");
                              break;
            case   UAT_EXCD:
            case UAT_NOTAVL:  prterr("uatbc: 4- INcorrect return code");
                              break;
         }
      }
      else
         prterr("uatbc: 4- INvalid SSU name (VOID) NOT detected");


      /*************************************/
      /*  UAT_NSSI- INactive SS name       */
      /*  Have at least two subsystems up  */
      /*************************************/

      if (numOfSubsystem > 1)
      {
         unsigned int n;
         unsigned char savedStatus = ' ', status = MKINACT;

         uatbc_parm.uat_call = UAT_NSSI;

         /***********************************************************************/
         /*  Decides which subsystem to deactivate (BSS can not be deactivated) */
         /***********************************************************************/

         if ((numOfSubsystem == 2) && (ecbptr()->ce1dbi_2 != 0))  {
            prtmsg("Not in BSS - Test cases of inactive subsystem are not tested.\n");
            goto skipTest;
         }
         else if ((numOfSubsystem > 2) && (ecbptr()->ce1dbi_2 == 1))
            n = 2;
         else
            n = 1;

         /********************************/
         /* Deactivates subsystem status */
         /********************************/

         memcpy(uatbc_parm.uat_parm, ssat->entry[n].ma0nam,sizeof(uatbc_parm.uat_parm));
         sprintf(printout,"uatbc: 5- Test inactive subsystem %4.4s.\n",
                 uatbc_parm.uat_parm);
         prtmsg(printout);

         struct mk0ck * ctkm = (struct mk0ck*) cinfc(CINFC_WRITE,CINFC_CMMCTKM);
         corhc(ctkm);
         memcpy( &savedStatus, &(ctkm->mk0ntry[n].mk0sst),sizeof(savedStatus));
         memcpy( &(ctkm->mk0ntry[n].mk0sst), &status,sizeof(status));
         keyrc_okey();

         if ((rc = uatbc(&uatbc_parm)))
         {
            switch (rc)
            {
               case UAT_NOTAVL:  prtmsg("uatbc: 5- Inactive Subsystem detected\n");
                                 break;
               case   UAT_EXCD:
               case UAT_INVLID:  prterr("uatbc: 5- Error return code.");
                                 break;
            }
         }
         else
               prterr("uatbc: 5- Subsystem should not be found.");

         /*****************************************************************/
         /*  UAT_NSSU- INactive SSU name                                  */
         /*  Gets Subsystem User name from ms0ut table                    */
         /*                                                               */
         /*  ssu_slot_ptr->mu0cnt           # of items in ms0ut table     */
         /*  struct ms0at {                 the structure of the SSAT     */
         /*    unsigned long   ma0cnt;      Number of defined subsystems  */
         /*    struct ms0ams   entry [64];  Up to 64 subsystem entries    */
         /*  };                                                           */
         /*                                                               */
         /*  struct ms0ams  {              the structure of an SSAT entry */
         /*    char           ma0nam [4];  Subsystem name                 */
         /*    unsigned char   ma0num;     Number of SSUs in this SS      */
         /*    unsigned char   ma0ind;     Index to 1st SSU in SSUT       */
         /*  };                                                           */
         /*                                                               */
         /*  If there are more than two subsystem and the SS id is 0x01   */
         /*  uses the last SSU name else uses the first SSU name in the   */
         /*  ms0ut Subsystem User Table                                   */
         /*                                                               */
         /*****************************************************************/

         uatbc_parm.uat_call = UAT_NSSU;
         memcpy(uatbc_parm.uat_parm, ((char*)ssu_slot_ptr + sizeof(ssu_slot_ptr->mu0cnt)+
                ssat->entry[n].ma0ind * SIZE_ENTRY),sizeof(uatbc_parm.uat_parm));
         sprintf(printout,"uatbc: 6- Test INactive subsystem user %4.4s.\n",
                 uatbc_parm.uat_parm);
         prtmsg(printout);

         if ((rc = uatbc(&uatbc_parm)))
         {
            switch (rc)
            {
               case UAT_NOTAVL:  prtmsg("uatbc: 6- Inactive user detected.\n");
                                 break;
               case   UAT_EXCD:
               case UAT_INVLID:  prterr("uatbc: 6- Error return code.");
                                 break;
            }
         }
         else
            prterr("uatbc: 6- Subsystem user should not be found.");

         /***********************************/
         /* restores original status value  */
         /***********************************/

         cinfc(CINFC_WRITE,CINFC_CMMCTKM);
         memcpy( &(ctkm->mk0ntry[n].mk0sst),&savedStatus,sizeof(savedStatus));
         keyrc_okey();
         coruc(ctkm);
      }
      else
         prtmsg("Only one subsystem's up, test cases of inactive subsystem are NOT tested!\n");

   }
skipTest:

   /*********************************************************/
   /*  UATBC testing completed                              */
   /*********************************************************/

   prtmsg("Completes uatbc_func successfully.\n");
   return;
}


/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/* Load module name: QMAC                                            */
/* Segment Name:     qmac20.cpp                                      */
/* Function Name:    wgtac_func                                      */
/* Abstract:                                                         */
/* Description: function test driver for WGTAC.                      */
/* Information: This function returns information from the WGTA      */
/*              table concerning a terminal LNIATA (or pseudo-       */
/*              LNIATA) value.                                       */
/*   z/TPF UPDATE                                                    */
/*       MODEC  This parameter is no longer useful in setting the    */
/*              addressing mode since 24BIT addressing is not        */
/*              supported.                                           */
/*                                                                   */
/*********************************************************************/
extern "C" void wgtac_func() {

   struct wg0ta *wgta = NULL;
   int cmp1, cmp2;

   if (ecbptr()->ebx000 == FALSE)
   {

   /***************************************/
   /* Testing NORMAL case of WGTAC macro  */
   /***************************************/

      prtmsg("Entering wgtac_func - Only one NORMAL test case.\n");

      /**********************************************/
      /* Error Return for getcc() is Not applicable */
      /* Gets a free working storage area to hold a */
      /* wg0ta structure                            */
      /**********************************************/

      wgta = (struct wg0ta *) getcc(D4, GETCC_TYPE, L1);

      /**************************************/
      /* Item not found if return was zero  */
      /**************************************/

      prtmsg("wgtac: 1- Using LNIATA value & cpuid from ECB\n");
      if (!(wgta = wgtac(ecbptr()->ebrout,ecbptr()->ce1cpd,wgta)))
      {
         prterr("LNIATA not referenced in WGTA table\n");
         prterr("wgtac: 1- Request NOT found");
      }
      else

      /**********************************************/
      /* Normal processing path- self-checking code */
      /**********************************************/

      {
         cmp1 = memcmp(&wgta->wg0lit,&ecbptr()->ce1top + 1,sizeof(wgta->wg0lit));
         cmp2 = memcmp(&wgta->wg0cid,&ecbptr()->ce1cpd,sizeof(wgta->wg0cid));
         if ( cmp1 == 0 && cmp2 == 0 )
            prtmsg("wgtac: 1- Request found is correct.\n");
         else
            prterr("wgtac: 1- The entry found is INCORRECT!");
      }

      /**********************************/
      /* Releasea Working Storage Block */
      /**********************************/

      relcc(D4);
   }
   else
   {

   /*************************************/
   /* Testing ERROR case of WGTAC macro */
   /*************************************/

      prtmsg("Entering wgtac_func - testing ERROR case.\n");
      prtmsg("wgtac: 1- no test case for ERROR case\n");
   }

   /*********************************************************/
   /*  WGTAC testing completed                              */
   /*********************************************************/

   prtmsg("Completes wgtac_func successfully.\n");
   return;
}


/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/* Load module name: QMAC                                            */
/* Segment Name:     qmac20.cpp                                      */
/* Function Name:    lodic_func                                      */
/* Abstract:                                                         */
/* Description: function test driver for LODIC.                      */
/* Information: An ECB that calls the lodic function is marked as    */
/*              a low-priority ECB. Once marked, the ECB can be      */
/*              suspended when system resources are below the        */
/*              shutdown levels defined for the BATCH priority       */
/*              class.                                               */
/*                                                                   */
/*   z/TPF UPDATE  Add one more condition: SYSTEM HEAP FRAMES        */
/*                                                                   */
/*              Suspends the ECB whenever the number of available    */
/*              ECBs, SWBs, 4 KB frames, 1 MB frames, SYSTEM HEAP    */
/*              FRAMES, or CMBs falls below the shutdown level for   */
/*              the BATCH priority class.                            */
/*                                                                   */
/*              Does not allow the ECB to receive control until the  */
/*              number of available ECBs, SWBs, 4 KB frames, 1 MB    */
/*              frames, SYSTEM HEAP FRAMES, and CMBs are above that  */
/*              shutdown level                                       */
/*                                                                   */
/*              enum t_lodic_opt {LODIC_UNMARK = 0x0000,  *PJ23178*  */
/*                                LODIC_CHECK = 0x0001,              */
/*                                LODIC_ECBCREATE = 0x0002,          */
/*                                LODIC_SUSPEND = 0x0004,            */
/*                                LODIC_HOLD = 0x0010,               */
/*                                LODIC_BATCH = 0x0100,              */
/*                                LODIC_LOBATCH = 0x0200,            */
/*                                LODIC_IBMHI = 0x0300,              */
/*                                LODIC_IBMLO = 0x0400};             */
/*                                                                   */
/*      ce2susp    Suspend ECB indicator                             */
/*                                                                   */
/*      #define    ce2lodic    0x80   1 = ECB running with low       */
/*                                        priority class (LODIC)     */
/*                 ce2lhold    0x40   1 = This ECB is running with   */
/*                                        a low priority class       */
/*                                        (LODIC) and may be         */
/*                                        suspended while holding    */
/*                                        a resource                 */
/*                 ce2thold    0x20   1 = This ECB is running with   */
/*                                        time-slicing (TMSLC) and   */
/*                                        is allowed to be           */
/*                                        suspended while            */
/*                                        holding a resource         */
/*                 ce2slice    0x10   1 = This means that this ECB   */
/*                                        needs to be time-sliced    */
/*                                        as soon as possible        */
/*                 ce2prvnt    0x08   1 = Prevent time-slicing       */
/*                                                                   */
/*********************************************************************/
extern "C" void lodic_func() {

   int rc;
   unsigned char suspend_indicator_saved,
                 suspend_indicator,
                 priority_class;

   /************************************************************/
   /* Records ECB resources parameters before invoking lodic() */
   /************************************************************/

   suspend_indicator_saved = ecbp2()->ce2susp,
   priority_class = ecbp2()->ce2class;

   if (ecbptr()->ebx000 == FALSE)
   {

   /********************************************/
   /* Testing NORMAL case of LODIC macro       */
   /* Error Return for lodic is Not applicable */
   /********************************************/

      prtmsg("Entering lodic_func - Only one NORMAL test case.\n");
      prtmsg("lodic: 1- Invokes lodic()\n");
      rc = lodic_ext((enum t_lodic_opt)(LODIC_ECBCREATE+LODIC_HOLD+LODIC_IBMHI), NULL);

      /********************************************************************/
      /* Checks ce2susp- the suspend ECB indicator                        */
      /* ce2lhold    0x40   This ECB is running with a low priority class */
      /* (LODIC) and may be suspended while holding a resource            */
      /********************************************************************/

      suspend_indicator = ce2lodic | ce2lhold;

      if (memcmp(&ecbp2()->ce2susp,&suspend_indicator,sizeof(suspend_indicator)))
         prterr("lodic: 1- The suspend_indicator of ECB is not set correctly");
      else
         prtmsg("lodic: 1- The suspend_indicator of ECB is set CORRECTLY\n");

      /**********************************************/
      /* Checks if ECB priority class (0-7) altered */
      /**********************************************/

      if (!memcmp(&ecbp2()->ce2class, &priority_class, sizeof(priority_class)))
         prterr("lodic: 1- The priority class of ECB is not altered");

      /********************************************************************/
      /* If rc == 1 the available system resources are above the shutdown */
      /* levels defined for the BATCH priority class (more work is        */
      /* allowed to be started).                                          */
      /********************************************************************/

      if (rc)
         prtmsg("lodic: 1- The available system resources are ABOVE the shutdown levels\n");

      /********************************************************************/
      /* If rc == 0 the available system resources are below the shutdown */
      /* levels defined for the BATCH priority class (more work will not  */
      /* be started).                                                     */
      /********************************************************************/

      else
         prtmsg("lodic: 1- The available system resources are BELOW the shutdown levels\n");

      /******************************************************/
      /* Restores the original setting of suspend_indicator */
      /******************************************************/

      lodic_ext((enum t_lodic_opt)(LODIC_UNMARK), NULL);
      if (ecbp2()->ce2susp == suspend_indicator_saved &&
          ecbp2()->ce2class == priority_class)
         prtmsg("lodic: 1- The original value of suspend_indicator is restored.\n");
      else
         prterr("lodic: 1- The original value of suspend_indicator is NOT restored.\n");

   }
   else
   {

   /*************************************/
   /* Testing ERROR case of LODIC macro */
   /*************************************/

      prtmsg("Entering lodic_func - testing ERROR case.\n");
      prtmsg("lodic: 1- No ERROR test case for LODIC\n");
   }

   /*********************************************************/
   /*  LODIC testing completed                              */
   /*********************************************************/

   prtmsg("Completes lodic_func successfully.\n");
   return;
}


/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/* Load module name: QMAC                                            */
/* Segment Name:     qmac20.cpp                                      */
/* Function Name:    tmslc_func                                      */
/* Abstract:                                                         */
/* Description: function test driver for TMSLC.                      */
/* Information: This function is used to enable or disable time      */
/*              slicing for an ECB.                                  */
/*                                                                   */
/*  z/TPF UPDATE   No HLDE document for Asm or C TMSLC macro.        */
/*                                                                   */
/*********************************************************************/
extern "C" void tmslc_func() {

   int rc;
   unsigned char suspend_indicator_saved, suspend_indicator;

   /**********************************************************/
   /* Saves ECB resources parameters before invoking tmslc() */
   /**********************************************************/

   suspend_indicator_saved = ecbp2()->ce2susp;

   if (ecbptr()->ebx000 == FALSE)
   {

   /********************************************/
   /* Testing NORMAL case of TMSLC macro       */
   /* Error Return for tmslc is Not applicable */
   /********************************************/

      prtmsg("Entering tmslc_func - Only one NORMAL test case.\n");
      prtmsg("tmslc: 1- Invokes tmslc()\n");
      rc = tmslc(TMSLC_ENABLE + TMSLC_HOLD, "IBMHIPRI");

      /********************************************************************/
      /* Checks ce2susp- the suspend ECB indicator                        */
      /* ce2thold    0x20   1 = This ECB is running with time-slicing     */
      /* (TMSLC) and is allowed to be suspended while holding a resource  */
      /********************************************************************/

      suspend_indicator = ce2thold;

      if (memcmp(&ecbp2()->ce2susp,&suspend_indicator,sizeof(suspend_indicator)))
         prterr("tmslc: 1- The suspend ECB indicator of ECB is not set correctly");
      else
      {
         prtmsg("tmslc: 1- The suspend ECB indicator of ECB is set CORRECTLY\n");

         /********************************************/
         /* Normal Return  The return code will be 0 */
         /********************************************/

         if (rc == 0)
            prtmsg("tmslc: 1- The return code was set CORRECTLY.\n");
         else
            prterr("tmslc: 1- The return code was NOT set correctly.");
      }

      /************************/
      /* Disable time slicing */
      /************************/

      rc = tmslc(TMSLC_DISABLE, NULL);
      if (memcmp(&ecbp2()->ce2susp,&suspend_indicator_saved,sizeof(suspend_indicator_saved)))
         prterr("tmslc: 1- Time slicing is NOT disabled.");
      else
      {
         prtmsg("tmslc: 1- The ECB is disabled for time slicing successfully.\n");

         if (rc == 0)
            prtmsg("tmslc: 1- The return code is set CORRECTLY.\n");
         else
            prterr("tmslc: 1- The return code is NOT set correctly.");
      }
   }
   else
   {

   /*************************************/
   /* Testing ERROR case of TMSLC macro */
   /*************************************/

      prtmsg("Entering tmslc_func - testing ERROR case.\n");
      prtmsg("tmslc: 1- No ERROR test case for TMSLC\n");
   }

   /*********************************************************/
   /*  TMSLC testing completed                              */
   /*********************************************************/

   prtmsg("Completes tmslc_func successfully.\n");
   return;
}


/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/* Load module name: QMAC                                            */
/* Segment Name:     qmac20.cpp                                      */
/* Function Name:    crexc_func                                      */
/* Abstract:                                                         */
/* Description: function test driver for CREXC.                      */
/* Information: This macro creates a low-priority independent ECB    */
/*              for deferred processing.                             */
/*              No linkage is provided between the created ECB and   */
/*              the active ECB calling this macro.                   */
/*              Error Return   Not applicable                        */
/*                                                                   */
/*  z/TPF UPDATE   No HLDE document for Asm or C of CREXC macro.     */
/*                                                                   */
/*********************************************************************/
extern "C" void crexc_func() {

   /***********************************************************/
   /* First three bytes of the string for three control flags */
   /***********************************************************/
   char macro_name[9] = "   CREXC";

   /****************************************************/
   /* Sets the third byte for prtmsg() control in QMAD */
   /****************************************************/
   if(ecbptr()->ebcm01)
      macro_name[2] = ecbptr()->ebcm01;

   /************************************************/
   /* Second byte stands for test case no. in QMAD */
   /************************************************/
   macro_name[1] = '1';

   if (ecbptr()->ebx000 == FALSE)
   {

   /**************************************/
   /* Testing NORMAL case of CREXC macro */
   /**************************************/

      prtmsg("Entering crexc_func - Only one NORMAL test case.\n");

      /*******************************************/
      /* First byte sets to 3 for case 3 in QMAD */
      /*******************************************/

      macro_name[0] = '3';
      prtmsg("crexc: 1- Invokes crexc().\n");
      crexc(strlen(macro_name), macro_name, QMAD);

      /***************************************************/
      /* Delays processing of this ECB in order to allow */
      /* processing of other ECBs. The current ECB is    */
      /* placed on the input list                        */
      /***************************************************/

      prtmsg("crexc: 1- Delays this ECB - Invokes dlayc().\n");
      dlayc();
      prtmsg("crexc: 1- Parent ECB resumes execution.\n");

      /*******************************************/
      /* Creates an ECB for immediate processing */
      /* First byte sets to 4 for case 4 in QMAD */
      /*******************************************/

      macro_name[0] = '4';
      prtmsg("crexc: 1- Invokes cremc().\n");
      cremc(strlen(macro_name), macro_name, QMAD);

      /***************************************************/
      /* Delays processing of this ECB in order to allow */
      /* processing of other ECBs. The current ECB is    */
      /* placed on the input list                        */
      /***************************************************/

      prtmsg("crexc: 1- Delays this ECB - Invokes dlayc().\n");
      dlayc();
      prtmsg("crexc: 1- Parent ECB resumes execution.\n");

   }
   else
   {

   /*************************************/
   /* Testing ERROR case of CREXC macro */
   /*************************************/

      prtmsg("Entering crexc_func - testing ERROR case.\n");
      prtmsg("crexc: 1- No ERROR test case for CREXC.\n");
   }

   /****************************/
   /*  CREXC testing completed */
   /****************************/

   prtmsg("Completes crexc_func successfully.\n");
   return;
}

/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/* Load module name: QMAC                                            */
/* Segment Name:     qmac20.cpp                                      */
/* Function Name:    corhc_func                                      */
/* Abstract:                                                         */
/* Description: function test driver for CORHC-CORUC.                */
/* Information: The CORHC macro is used to define to the control     */
/*              program a shared resource and to control access      */
/*              to the resource among ECBs. The macro returns        */
/*              control to the calling ECB only when the shared      */
/*              resource is available for use by the ECB.            */
/*              This macro is used with the CORUC macro.             */
/*                                                                   */
/*              For example, to update a storage area                */
/*              such as a global record that is shared across        */
/*              processors in a multiprocessing environment,         */
/*              obtaining a pointer to the global and requesting     */
/*              corhc against it forces serialization on the global  */
/*              record during update.                                */
/*                                                                   */
/*              Normal Return   Void.                                */
/*              Error Return    Not applicable.                      */
/*                                                                   */
/*       SIZE   This parameter is optional. This specifies whether   */
/*              the resource identification is a 32-bit or a 64-bit  */
/*              resource identification.  The default is 32-bit.     */
/*                                                                   */
/*              A 32 bit resource ID is padded with 0's to make the  */
/*              64bit resource id.  If a CORHC-CORUC is done with    */
/*              size=32bit and the 32-bit resource id (31 bit value) */
/*              in the registers given is x'12345678'  this maps to  */
/*              the same event name as a CORHC-CORUC with size=64bit */
/*              and the 64-bit resource id set to x'0000000012345678'*/
/*                                                                   */
/*  z/TPF UPDATE   No HLDE for C yet.                                */
/*                                                                   */
/*********************************************************************/
extern "C" void corhc_func() {

   unsigned int num_of_cases = 3;
   void * q02gcvf;
   struct ev0bk eb;
   eb.evnpstinf.evnbkc1 = 1;
   strcpy (eb.evnbkn, "cremcOK");
   struct {
      unsigned int     case_No;
      char             printmsg;
      unsigned int     ECB_No;
      unsigned int     num_ECBs;
      char             macro_name[6];
      void*            addr;
   } para, * parameters;
   parameters = &para;

   /**********************************************************/
   /* Sets the prtmsg() control for passing the flag to QMAH */
   /**********************************************************/

   parameters->printmsg = ecbptr()->ebcm01;

   if (ecbptr()->ebx000 == FALSE)
   {

   /********************************************/
   /* Testing NORMAL case of CORHC-CORUC macro */
   /********************************************/

      prtmsg("Entering corhc_func - Only one NORMAL test case.\n");

      /**************************************************/
      /* Obtains a hold on global record _q02gcvf based */
      /* on the address of the record                   */
      /**************************************************/

      q02gcvf = glob(_q02gcvf);
      prtmsg("corhc: 1- Trys to hold on global record _q02gcvf, Invokes corhc().\n");
      corhc(q02gcvf);
      prtmsg("corhc: 1- Holds on global record _q02gcvf successfully.\n");

      /*********************************************************/
      /* Defines to the control program a named event that can */
      /* be waited on by this ECB and posted by other ECBs     */
      /*********************************************************/

      prtmsg("corhc: 1- Defines a named event, Invokes evntc().\n");
      int rc = evntc(&eb, EVENT_CNT, 'Y', 50, EVNTC_NORM);
      if (rc) {
         prterr("corhc: evntc() error msg - The event name already exists.\n");
         return;
      }

      /*********************************************************/
      /* Creates three ECBs to compete the same resource       */
      /* The second byte is set for the No. of ECBs            */
      /* Passing the pointer to the resource at the third byte */
      /*********************************************************/
      memcpy(&parameters->addr, &q02gcvf, sizeof(q02gcvf));
      prtmsg("corhc: 1- Creates three ECBs to compete the resource, Invokes cremc().\n");

      strcpy(parameters->macro_name, "corhc");
      for (unsigned int i = 1; i <= num_of_cases; ++i)
      {
         parameters->ECB_No = i;
         parameters->case_No = i;
         cremc(sizeof(*parameters), parameters, QMAH);
      }

      /**********************************************/
      /* Suspends this ECB until three ECBs created */
      /* Normal Return of evnwc() or evnwc is an    */
      /* integer value of zero                      */
      /**********************************************/

      prtmsg("corhc: 1- Waits for ECBs being created, Invokes evnwc().\n");
      rc = evnwc(&eb, EVENT_CNT);
      if (rc) {
         prterr("corhc: evnwc() error msg.\n");
         return;
      }

      /****************************************************/
      /* Resumes execution after three ECBs being created */
      /* Releases the resource held                       */
      /****************************************************/

      prtmsg("corhc: 1- Trys to releases global record _q02gcvf, Invokes coruc().\n");
      coruc(q02gcvf);
      prtmsg("corhc: 1- Releases global record _q02gcvf successfully.\n");

   }
   else
   {

   /*******************************************/
   /* Testing ERROR case of CORHC-CORUC macro */
   /*******************************************/

      prtmsg("Entering corhc_func - testing ERROR case.\n");
      prtmsg("corhc: 1- No ERROR test case for CORHC-CORUC\n");
   }

   /***********************************/
   /*  CORHC-CORUC testing completed  */
   /***********************************/

   prtmsg("Completes corhc_func successfully.\n");
   return;
}


/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/* Load module name: QMAC                                            */
/* Segment Name:     qmac20.cpp                                      */
/* Function Name:    movec_func                                      */
/* Abstract:                                                         */
/* Description: function test driver for tpf_movec().                */
/* Information: tpf_movec() moves data between an ECB virtual memory */
/*              (EVM) address space and a system virtual memory      */
/*              (SVM) address space. tpf_movec() permits an          */
/*              ECB-controlled program to read and modify storage    */
/*              that is not part of its own address space.           */
/*                                                                   */
/*  z/TPF UPDATE   No HLDE for C yet.                                */
/*                                                                   */
/*********************************************************************/
extern "C" void movec_func() {

   char text[] = "tpf_movec - Move Data between EVM and SVM.";
   void *sva, *from_addr, *to_addr;

   /******************************************************************/
   /* Gets two ECB virtual addresses (EVAs) for using in tpf_movec() */
   /******************************************************************/

   from_addr = malloc(sizeof(text)+1);
   to_addr = malloc(sizeof(text)+1);
   if ((! from_addr) || (! to_addr)) {
      prterr("movec: malloc() error msg - Out of storage.");
      return;
   }

   /**************************/
   /* Initializes *from_addr */
   /**************************/

   memcpy(from_addr,text,sizeof(text));

   if (ecbptr()->ebx000 == FALSE)
   {

   /**************************************/
   /* Testing NORMAL case of MOVEC macro */
   /**************************************/

      prtmsg("Entering movec_func - Only one NORMAL test case.\n");

     /****************************************************************/
     /* Gets a system virtual address (SVA) for using in tpf_movec() */
     /****************************************************************/

     prtmsg("movec: 1- Invokes gsysc().. Gets a SVA for using in tpf_movec().\n");
     sva = gsysc(1,"movec");

     if (sva)
        prtmsg("movec: 1- Gets storage from the system heap successfully.\n");
     else
     {
        prterr("movec: 1- ERROR allocating storage from the system heap.");
        return;
     }

     /*******************************/
     /* Moves data from EVA to SVA  */
     /*******************************/

      prtmsg("movec: 1- Invokes tpf-movec().. Moves data from EVA to SVA.\n");

      tpf_movec(from_addr, TPF_MOVEC_EVA,
                      sva,       TPF_MOVEC_SVA,
                sizeof(text));

     /********************************************/
     /* Moves data from SVA back to another EVA  */
     /********************************************/

      prtmsg("movec: 1- Invokes tpf_movec().. Moves data back from SVA to another EVA.\n");

      tpf_movec(sva,     TPF_MOVEC_SVA,
                to_addr, TPF_MOVEC_EVA,
                sizeof(text));

     /***************************************************************************/
     /* Checks data moved.. Compares the contents of the source and destination */
     /***************************************************************************/

      if (! memcmp(from_addr, to_addr, sizeof(text)))
          prtmsg("movec: 1- tpf_movec() moves data successfully.\n");
      else
          prterr("movec: 1- tpf_movec() moves data FAILED.");

      int rc = rsysc(sva,1,"movec");
      if (rc == RSYSC_OK)
         prtmsg("movec: 1- Releases storage from the system heap successfully.\n");
      else
      {
        prterr("movec: 1- ERROR releasing storage from the system heap.");
        return;
      }

   }
   else
   {

   /*************************************/
   /* Testing ERROR case of MOVEC macro */
   /*************************************/

      prtmsg("Entering movec_func - testing ERROR case.\n");
      prtmsg("movec: 1- No ERROR test case for MOVEC\n");
   }

   /*****************************/
   /*  MOVEC testing completed  */
   /*****************************/

   prtmsg("Completes movec_func successfully.\n");
   return;
}

extern "C" void uxrit_func()                                                         //f10789
{
   const char * uexit_yes_recid1 = "Y1",                                             //f10789
              * uexit_yes_recid2 = "YC",                                             //f10789
              * non_existent_recid = "?Z",                                           //f10789
                     * init    = "TIR",                                              //d12462
                     * checker = "RIT";                                              //f10789

   char          msg[80];            /* used to print out messages */                //f10789
   unsigned int return_code;                                                         //f10789

   if (ecbptr()->ebx000 == FALSE)                                                    //f10789
   {
      /**********************/
      /* Test record ID- Y1 */
      /**********************/
      if(levtest(D2))                                                                //f10789
         relcc(D2);                                                                  //f10789

      sprintf(msg, "QMAC/uxrit: Test record ID %s\n", uexit_yes_recid1);             //f10789
      prtmsg(msg);                                                                   //f10789
      return_code = getfc(D2,                                                        //f10789
                          GETFC_TYPE1,                                               //f10789
                          uexit_yes_recid1,                                          //f10789
                          GETFC_BLOCK|GETFC_FILL,                                    //f10789
                          GETFC_NOSERRC                                              //f10789
                          ,0xF0);                                                    //f10789

      sprintf((char*)ecbptr()->ce1cr2, uexit_yes_recid1);                            //f10789
      ecbptr()->ebcrc2 = 0;                                /* record code check */   //f10789
      memcpy(&(ecbptr()->ebcid2), uexit_yes_recid1, 2);    /* record id         */   //f10789

      /* Initialize the field for flag */
      memcpy(&(ecbptr()->ebw100), init, 3);                                          //d12462

      filnc(D2);                                                                     //f10789
      if (waitc())                                                                   //f10789
          prterr("QMAC/uxrit: Error return from waitc()");                           //f10789

      if (memcmp(&(ecbptr()->ebw100), checker, 3) == 0)                              //f10789
         sprintf(msg, "QMAC/uxrit: User Exit entered (record ID-%s)\n",              //f10789
                 uexit_yes_recid1);                                                  //f10789
      else                                                                           //f10789
         sprintf(msg, "QMAC/uxrit: User Exit NOT entered (record ID-%s)\n",          //f10789
                 uexit_yes_recid1);                                                  //f10789

      prtmsg(msg);                                                                   //f10789

      /**********************/
      /* Test record ID- YC */
      /**********************/
      if(levtest(D2))                                                                //f10789
         relcc(D2);                                                                  //f10789

      sprintf(msg, "QMAC/uxrit: Test record ID %s\n", uexit_yes_recid2);             //f10789
      prtmsg(msg);                                                                   //f10789

      return_code = getfc(D2,                                                        //f10789
                          GETFC_TYPE1,                                               //f10789
                          uexit_yes_recid2,                                          //f10789
                          GETFC_NOBLOCK,                                             //f10789
                          GETFC_NOSERRC);                                            //f10789
      if (return_code)                                                               //f10789
      {
         /* Initialize the field for flag */
         memcpy(&(ecbptr()->ebw100), init, 3);                                       //f10789

         find_record(D2, (const unsigned int *)&return_code,                         //f10789
                     uexit_yes_recid2,                                               //f10789
                     '\0',                                                           //f10789
                     NOHOLD);                                                        //f10789

         if (memcmp(&(ecbptr()->ebw100), checker, 3) == 0)                           //f10789
            sprintf(msg, "QMAC/uxrit: User Exit entered (record ID-%s)\n",           //f10789
                    uexit_yes_recid2);                                               //f10789
         else                                                                        //f10789
            sprintf(msg, "QMAC/uxrit: User Exit NOT entered (record ID-%s)\n",       //f10789
                    uexit_yes_recid2);                                               //f10789
      }
      else                                                                           //f10789
      {
         sprintf(msg, "QMAC/uxrit: getfc can NOT find record %s\n",                  //f10789
                 non_existent_recid);                                                //f10789
      }
      prtmsg(msg);

      /***********************************/
      /* Test NON existent record ID- ?Z */
      /***********************************/
      if(levtest(D2))                                                                //f10789
         relcc(D2);                                                                  //f10789

      sprintf(msg, "QMAC/uxrit: Test record ID %s\n", non_existent_recid);           //f10789
      prtmsg(msg);                                                                   //f10789

      return_code = getfc(D2,                                                        //f10789
                          GETFC_TYPE1,                                               //f10789
                          non_existent_recid,                                        //f10789
                          GETFC_NOBLOCK,                                             //f10789
                          GETFC_NOSERRC);                                            //f10789
      if (return_code)                                                               //f10789
      {
         serrc_op(SERRC_RETURN,0x12345,"QMAC-UXRIT: GETFC SHOULD FAIL BUT DID NOT",  //f10789
                  NULL);                                                             //f10789
      }
      else                                                                           //f10789
      {
         sprintf(msg, "QMAC/uxrit: getfc can NOT find record %s\n",                  //f10789
                 non_existent_recid);                                                //f10789

         prtmsg(msg);                                                                //f10789
      }

      if(levtest(D2))  {                                                             //f10789
         relcc(D2);                                                                  //f10789
         prterr("QMAC/uxrit: D2 should not have block\n");                           //f10789
      }

      /* Restore the field */
      memset(&(ecbptr()->ebw100), 0, 3);                                             //d12462

      /* Completion msg */
      prtmsg("QMAC/uxrit: Completes all test cases, now return\n");                  //f10789
   }
   else                                                                              //f10789
   {
      /* No ERROR case */
      prtmsg("QMAC/uxrit: No error test case for UCCRIT\n");                         //f10789
   }

   return;                                                                           //f10789
}
