/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2005                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20041001  Initial Version                                         */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name:  qmac40.cpp                                         */
/*                                                                   */
/* Abstract:      This driver tests the use of the following         */
/*                functions:                                         */
/*                   progc   -   progc_func()                        */
/*                   getpc   -   getpc_func()                        */
/*                                                                   */
/* Description:   The driver to run this program is qmac00.cpp.  To  */
/*                enter the flipc_func to test the flipc() function, */
/*                the user would enter:  ZTEST API-FLIPC after having*/
/*                defined a Z message:  ZFMSG add ZTEST prog-QMAC.   */
/*                The following is the grammar to this driver:       */
/*                   {API-caa+++ [(NO)ERR] [(NO)Verbose]             */
/*                            {[CNT-d++++] | [TIme-d+++]}}           */
/*********************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <tpf/tpfapi.h>
#include <tpf/c_eb0eb.h>
#include <tpf/tpfio.h>
#include <unistd.h>                                                             //C64CF
#include <tpf/sysapi.h>
#include <tpf/c_uatbc.h>
#include <tpf/c_ms0ut.h>
#include <tpf/c_idspat.h>
#include <tpf/tpfeq.h>
#include <c_api5.h>


extern "C"{
 void getpc_emsg(int);
 pat *pat_p;
 api5parms *api5parms_p;

 struct snapc_list *snapstuff[2],list[2];

/*********************************************************************/
/*********************************************************************/
/* Function Name:    progc_func()                                    */
/*                                                                   */
/* Number of Cases:  ? regular exit test cases                       */
/*                   ? system exit test cases                        */
/*                                                                   */
/*********************************************************************/
void progc_func(void)
{
  char msgBuffer[80];

  memcpy(&(api5parms_p),&(ecbptr()->ebx008),sizeof(api5parms_p));


  prtmsg("progc: entering progc_func\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    if (api5parms_p->icase==0 | api5parms_p->icase==1)
    {
      /*****************************************************************/
      /* Test Case 1:       testing PBI parm                           */
      /*                                                               */
      /* Test Result:       Normal exit with no errors.                */
      /* Error Msg Output:  None                                       */
      /*****************************************************************/
      sprintf(msgBuffer, "progc: case-1 progc(%.4s,PROGC_PBI)\n",api5parms_p->pname);
      prtmsg(msgBuffer);
      pat_p=progc(api5parms_p->pname,PROGC_PBI);
      if(pat_p!=NULL)
      {
        prtmsg("progc: case-1 completed successfully\n");
        if(api5parms_p->snap !=0)
        {
          snapstuff[0]=(struct snapc_list *) &list[0];
          snapstuff[1]=(struct snapc_list *) &list[1];

          snapstuff[0]->snapc_len=api5parms_p->snap;
          snapstuff[0]->snapc_name="PAT";
          snapstuff[0]->snapc_tag = pat_p;
          snapstuff[0]->snapc_indir=SNAPC_NOINDIR;

          snapstuff[1]->snapc_len=0;
          snapc(SNAPC_RETURN,0x00001,"PAT",snapstuff,'I',SNAPC_NOREGS,SNAPC_ECB,NULL);
        }
      }
      else
      {
        prtmsg("progc: case-1 failed\n");
      }
    }
    if (api5parms_p->icase==0 | api5parms_p->icase==2)
    {
      /*****************************************************************/
      /* Test Case 2:       Testing DBI parm                           */
      /*                                                               */
      /* Test Result:       Normal exit with no errors.                */
      /* Error Msg Output:  None                                       */
      /*****************************************************************/
      sprintf(msgBuffer, "progc: 2-progc(%.4s,PROGC_PBI)\n",api5parms_p->pname);
      prtmsg(msgBuffer);
      pat_p =progc(api5parms_p->pname,PROGC_DBI);
      if(pat_p!=NULL)
      {
        prtmsg("progc: case-2 completed successfully\n");
        if(api5parms_p->snap !=0)
        {
          snapstuff[0]=(struct snapc_list *) &list[0];
          snapstuff[1]=(struct snapc_list *) &list[1];

          snapstuff[0]->snapc_len=api5parms_p->snap;
          snapstuff[0]->snapc_name="PAT";
          snapstuff[0]->snapc_tag = pat_p;
          snapstuff[0]->snapc_indir=SNAPC_NOINDIR;

          snapstuff[1]->snapc_len=0;
          snapc(SNAPC_RETURN,0x00001,"PAT" ,snapstuff,'I',SNAPC_NOREGS,SNAPC_ECB,NULL);
        }
      }
      else
      {
        prtmsg("progc: case-2 failed\n");
      }
    }
    prtmsg("progc: completed progc_func.\n");
    return;

  }/*end if                                                          */

  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
  {
    /*****************************************************************/
    /* NULL program name pointer test                                */
    /*****************************************************************/
    if (api5parms_p->icase==0 | api5parms_p->icase==1)
    {
      prtmsg("progc: processing progc_func error case 1.\n");
      pat_p =progc(NULL,PROGC_PBI);
      if(pat_p!=NULL)
      {
        prtmsg("progc: NULL program name pointer error case-1 failed\n");
      }
      else
      {
        prtmsg("progc: NULL program name pointer error case-1 success\n");
      }
    }
    prtmsg("progc: processing progc_func error cases.\n");

  }/*end else                                                        */

  prtmsg("progc: completed progc_func.\n");
  return;

}/*end of progc_func()                                               */

/*********************************************************************/
/*********************************************************************/
/* Function Name:    getpc_func()                                    */
/*                                                                   */
/* Number of Cases:  ? regular exit test cases                       */
/*                   0 system exit test cases                        */
/*                                                                   */
/*********************************************************************/
void getpc_func(void)
{
  int rc;
  void *prog_p;
  api5parms *api5parms_p;
  memcpy(&(api5parms_p),&(ecbptr()->ebx008),sizeof(api5parms_p));
  prtmsg("getpc: entering getpc_func.\n");

  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    if (api5parms_p->icase==0 | api5parms_p->icase==1)
    {
      /*****************************************************************/
      /* Test Case 1:       Normal Completion Test.                    */
      /* Test Result:       Normal exit with no errors.                */
      /* Error Msg Output:  None                                       */
      /*****************************************************************/
      prtmsg("getpc: Case-1 GETPC_DBI.\n");
      rc=getpc(api5parms_p->pname,GETPC_LOCK,NO_LVL,&prog_p,GETPC_NODUMP,GETPC_DBI,api5parms_p->lset);
      if(rc ==GETPC_SUCCESSFUL)
      {
        prtmsg("getpc: case-1 completed successfully.\n");
        if(api5parms_p->snap !=0)
        {
          snapstuff[0]=(struct snapc_list *) &list[0];
          snapstuff[1]=(struct snapc_list *) &list[1];

          snapstuff[0]->snapc_len=api5parms_p->snap;
          snapstuff[0]->snapc_name="GETPC";
          snapstuff[0]->snapc_tag = prog_p;
          snapstuff[0]->snapc_indir=SNAPC_NOINDIR;

          snapstuff[1]->snapc_len=0;
          snapc(SNAPC_RETURN,0x00001,"PROG" ,snapstuff,'I',SNAPC_NOREGS,SNAPC_ECB,NULL);
        }
      }
      else
      {
        prtmsg("getpc: case-1 failed. ");
        getpc_emsg(rc);
      }
    }
    if (api5parms_p->icase==0 | api5parms_p->icase==2)
    {
      /*****************************************************************/
      /* Test Case 2:       GETPC_PBI.                                 */
      /* Test Result:       Normal exit with no errors.                */
      /* Error Msg Output:  None                                       */
      /*****************************************************************/
      prtmsg("getpc: Case-2 GETPC_PBI.\n");
      rc=getpc(api5parms_p->pname,GETPC_LOCK,NO_LVL,&prog_p,GETPC_NODUMP,GETPC_PBI,api5parms_p->lset);
      if(rc ==GETPC_SUCCESSFUL)
      {
        prtmsg("getpc: case-2 completed successfully.\n");
        if(api5parms_p->snap !=0)
        {
          snapstuff[0]=(struct snapc_list *) &list[0];
          snapstuff[1]=(struct snapc_list *) &list[1];

          snapstuff[0]->snapc_len=api5parms_p->snap;
          snapstuff[0]->snapc_name="GETPC";
          snapstuff[0]->snapc_tag = prog_p;
          snapstuff[0]->snapc_indir=SNAPC_NOINDIR;

          snapstuff[1]->snapc_len=0;
          snapc(SNAPC_RETURN,0x00001,"PROG" ,snapstuff,'I',SNAPC_NOREGS,SNAPC_ECB,NULL);
        }
      }
      else
      {
        prtmsg("getpc: case-2 failed. ");
        getpc_emsg(rc);
      }
    }
    if (api5parms_p->icase==0 | api5parms_p->icase==3)
    {
      /*****************************************************************/
      /* Test Case 3:       GETPC_DUMP.                                */
      /* Test Result:       Normal exit with no errors.                */
      /* Error Msg Output:  None                                       */
      /*****************************************************************/
      prtmsg("getpc: Case-3 GETPC_DUMP.\n");
      rc=getpc(api5parms_p->pname,GETPC_LOCK,NO_LVL,&prog_p,GETPC_DUMP,GETPC_DBI,api5parms_p->lset);
      if(rc ==GETPC_SUCCESSFUL)
      {
        prtmsg("getpc: case-3 completed successfully.\n");
        if(api5parms_p->snap !=0)
        {
          snapstuff[0]=(struct snapc_list *) &list[0];
          snapstuff[1]=(struct snapc_list *) &list[1];

          snapstuff[0]->snapc_len=api5parms_p->snap;
          snapstuff[0]->snapc_name="GETPC";
          snapstuff[0]->snapc_tag = prog_p;
          snapstuff[0]->snapc_indir=SNAPC_NOINDIR;

          snapstuff[1]->snapc_len=0;
          snapc(SNAPC_RETURN,0x00001,"PROG" ,snapstuff,'I',SNAPC_NOREGS,SNAPC_ECB,NULL);
        }
      }
      else
      {
        prtmsg("getpc: case-3 failed. ");
        getpc_emsg(rc);
      }
    }
    if (api5parms_p->icase==0 | api5parms_p->icase==4)
    {
      /*****************************************************************/
      /* Test Case 4:       LSET=NULL .                                */
      /* Test Result:       Normal exit with no errors.                */
      /* Error Msg Output:  None                                       */
      /*****************************************************************/
      prtmsg("getpc: Case-4 LOADSET NULL\n");
      rc=getpc(api5parms_p->pname,GETPC_LOCK,NO_LVL,&prog_p,GETPC_DUMP,GETPC_DBI,NULL);
      if(rc ==GETPC_SUCCESSFUL)
      {
        prtmsg("getpc: case-4 completed successfully.\n");
        if(api5parms_p->snap !=0)
        {
          snapstuff[0]=(struct snapc_list *) &list[0];
          snapstuff[1]=(struct snapc_list *) &list[1];

          snapstuff[0]->snapc_len=api5parms_p->snap;
          snapstuff[0]->snapc_name="GETPC";
          snapstuff[0]->snapc_tag = prog_p;
          snapstuff[0]->snapc_indir=SNAPC_NOINDIR;

          snapstuff[1]->snapc_len=0;
          snapc(SNAPC_RETURN,0x00001,"PROG" ,snapstuff,'I',SNAPC_NOREGS,SNAPC_ECB,NULL);
        }
      }
      else
      {
        prtmsg("getpc: case-4 failed. ");
        getpc_emsg(rc);
      }
    }


    if (api5parms_p->icase==0 | api5parms_p->icase==5)
    {
      /*****************************************************************/
      /* Test Case 5:       Normal Completion Test.                    */
      /* Test Result:       Normal exit with no errors.                */
      /* Error Msg Output:  None                                       */
      /*****************************************************************/
      prtmsg("getpc: Case-5 GETPC_NOLOCK.\n");
      rc=getpc(api5parms_p->pname,GETPC_NOLOCK,DF,&prog_p,GETPC_NODUMP,GETPC_DBI,api5parms_p->lset);
      if(rc ==GETPC_SUCCESSFUL)
      {
        prtmsg("getpc: case-5 completed successfully.\n");
        if(api5parms_p->snap !=0)
        {
          snapstuff[0]=(struct snapc_list *) &list[0];
          snapstuff[1]=(struct snapc_list *) &list[1];

          snapstuff[0]->snapc_len=api5parms_p->snap;
          snapstuff[0]->snapc_name="GETPC";
          snapstuff[0]->snapc_tag = prog_p;
          snapstuff[0]->snapc_indir=SNAPC_NOINDIR;

          snapstuff[1]->snapc_len=0;
          snapc(SNAPC_RETURN,0x00001,"PROG" ,snapstuff,'I',SNAPC_NOREGS,SNAPC_ECB,NULL);
        }
      }
      else
      {
        prtmsg("getpc: case-5 failed. ");
        getpc_emsg(rc);
      }
    }


    if (api5parms_p->icase==0 | api5parms_p->icase==6)
    {
      /*****************************************************************/
      /* Test Case 6:       Normal Completion Test.                    */
      /* Test Result:       Normal exit with no errors.                */
      /* Error Msg Output:  None                                       */
      /*****************************************************************/
      prtmsg("getpc: Case-6 GETPC_SPECIAL.\n");
      rc=getpc(api5parms_p->pname,GETPC_SPECIAL,DF,&prog_p,GETPC_NODUMP,GETPC_DBI,api5parms_p->lset);
      if(rc ==GETPC_SUCCESSFUL)
      {
        prtmsg("getpc: case-5 completed successfully.\n");
        if(api5parms_p->snap !=0)
        {
          snapstuff[0]=(struct snapc_list *) &list[0];
          snapstuff[1]=(struct snapc_list *) &list[1];

          snapstuff[0]->snapc_len=api5parms_p->snap;
          snapstuff[0]->snapc_name="GETPC";
          snapstuff[0]->snapc_tag = prog_p;
          snapstuff[0]->snapc_indir=SNAPC_NOINDIR;

          snapstuff[1]->snapc_len=0;
          snapc(SNAPC_RETURN,0x00001,"PROG" ,snapstuff,'I',SNAPC_NOREGS,SNAPC_ECB,NULL);
        }
      }
      else
      {
        prtmsg("getpc: case-5 failed. ");
        getpc_emsg(rc);
      }
    }

  }/*end if                                                          */

  /*******************************************************************/
  /* if ERR is specified                                             */
  /*******************************************************************/
  else
    prtmsg("getpc: no error cases for getpc()\n");

  prtmsg("getpc: completed getpc_func.\n");
  return;
}/*end of getpc_func()                                               */

void getpc_emsg(int inrc)
{
  char *txtmsg;
  switch (inrc)
  {
    case GETPC_SNAPC_ERR:
      txtmsg = "SNAPC ERROR\n";
      break;
    case GETPC_LS_NF:
      txtmsg = "Loadset not found\n";
      break;
    case GETPC_RETRIEVE_ERROR:
      txtmsg = "Retrieval error\n";
      break;
    case GETPC_PGM_NF:
      txtmsg = "Program not found\n";
      break;
    default:
      txtmsg = "Unknown error\n";
  }
  prtmsg(txtmsg);
  return;
}

}/*end of extern "C"                                                 */
