/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2007                                     */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20041001  Initial Version                                         */
/* 20070205  tpf_cryptc() driver                                     */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name: qmac60.cpp                                          */
/* Contents:     sonic_func                                          */
/*               snapc_func                                          */
/*               serrc_func                                          */
/*               numbc_func                                          */
/*               cifrc_func                                          */
/* Abstract:                                                         */
/*                                                                   */
/* Description:                                                      */
/*                                                                   */
/*********************************************************************/
#include <c_api5.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <tpf/tpfapi.h>
#include <tpf/c_eb0eb.h>
#include <tpf/tpfio.h>
#include <tpf/tpfeq.h>
#include <tpf/sysapi.h>
#include <tpf/tpfapi.h>

#define SONIC_TYPE_INDICATOR 0x80000000
#define SONIC_POOL_LONGEVITY 0x40000000
#define SONIC_RECORD_UNIQUE  0x00008000
#define SONIC_DUPLICATED     0x00000004
#define SONIC_REC_SIZE       0x00000001

extern "C" {

void sonic_func(void)
{
/********************************************************************/
/*          sonic_func                                              */
/* Given a symbolic file address, this function returns the file    */
/* address reference format (FARFx) addressing mode and record type */
/* of that file address.                                            */
/********************************************************************/

  unsigned int fa4 = 0;
  unsigned int prog1_ordinal;
  unsigned int attributes;
  TPF_FA8 fa8 =0;



  prtmsg("sonic:  entering sonic_func test\n");

/********************************************************************/
//          if (NO)ERR is specified...   CASES #1 & #2
/********************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    prtmsg("sonic: processing valid cases\n");

/********************************************************************/
//                       CASE#1
/********************************************************************/
    prtmsg("sonic: 1-entering function VALID CASE#1\n");
    prtmsg("sonic: 1-HOTREC rec type,fixed, dupe, large\n");

// Moving fixed record address into FARW:  ecbptr()->ebcfa1
    int rc = face_facs(1,"#HOTREC",0,D1,&prog1_ordinal);

    if(rc!=0)
    {
      prterr("sonic: 1-unexpected result -- invalid file address\n");
    }

    if (sonic(D1)==0)
     prterr("sonic: 1-unexpected result -- file unsuccessfully decoded\n");

    else

    {
      //   Assigning the output of sonic to a savearea (attributes)
      attributes = sonic(D1);

      if ((attributes && SONIC_RECORD_UNIQUE)==0)
      {
        prterr("sonic: 1-unexpected result -- record unique\n");
      }

      if ((attributes && SONIC_TYPE_INDICATOR)==0)
      {
        prterr("sonic: 1-unexpected result -- pool record\n");
      }

      if ((attributes && SONIC_DUPLICATED)!=0)
      {
        if ((attributes && SONIC_REC_SIZE)== 0)
          prterr("sonic: 1-unexpected result -- small record\n");
        else
          prtmsg("sonic: 1-successfully completing VALID CASE#1\n");
      }
      else prterr("sonic: 1-unexpected result -- record not duplicated\n");
    }/* (sonic(D1)==0)*/

    /*********************************************************************/
    //                             CASE#2
    /*********************************************************************/
    prtmsg("sonic: 2- entering function VALID CASE#2\n");
    prtmsg("sonic: 2- 'OM' rec type, short term pool, not DUPE, LARGE\n");

    fa4=getfc(D2,GETFC_TYPE0,"OM",GETFC_NOBLOCK,GETFC_SERRC);


    tpf_fa4x4c(FACONV_4TO8,&fa4,&fa8);

    if (sonic(&fa8)==0)

      prterr("sonic: 2-unexpected result -- file unsuccessfully decoded\n");
    else
    {
      //   Assigning the output of sonic to a savearea  (attributes)

      attributes=sonic(&fa8);

      if((attributes && SONIC_TYPE_INDICATOR)!=0)
      {
         if((attributes && SONIC_POOL_LONGEVITY)!=0)
         {
           if((attributes && SONIC_DUPLICATED)==0)

              prterr("sonic: 2-unexpected result -- record duplicated\n");

           else
           {
              if ((attributes && SONIC_REC_SIZE)==0)
                 prterr("sonic: 2-unexpected result -- small record\n");
              else
                 prtmsg("sonic: 2-successfully completing VALID CASE#2\n");
           }
        } /* SONIC_LONGEVITY */
        else
           prterr("sonic: 2-unexpected result -- not short term pool\n");
      }/* SONIC_TYPE_INDICATOR)*/
      else
        prterr("sonic: 2-unexpected result -- fixed record\n");
    } /* sonic(&fa8)==0) */

    relfc(D2);

    prtmsg("sonic: exiting VALID CASE test\n");
  }
   else

/********************************************************************/
//                 if ERR specified...   CASES #1 & #2 & #3
/********************************************************************/

  {
    prtmsg("sonic: processing error cases\n");

    unsigned int    err_fa4 = 0x00000001;
    TPF_FA8          err1_fa8 = 0x0000000000000001;
    TPF_FA8          err2_fa8 = NULL;

    /*********************************************************************/
    //                       ERROR CASE#1
    /*********************************************************************/


    prtmsg("sonic: 1-entering function ERROR CASE#1\n");

    memcpy(&ecbptr()->ce1fm1,(void *)&err_fa4,4);

    if (sonic(D1) != 0)
    {
      prterr("sonic: 1-unexpected result -- error address on data level decoded\n");
    }

    prtmsg("sonic: 1-successfully completing ERROR CASE#1\n");


    /********************************************************************/
    //                    ERROR CASE#2
    /********************************************************************/
    prtmsg("sonic: 2-entering function ERROR CASE#2\n");

    if (sonic(&err1_fa8)!=0)
    {
      prterr("sonic: 2-unexpected result -- error 8-byte file address decoded\n");
    }

    prtmsg("sonic: 2-successfully completing ERROR CASE#2\n");

    /********************************************************************/
    //                    ERROR CASE#3
    /********************************************************************/

    prtmsg("sonic: 3-entering function ERROR CASE#3\n");

    if (sonic(&err2_fa8)!=0)
    {
      prterr("sonic: 3-unexpected result -- null file address decoded\n");
    }

    prtmsg("sonic: 3-successfully completing ERROR CASE#3\n");

    prtmsg("sonic: exiting ERROR CASE test\n");

  }
  prtmsg("sonic: exiting sonic_func test\n");
  return;
  } /* sonic_func() */

void snapc_func(void)
{
/*********************************************************************/
/*                     snapc_func                                    */
/* This function causes the system error routine to issue a          */
/* a snapshot dump and display, optionally, a message at prime       */
/* CRAS. SNAPC_EXIT to exit the ECB, SNAPC_RETURN to return          */
/* to the calling program.                                           */
/***************                                                     */
/* HLDE updates:                                                     */
/***************                                                     */
/* 1. Code defined term SNAPC_MTRACE_CONS to force the ECB macro     */
/*    macro trace to be dumped and the last 8 entries formatted  to  */
/*    the console.                                                   */
/* 2. Code defined term SNAPC_MTRACE to force the ECB macro trace    */
/*    to be dumped.                                                  */
/*********************************************************************/

  prtmsg("snapc: entering snapc_func test\n");

/*********************************************************************/
//          if ERR is specified...
/*********************************************************************/
  if (ecbptr()->ebx000 == TRUE)
  {
    prtmsg("snapc: processing valid case\n");

     struct snapc_list *snapstuff[2],list[2];

  snapstuff[0]=(struct snapc_list *) &list[0];
  snapstuff[1]=(struct snapc_list *) &list[1];


/*  Dump the 8 bytes of data in EBW000-EBW007                        */
  snapstuff[0]->snapc_len=8;
  snapstuff[0]->snapc_name="TESTDUMP";
  snapstuff[0]->snapc_tag=&ecbptr()->ebw000;
  snapstuff[0]->snapc_indir=SNAPC_NOINDIR;

  snapstuff[1]->snapc_len=0;

/*   'eye-catcher' data to verify dump area EBW000-EBW007            */
  memcpy(&ecbptr()->ebw000,"FFFFFFFF",8);

/*  Function issues Snapshot dump                                    */
/*  ID# 2345, Prefix "A'                                             */

/*********************************************************************/
/* Program not picking up parameters SNAPC_MTRACE/SNAPC_MTRACE_CONS  */
/* because the header <tpfapi.h> has not been updated as yet.        */
/*********************************************************************/

/*  z/TPF UPDATE:                                                    */
/* snapc(SNAPC_RETURN,0x2345,"SNAPC DRIVER TEST SUCCESSFUL",         */
/*   snapstuff,'A',SNAPC_REGS,SNAPC_ECB,SNAPC_MTRACE_CONS,NULL);     */

   snapc(SNAPC_RETURN,0x2345,"SNAPC DRIVER TEST SUCCESSFUL",
   snapstuff,'A',SNAPC_REGS,SNAPC_ECB,NULL);

   }
 else
/********************************************************************/
//                 if (NO)ERR specified...
/********************************************************************/

  {
    prtmsg("snapc: no valid case for snapc_func test\n");
  }

   prtmsg("snapc: exiting snapc_func test\n");
   return;
} /* snapc_func() */

/*********************************************************************/
/*                serrc_func                                         */
/*********************************************************************/
/* This function causes a storage dump to be issued by the           */
/* system error routine, and may optionally display a message        */
/* at prime CRAS. The ECB may be exited if desired.                  */
/**************                                                      */
/* HLDE update:                                                      */
/**************                                                      */
/* 1. To dump both 31/64 bit versions, code SERRC_NO_HEAP option.    */
/*                                                                   */
/*********************************************************************/

void serrc_func()
{
   prtmsg("serrc: entering serrc_func test\n");

/*********************************************************************/
//          if ERR is specified...
/*********************************************************************/
  if (ecbptr()->ebx000 == TRUE)
  {
    prtmsg("serrc: processing valid case\n");

     serrc_op_ext(SERRC_RETURN,0x2346,
               "SERRC DRIVER TEST SUCCESSFUL", 'A',NULL);


/* SERRC_NO_HEAP not applied to header file as yet                   */

/*  Code Changes to be made........                                  */
/*  z/TPF UPDATE                                                     */
/* serrc_op_ext(SERRC_RETURN+SERRC_NO_HEAP,0x2345,                   */
/*     "SERRC DRIVER TEST SUCCESSFUL", 'A',NULL);                    */

  serrc_op_ext(SERRC_RETURN,0x2346,
               "SERRC DRIVER TEST SUCCESSFUL", 'A',NULL);


 }
/********************************************************************/
//                 if (NO)ERR specified...
/********************************************************************/

 else
  {
    prtmsg("serrc: no valid case for serrc_func test\n");
  }

   prtmsg("serrc: exiting serrc_func test\n");
   return;

} /* serrc_func */

/*********************************************************************/
/*                numbc_func -- no HLDE update (to date)             */
/* This function queries the number of storage blocks available and  */
/* returns the number of blocks currently available for a specified  */
/* physical block type.                                              */
/* Valid physical block type: LIOCB, L3, LECB, LSWB, LCOMMON, LFRAME */
/* or LMAX.                                                          */
/*                                                                   */
/* RUN INSTRUCTIONS: Use ZSTAT to display available blocks.          */
/*                   Compare with values produced by numbc,          */
/*                   ie. currently available blocks.                 */
/*********************************************************************/
void numbc_func()
{
  prtmsg("numbc:  entering numbc_func test\n");

/*********************************************************************/
//          if (NO)ERR is specified...
/*********************************************************************/
  if (ecbptr()->ebx000 == FALSE)
{
 prtmsg("numbc: processing valid case\n");
 prtmsg("numbc: 1-entering numbc_func test\n");
 prtmsg("numbc: 1-use ZSTAT to display available blocks\n");

 char buffer[100];

/*  z/TPF UPDATE: not yet available  2/28/02  */

/*           iobs currently available    */
   int iobs = numbc(LIOCB);
   sprintf(buffer,"numbc: 1-IOBS currently available = %d\n",iobs);
   prtmsg(buffer);

/*           swbs currently available  */
   int swbs = numbc(LSWB);
   sprintf(buffer,"numbc: 1-SWBS currently available = %d\n",swbs);
   prtmsg(buffer);

/*           ecbs currently available  */
   int ecbs = numbc(LECB);
   sprintf(buffer,"numbc: 1-ECBS currently available = %d\n",ecbs);
   prtmsg(buffer);

/*           frames currently available  */
  int frms = numbc(LFRAME);
  sprintf(buffer,"numbc: 1-FRAMES currently available = %d\n",frms);
  prtmsg(buffer);

/*           common blocks currently available  */
  int cmbs = numbc(LCOMMON);
  sprintf(buffer,"numbc: 1-COMMON BLOCKS currently available = %d\n",cmbs);
  prtmsg(buffer);

   }

else
  {
    prtmsg("numbc: no error case for numbc_func test\n");
  }

   prtmsg("numbc: exiting numbc_func test\n");
   return;

} /* numbc_func */


/*********************************************************************/
/*                cifrc_func -- no HLDE update (to date)             */
/* This function tests the cifrc() TPF API to verify we can          */
/* cypher and decypher both character and binary data.               */
/*                                                                   */
/*********************************************************************/
void cifrc_func()
{
#define plaintext "AbCdEfGh"
#define etext      0xf9ae804f6cb140a4L
#define plainbin   0x0877665544332211L
#define ebin       0x04d412289224beb3L
#define deskey     720510L

   struct cifrc_arg cifrc_parm;
   u_long       key = deskey;                                   //64CIFRC
   char        *char_data, *buf = NULL;                         //64CIFRC
   u_long       bin_data = plainbin;                            //64CIFRC
   short        err = 0;                                        //64CIFRC

   prtmsg("cifrc: entering cifrc_func test\n");
   char_data = (char *)malloc(32);  // space for long, plus some  64CIFRC

   /*----------------------------------------------------------------*/
   /* Start test for a character field.                              */
   /*----------------------------------------------------------------*/
   strcpy(char_data, plaintext);                                //64CIFRC
   cifrc_parm.cifreqst = 0;
   cifrc_parm.cifkey = (void *)&key;                            //64CIFRC
   cifrc_parm.ciftext = char_data;                              //64CIFRC
   cifrc(CIFRC_DES, &cifrc_parm);                               //64CIFRC

/* Test that key wasn't changed on encipher */
   if (cifrc_parm.cifkey != (char *)&key ) {                    //64CIFRC
      prtmsg("cifrc: failed key test 1\n");                     //64CIFRC
      err = 1;                                                  //64CIFRC
   }                                                            //64CIFRC
   key = deskey;                                                //64CIFRC
   if (memcmp(cifrc_parm.cifkey, &key, 8)) {                    //64CIFRC
      prtmsg("cifrc: failed key test 2\n");                     //64CIFRC
      err = 1;                                                  //64CIFRC
   }                                                            //64CIFRC
   if (cifrc_parm.ciftext != char_data ) {                      //64CIFRC
      prtmsg("cifrc: failed cipher text 1\n");                  //64CIFRC
      err = 1;                                                  //64CIFRC
   }                                                            //64CIFRC
   if (etext != *(u_long*)char_data ) {                         //64CIFRC
      prtmsg("cifrc: failed cipher text 2\n");                  //64CIFRC
      buf = (char *)malloc(81);                                 //64CIFRC
      sprintf(buf, "cifrc: expected %016lx, got %016lx\n",      //64CIFRC
         etext, *(u_long*)char_data);
      prtmsg(buf);                                              //64CIFRC
      err = 1;                                                  //64CIFRC
   }                                                            //64CIFRC

   cifrc_parm.cifreqst = 1;
   cifrc_parm.cifkey = (char *)&key;                            //64CIFRC
   cifrc_parm.ciftext = char_data;                              //64CIFRC
   cifrc(CIFRC_DES, &cifrc_parm);                               //64CIFRC
   if (!strcmp(char_data, plaintext)) {                         //64CIFRC
      prtmsg("cifrc:  passed character encryption test\n");
   } else {
      prtmsg("cifrc:  failed character encryption test\n");
      printf("cifrc: expected %s, got %s\n", plaintext, char_data); //64CIFRC
      err = 1;                                                  //64CIFRC
   }                                                            //64CIFRC

/* Test that key wasn't changed on decipher */
   if (cifrc_parm.cifkey != (char *)&key ) {                    //64CIFRC
      prtmsg("cifrc: failed key test 3\n");                     //64CIFRC
      err = 1;                                                  //64CIFRC
   }                                                            //64CIFRC
   key = deskey;                                                //64CIFRC
   if (memcmp(cifrc_parm.cifkey, &key, 8)) {                    //64CIFRC
      prtmsg("cifrc: failed key test 4\n");                     //64CIFRC
      err = 1;                                                  //64CIFRC
   }                                                            //64CIFRC

   /*----------------------------------------------------------------*/
   /* Start test for a binary field.                                 */
   /*----------------------------------------------------------------*/
   cifrc_parm.cifreqst = 0;
   cifrc_parm.cifkey = (char *)&key;                            //64CIFRC
   cifrc_parm.ciftext = (char *)&bin_data;                      //64CIFRC
   cifrc(CIFRC_DES, &cifrc_parm);                               //64CIFRC

   if (bin_data != ebin) {                                      //64CIFRC
      prtmsg("cifrc:  failed binary encryption test 1\n");      //64CIFRC
      if (buf == NULL)                                          //64CIFRC
         buf = (char *)malloc(81);                              //64CIFRC
      sprintf(buf, "cifrc: expected %016lx, got %016lx\n",      //64CIFRC
         ebin, bin_data);
      prtmsg(buf);                                              //64CIFRC
      err = 1;                                                  //64CIFRC
   }
   cifrc_parm.cifreqst = 1;
   cifrc_parm.cifkey = (char *)&key;                            //64CIFRC
   cifrc_parm.ciftext = (char *)&bin_data;                      //64CIFRC
   cifrc(CIFRC_DES, &cifrc_parm);                               //64CIFRC
   if (bin_data == plainbin) {                                  //64CIFRC
       prtmsg("cifrc:  passed binary encryption test\n");
   } else {
      prtmsg("cifrc:  failed binary encryption test 2\n");
      err = 1;                                                  //64CIFRC
   }

   if (err)                                                     //64CIFRC
      prtmsg("cifrc:  exiting cifrc_func test, with errors\n"); //64CIFRC
   else                                                         //64CIFRC
      prtmsg("cifrc:  exiting cifrc_func test\n");

   free(char_data);                                             //64CIFRC
   if (buf != NULL)                                             //64CIFRC
      free(buf);                                                //64CIFRC
   return;

} /* cifrc_func */
/*********************************************************************/
/*                cryptc_func -- no HLDE update (to date)            */
/* This function tests the tpf_cryptc() TPF API to cipher and        */
/* decipher the character and binary data with                       */
/* cipher parameter TPF_DES, TPF_TDES, TPF_DES_CBC                   */
/* and TPF_TDES_CBC.                                                 */
/*                                                                   */
/*********************************************************************/
void cryptc_func()                                                              //cpacfdrv
{
#define ERROR    1
#define plainbinx   0x0877665544332211L
#define plaintxt   "ABCDEFGH12345678"

    int  rc;
    struct tpf_crypt_parms parms;
    char buffer[24];
    char key[24];
    char icv[24];
    short    err = 0;
    u_long    bin_data = plainbinx;

    memset(buffer, '\0', sizeof(buffer));
    memset(key, '\0', sizeof(key));
    memset(icv, '\0', sizeof(icv));
    strcpy(buffer, plaintxt);
    strcpy(key, "abcdefgh");
    strcpy(icv, "12345678");

    /**********************************************************/
    /* Test invalid parm                                      */
    /**********************************************************/
    rc = tpf_cryptc(TPF_ENC, 3, &parms);    /* Invalid cipher */
    if (rc != TPF_CRYPT_INVCIPHER) {
     printf("tpf_cryptc: Failed on tesing invalid cipher\n");
     err = ERROR;
    }

    rc = tpf_cryptc(9, TPF_DES, &parms);    /* Invalid request*/
    if (rc != TPF_CRYPT_INVREQ) {
     printf("tpf_cryptc: Failed on tesing invalid request\n");
     err = ERROR;
    }

    parms.tpf_crypt_in = buffer;
    parms.tpf_crypt_len = 6;                /* Invalid length */
    parms.tpf_crypt_out = buffer;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = icv;
    rc = tpf_cryptc(TPF_ENC, TPF_DES, &parms);
    if (rc != TPF_CRYPT_BADLENG) {
     printf("tpf_cryptc: BAD DATA LENGTH (len = 6)\n");
     err = ERROR;
    }

    parms.tpf_crypt_in = buffer;
    parms.tpf_crypt_len = 42;                /* Invalid length */
    parms.tpf_crypt_out = buffer;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = icv;
    rc = tpf_cryptc(TPF_ENC, TPF_DES, &parms);
    if (rc != TPF_CRYPT_BADLENG) {
     printf("tpf_cryptc: BAD DATA LENGTH (len = 42)\n");
     err = ERROR;
    }

    parms.tpf_crypt_in = buffer;
    parms.tpf_crypt_len = 2000000;            /* Invalid length */
    parms.tpf_crypt_out = buffer;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = icv;
    rc = tpf_cryptc(TPF_ENC, TPF_DES, &parms);
    if (rc != TPF_CRYPT_BADLENG) {
     printf("tpf_cryptc: BAD DATA LENGTH (len = 2000000)\n");
     err = ERROR;
    }

    parms.tpf_crypt_in = NULL;       /* no data address */
    parms.tpf_crypt_len = 8;
    parms.tpf_crypt_out = buffer;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = icv;
    rc = tpf_cryptc(TPF_ENC, TPF_DES, &parms);
    if (rc != TPF_CRYPT_NOADDR) {
     printf("tpf_cryptc: Failed on tesing no data addr\n");
     err = ERROR;
    }


    parms.tpf_crypt_in = buffer;
    parms.tpf_crypt_len = 8;
    parms.tpf_crypt_out = buffer;
    parms.tpf_crypt_key = NULL;      /* No key */
    parms.tpf_crypt_icv = icv;
    rc = tpf_cryptc(TPF_ENC, TPF_DES, &parms);
    if (rc != TPF_CRYPT_NOKEY) {
    printf("tpf_cryptc: Failed on tesing NO KEY SPECIFIED\n");
     err = ERROR;
    }

    parms.tpf_crypt_in = buffer;
    parms.tpf_crypt_len = 8;
    parms.tpf_crypt_out = buffer;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = NULL;     /* No ICV */
    rc = tpf_cryptc(TPF_ENC, TPF_DES_CBC, &parms);
    if (rc != TPF_CRYPT_NOICV) {
     printf("tpf_cryptc: Failed on tesing NO ICV\n");
     err = ERROR;
    }

/***** end of invalid parm test cases *******************************/

/****************************************************************/
/*  TPF_DES_CBC  (Character data)                               */
/****************************************************************/

    parms.tpf_crypt_in = buffer;
    parms.tpf_crypt_len = 16;
    parms.tpf_crypt_out = buffer;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = icv;

/*  printf("******************************************\n");*/
/*  printf("** Encipher character data on DES_CBC **\n");*/
    rc = tpf_cryptc(TPF_ENC, TPF_DES_CBC, &parms);
/*  printf("rc from tpf_cryptc ENC, TPF_DES_CBC is : %i\n", rc);*/

/* Test that KEY wasn't changed on encipher */
/* printf("**** Before encipher, KEY is %s \n",key);*/
/* printf("**** After encipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Encipher KEY err1, key should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_key, key, 8)) {
      printf("tpf_cryptc: Encipher KEY err2, key should be the same\n");
      err = ERROR;
   }

/* Test that ICV wasn't changed on encipher */
/* printf("**** Before enciper, ICV is %s \n",icv);*/
/* printf("**** After encipher, ICV is %s \n",parms.tpf_crypt_icv);*/

   if (parms.tpf_crypt_icv != icv ) {
      printf("tpf_cryptc: Encipher ICV err1, ICV should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_icv, icv, 8)) {
      printf("tpf_cryptc: Encipher ICV err2, ICV should be the same\n");
      err = ERROR;
   }

/*printf("** Text (character) after encipher %016lx\n",(long*)buffer);*/

   rc = memcmp(buffer, plaintxt, 16);
   if (rc == 0) {
  printf("tpf_cryptc: Encipher text err1, text is the same\n");
      err = ERROR;
   }

/*    printf("******************************************\n");*/
/*  printf("** Decipher character data on DES_CBC **\n");*/

    strcpy(icv, "12345678");
    parms.tpf_crypt_icv = icv;

    rc = tpf_cryptc(TPF_DEC, TPF_DES_CBC, &parms); /* Decryption */
/*  printf("rc from tpf_cryptc DEC, TPF_DES_CBC is : %i\n", rc);*/

/* Test that key wasn't changed on decipher */
/* printf("**** Before decipher, KEY is %s \n",key);*/
/* printf("**** After decipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
  printf("tpf_cryptc: Decipher KEY err1, key should be the same\n");
      err = ERROR;
   }
   rc = memcmp(parms.tpf_crypt_key, key, 8);
   if (rc != 0) {
   printf("tpf_cryptc: Decipher KEY err2  key should be the same\n");
      err = ERROR;
   }

/* Test that ICV wasn't changed on decipher */
/* printf("**** Before deciper, ICV is %s \n",icv);*/
/* printf("**** After decipher, ICV is %s \n",parms.tpf_crypt_icv);*/

   if (parms.tpf_crypt_icv != icv ) {
      printf("tpf_cryptc: Decipher ICV err1, ICV should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_icv, icv, 8)) {
      printf("tpf_cryptc: Decipher ICV err2, ICV should be the same\n");
      err = ERROR;
   }
/* Test that text wasn't changed after encipher and decipher */
// printf("**** Text (char data) DES_CBC after decipher is %s \n",buffer);

   rc = memcmp(buffer, plaintxt, 16);
   if (rc != 0) {
    printf("tpf_cryptc: Decipher text err(char text) - DES_CBC, text should be the same\n");
      err = ERROR;
   }

/****************************************************************/
/*  TPF_DES_CBC (Binary data)                                   */
/****************************************************************/

    memset(key, '\0', sizeof(key));
    memset(icv, '\0', sizeof(icv));
    strcpy(key, "abcdefgh");
    strcpy(icv, "12345678");

    bin_data = plainbinx;
    parms.tpf_crypt_in = (char *)&bin_data;
    parms.tpf_crypt_len = 8;
    parms.tpf_crypt_out = (char *)&bin_data;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = icv;

/*  printf("******************************************\n");*/
/*  printf("** Encipher binary data on DES_CBC **\n");*/
    rc = tpf_cryptc(TPF_ENC, TPF_DES_CBC, &parms);
/*  printf("rc from tpf_cryptc ENC, TPF_DES_CBC is : %i\n", rc);*/

/* Test that KEY wasn't changed on encipher */
/* printf("**** Before encipher, KEY is %s \n",key);              */
/* printf("**** After encipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Encipher KEY err3, key should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_key, key, 8)) {
      printf("tpf_cryptc: Encipher KEY err4, key should be the same\n");
      err = ERROR;
   }


/* Test that text was changed on encipher */
/* printf("** Text (binary ) before encipher & decipher is %016lx\n",*/
/*         plainbinx);*/
/* printf("** Text after encipher is %016lx\n",bin_data);*/

   if (bin_data == plainbinx) {
printf("tpf_cryptc: Encipher text err2, text should not be the same\n");
    err = ERROR;
   }

    strcpy(icv, "12345678");
    parms.tpf_crypt_icv = icv;

/*  printf("******************************************\n");*/
/*  printf("** Decipher binary data on DES_CBC **\n");*/
    rc = tpf_cryptc(TPF_DEC, TPF_DES_CBC, &parms); /* Decryption */
/*  printf("rc from tpf_cryptc DEC, TPF_DES_CBC is : %i\n", rc);*/

/* Test that key wasn't changed on decipher */
/* printf("**** Before decipher, KEY is %s \n",key);              */
/* printf("**** After decipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Decipher KEY err3  key should be the same\n");
      err = ERROR;
   }
   rc = memcmp(parms.tpf_crypt_key, key, 8);
   if (rc != 0) {
      printf("tpf_cryptc: Decipher KEY err4  key should be the same\n");
      err = ERROR;
   }

/* Test that text wasn't changed after encipher and decipher */
//   printf("** Text before encipher & decipher is %016lx\n",
//         plainbinx);
//   printf("** Text (binary) after decipher is %016lx\n",bin_data);

   if (bin_data != plainbinx) {
    printf("tpf_cryptc: Decipher text err(BIN data) - DES_CBC, text should be the same\n");
      err = ERROR;
   }

/****************************************************************/
/*  TPF_TDES_CBC  (Character data)                              */
/****************************************************************/

    strcpy(icv, "12345678abcdefgh12345678");

    parms.tpf_crypt_in = buffer;
    parms.tpf_crypt_len = 16;
    parms.tpf_crypt_out = buffer;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = icv;

/*  printf("******************************************\n");*/
/*  printf("** Encipher character data on TDES_CBC **\n");*/
    rc = tpf_cryptc(TPF_ENC, TPF_TDES_CBC, &parms);
/*  printf("rc from tpf_cryptc ENC, TPF_TDES_CBC is : %i\n", rc);*/

/* Test that KEY wasn't changed on encipher */
/* printf("**** Before encipher, KEY is %s \n",key);              */
/* printf("**** After encipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Encipher KEY err5, key should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_key, key, 8)) {
      printf("tpf_cryptc: Encipher KEY err6, key should be the same\n");
      err = ERROR;
   }

/* Test that ICV wasn't changed on encipher */
/* printf("**** Before enciper, ICV is %s \n",icv);                */
/* printf("**** After encipher, ICV is %s \n",parms.tpf_crypt_icv);*/

   if (parms.tpf_crypt_icv != icv ) {
      printf("tpf_cryptc: Encipher ICV err5, ICV should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_icv, icv, 8)) {
      printf("tpf_cryptc: Encipher ICV err6, ICV should be the same\n");
      err = ERROR;
   }

/* Test that text was changed on encipher */

   rc = memcmp(buffer, plaintxt, 16);
   if (rc == 0) {
printf("tpf_cryptc: Encipher text err3(char data), text should not be the same\n");

         err = ERROR;
      }

/*  printf("******************************************\n");*/
/*  printf("** Decipher character data on TDES_CBC **\n");*/

    strcpy(icv, "12345678abcdefgh12345678");
    parms.tpf_crypt_icv = icv;

    rc = tpf_cryptc(TPF_DEC, TPF_TDES_CBC, &parms);
/*  printf("rc from tpf_cryptc DEC, TPF_TDES_CBC is : %i\n", rc);*/

/* Test that key wasn't changed on decipher */
/* printf("**** Before decipher, KEY is %s \n",key);              */
/* printf("**** After decipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Decipher KEY err5  key should be the same\n");
      err = ERROR;
   }
   rc = memcmp(parms.tpf_crypt_key, key, 8);
   if (rc != 0) {
      printf("tpf_cryptc: Decipher KEY err6  key should be the same\n");
      err = ERROR;
   }

/* Test that ICV wasn't changed on decipher */
/* printf("**** Before deciper, ICV is %s \n",icv);                */
/* printf("**** After decipher, ICV is %s \n",parms.tpf_crypt_icv);*/

   if (parms.tpf_crypt_icv != icv ) {
      printf("tpf_cryptc: Decipher ICV err5, ICV should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_icv, icv, 8)) {
      printf("tpf_cryptc: Decipher ICV err6, ICV should be the same\n");
      err = ERROR;
   }
/* Test that text wasn't changed after encipher and decipher */
/* printf("**** Text(char data) TDES_CBC after decipher is %s \n",buffer);*/

   rc = memcmp(buffer, plaintxt, 16);
   if (rc != 0) {
    printf("tpf_cryptc: Decipher text err(CHAR DATA) - TDES_CBC, text should be the same\n");
      err = ERROR;
   }

/****************************************************************/
/*  TPF_TDES_CBC (Binary data)                                  */
/****************************************************************/

    memset(key, '\0', sizeof(key));
    memset(icv, '\0', sizeof(icv));
    strcpy(key, "abcdefgh");
    strcpy(icv, "12345678abcdefgh12345678");
    bin_data = plainbinx;

    parms.tpf_crypt_in = (char *)&bin_data;
    parms.tpf_crypt_len = 8;
    parms.tpf_crypt_out = (char *)&bin_data;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = icv;

/*  printf("******************************************\n");*/
/*  printf("** Encipher binary data on TDES_CBC **\n");*/
    rc = tpf_cryptc(TPF_ENC, TPF_TDES_CBC, &parms);
/*  printf("rc from tpf_cryptc ENC, TPF_TDES_CBC is : %i\n", rc);*/

/* Test that KEY wasn't changed on encipher */
/* printf("**** Before encipher, KEY is %s \n",key);              */
/* printf("**** After encipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Encipher KEY err7, key should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_key, key, 8)) {
      printf("tpf_cryptc: Encipher KEY err8, key should be the same\n");
      err = ERROR;
   }

/* Test that ICV wasn't changed on encipher */
/* printf("**** Before enciper, ICV is %s \n",icv);*/
/* printf("**** After encipher, ICV is %s \n",parms.tpf_crypt_icv);*/

   if (parms.tpf_crypt_icv != icv ) {
      printf("tpf_cryptc: Encipher ICV err7, ICV should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_icv, icv, 8)) {
      printf("tpf_cryptc: Encipher ICV err8, ICV should be the same\n");
      err = ERROR;
   }

/* Test that text was changed on encipher */

   if (bin_data == plainbinx) {
printf("tpf_cryptc: Encipher text err4, text should not be the same\n");
    err = ERROR;
   }

    strcpy(icv, "12345678abcdefgh12345678");

    parms.tpf_crypt_in = (char *)&bin_data;
    parms.tpf_crypt_len = 8;
    parms.tpf_crypt_out = (char *)&bin_data;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = icv;

/*  printf("******************************************\n");*/
/*  printf("** Decipher binary data on TDES_CBC **\n");*/
    rc = tpf_cryptc(TPF_DEC, TPF_TDES_CBC, &parms); /* Decryption */
/*  printf("rc from tpf_cryptc DEC is, TPF_TDES_CBC: %i\n", rc);*/

/* Test that key wasn't changed on decipher */
/* printf("**** Before decipher, KEY is %s \n",key);              */
/* printf("**** After decipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Decipher KEY err7, key should be the same\n");
      err = ERROR;
   }
   rc = memcmp(parms.tpf_crypt_key, key, 8);
   if (rc != 0) {
      printf("tpf_cryptc: Decipher KEY err8, key should be the same\n");
      err = ERROR;
   }

/* Test that ICV wasn't changed on decipher */
/* printf("**** Before deciper, ICV is %s \n",icv);                */
/* printf("**** After decipher, ICV is %s \n",parms.tpf_crypt_icv);*/

   if (parms.tpf_crypt_icv != icv ) {
      printf("tpf_cryptc: Decipher ICV err7, ICV should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_icv, icv, 8)) {
      printf("tpf_cryptc: Decipher ICV err8, ICV should be the same\n");
      err = ERROR;
   }
/* Test that text wasn't changed after encipher and decipher */
// printf("** Text before encipher & decipher is %016lx\n",
//         plainbinx);
// printf("** Text after decipher is %016lx\n",bin_data);

   if (bin_data != plainbinx) {
    printf("tpf_cryptc: Decipher text err(BIN data) - TDES_CBC, text should be the same\n");
      err = ERROR;
   }


/****************************************************************/
/*  TPF_TDES      (Character data)                              */
/****************************************************************/

    parms.tpf_crypt_in = buffer;
    parms.tpf_crypt_len = 16;
    parms.tpf_crypt_out = buffer;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = NULL;

/*  printf("******************************************\n");*/
/*  printf("** Encipher character data on TDES **\n");*/
    rc = tpf_cryptc(TPF_ENC, TPF_TDES, &parms);
/*  printf("rc from tpf_cryptc ENC, TPF_TDES is : %i\n", rc);*/

/* Test that KEY wasn't changed on encipher */
/* printf("**** Before encipher, KEY is %s \n",key);              */
/* printf("**** After encipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Encipher KEY err9, key should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_key, key, 8)) {
      printf("tpf_cryptc: Encipher KEY er10, key should be the same\n");
      err = ERROR;
   }

/* Test that text was changed on encipher */
/* printf("** Text (character) after encipher %016lx\n",*/
     /*    (long*)buffer);*/

   rc = memcmp(buffer, plaintxt, 8);
   if (rc == 0) {
printf("tpf_cryptc: Encipher text err5, text should not be the same\n");

         err = ERROR;
      }

/*  printf("******************************************\n");*/
/*  printf("** Decipher character data on TDES **\n");*/
    rc = tpf_cryptc(TPF_DEC, TPF_TDES, &parms);
/*  printf("rc from tpf_cryptc DEC, TPF_TDES is : %i\n", rc);*/

/* Test that key wasn't changed on decipher */
/* printf("**** Before decipher, KEY is %s \n",key);              */
/* printf("**** After decipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Decipher KEY err9  key should be the same\n");
      err = ERROR;
   }
   rc = memcmp(parms.tpf_crypt_key, key, 8);
   if (rc != 0) {
      printf("tpf_cryptc: Decipher KEY erra  key should be the same\n");
      err = ERROR;
   }

/* Test that text wasn't changed after encipher and decipher */
/* printf("**** Text after decipher is %s \n",buffer);*/

   rc = memcmp(buffer, plaintxt, 16);
   if (rc != 0) {
    printf("tpf_cryptc: Decipher text (CHAR DATA) TPF_TDES err5, text should be the same\n");
      err = ERROR;
   }

/****************************************************************/
/*  TPF_TDES     (Binary data)                                  */
/****************************************************************/

    memset(key, '\0', sizeof(key));
    memset(icv, '\0', sizeof(icv));
    strcpy(key, "abcdefgh");
    bin_data = plainbinx;

    parms.tpf_crypt_in = (char *)&bin_data;
    parms.tpf_crypt_len = 8;
    parms.tpf_crypt_out = (char *)&bin_data;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = NULL;

/*  printf("** Encipher binary data on TDES **\n");*/
    rc = tpf_cryptc(TPF_ENC, TPF_TDES, &parms);
/*  printf("rc from tpf_cryptc ENC, TPF_TDES is : %i\n", rc);*/

/* Test that KEY wasn't changed on encipher */
/* printf("**** Before encipher, KEY is %s \n",key);              */
/* printf("**** After encipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Encipher KEY er11, key should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_key, key, 8)) {
      printf("tpf_cryptc: Encipher KEY er12, key should be the same\n");
      err = ERROR;
   }

/* Test that text was changed on encipher */

   if (bin_data == plainbinx) {
printf("tpf_cryptc: Encipher text err6, text should not be the same\n");
    err = ERROR;
   }

    parms.tpf_crypt_in = (char *)&bin_data;
    parms.tpf_crypt_len = 8;
    parms.tpf_crypt_out = (char *)&bin_data;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = NULL;


/*  printf("** Decipher binary data on TDES **\n");*/
    rc = tpf_cryptc(TPF_DEC, TPF_TDES, &parms); /* Decryption */
/*  printf("rc from tpf_cryptc DEC is, TPF_TDES: %i\n", rc);*/

/* Test that key wasn't changed on decipher */
/* printf("**** Before decipher, KEY is %s \n",key);              */
/* printf("**** After decipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Decipher KEY errb, key should be the same\n");
      err = ERROR;
   }
   rc = memcmp(parms.tpf_crypt_key, key, 8);
   if (rc != 0) {
      printf("tpf_cryptc: Decipher KEY errc, key should be the same\n");
      err = ERROR;
   }

/* Test that text wasn't changed after encipher and decipher */
// printf("** Text before encipher & decipher is %016lx\n",
//         plainbinx);
//printf("** TPF_TDES - Text after decipher is %016lx\n",bin_data);

   if (bin_data != plainbinx) {
    printf("tpf_cryptc: Decipher text err6(BIN DATA)-TPF_TDES, text should be the same\n");
      err = ERROR;
   }


/****************************************************************/
/*  TPF_DES      (Character data)                               */
/****************************************************************/

    parms.tpf_crypt_in = buffer;
    parms.tpf_crypt_len = 16;
    parms.tpf_crypt_out = buffer;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = NULL;

/*  printf("** Encipher character data on DES **\n");*/
    rc = tpf_cryptc(TPF_ENC, TPF_DES, &parms);
/*  printf("rc from tpf_cryptc ENC, TPF_DES is : %i\n", rc);*/

/* Test that KEY wasn't changed on encipher */
/* printf("**** Before encipher, KEY is %s \n",key);*/
/* printf("**** After encipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Encipher KEY er13, key should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_key, key, 8)) {
      printf("tpf_cryptc: Encipher KEY er14, key should be the same\n");
      err = ERROR;
   }

/* Test that text was changed on encipher */
/* printf("** Text (character) after encipher %016lx\n",*/
/*         (long*)buffer);*/

   rc = memcmp(buffer, plaintxt, 16);
   if (rc == 0) {
printf("tpf_cryptc: Encipher text err7, text should not be the same\n");

         err = ERROR;
      }

/*  printf("** Decipher character data on DES **\n");*/

    rc = tpf_cryptc(TPF_DEC, TPF_DES, &parms);
/*  printf("rc from tpf_cryptc DEC, TPF_DES is : %i\n", rc);*/

/* Test that key wasn't changed on decipher */
/* printf("**** Before decipher, KEY is %s \n",key);              */
 /*printf("**** After decipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Decipher KEY err9  key should be the same\n");
      err = ERROR;
   }
   rc = memcmp(parms.tpf_crypt_key, key, 8);
   if (rc != 0) {
      printf("tpf_cryptc: Decipher KEY erra  key should be the same\n");
      err = ERROR;
   }

/* Test that text wasn't changed after encipher and decipher */
/* printf("**** Text after decipher is %s \n",buffer);*/

   rc = memcmp(buffer, plaintxt, 16);
   if (rc != 0) {
    printf("tpf_cryptc: Decipher text (CAHR DATA) TPF_DES err7, text should be the same\n");
      err = ERROR;
   }

/****************************************************************/
/*  TPF_DES     (Binary data)                                   */
/****************************************************************/

    memset(key, '\0', sizeof(key));
    memset(icv, '\0', sizeof(icv));
    strcpy(key, "abcdefgh");
    bin_data = plainbinx;

    parms.tpf_crypt_in = (char *)&bin_data;
    parms.tpf_crypt_len = 8;
    parms.tpf_crypt_out = (char *)&bin_data;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = NULL;

/*  printf("** Encipher binary data on DES **\n");*/
    rc = tpf_cryptc(TPF_ENC, TPF_DES, &parms);
/*  printf("rc from tpf_cryptc ENC, TPF_DES is : %i\n", rc);*/

/* Test that KEY wasn't changed on encipher */
/* printf("**** Before encipher, KEY is %s \n",key);              */
/* printf("**** After encipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Encipher KEY er15, key should be the same\n");
      err = ERROR;
   }
   if (memcmp(parms.tpf_crypt_key, key, 8)) {
      printf("tpf_cryptc: Encipher KEY er16, key should be the same\n");
      err = ERROR;
   }

/* Test that text was changed on encipher */

   if (bin_data == plainbinx) {
printf("tpf_cryptc: Encipher text err8, text should not be the same\n");
    err = ERROR;
   }

    parms.tpf_crypt_in = (char *)&bin_data;
    parms.tpf_crypt_len = 8;
    parms.tpf_crypt_out = (char *)&bin_data;
    parms.tpf_crypt_key = key;
    parms.tpf_crypt_icv = NULL;

/*  printf("** Decipher binary data on DES **\n");*/
    rc = tpf_cryptc(TPF_DEC, TPF_DES, &parms); /* Decryption */
/*  printf("rc from tpf_cryptc DEC is, TPF_DES: %i\n", rc);*/

/* Test that key wasn't changed on decipher */
/* printf("**** Before decipher, KEY is %s \n",key);              */
/* printf("**** After decipher KEY is %s \n",parms.tpf_crypt_key);*/

   if (parms.tpf_crypt_key != key ) {
      printf("tpf_cryptc: Decipher KEY errb, key should be the same\n");
      err = ERROR;
   }
   rc = memcmp(parms.tpf_crypt_key, key, 8);
   if (rc != 0) {
      printf("tpf_cryptc: Decipher KEY errc, key should be the same\n");
      err = ERROR;
   }

/* Test that text wasn't changed after encipher and decipher */
/* printf("** Text before encipher & decipher is %016lx\n",*/
 /*        plainbinx);*/
/* printf("** Text after decipher is %016lx\n",bin_data);*/

   if (bin_data != plainbinx) {
    printf("tpf_cryptc: Decipher text err8(BIN DATA) - TPF_DES, text should be the same\n");
      err = ERROR;
   }

   if (err)
      printf("tpf_cryptc:  exiting tpf_cryptc test, with errors\n");
   else
      printf("tpf_cryptc:  tpf_cryptc test completed without err\n");

} /* crypc_func: tpf_crypc() test */

}/* extern C */
