/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20060403  F14109  Initial version. Add tpf_stpoc to API5 driver.  */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name: qmac70.cpp                                          */
/*                                                                   */
/* Contents:     stpoc_func                                          */
/*                                                                   */
/* Abstract:     Member of the QMAC DLM which comprises the          */
/*               z/TPF API driver.                                   */
/*                                                                   */
/* Description:  Test tpf_stopc API using the TPF_STPOC_RESTORE      */
/*               parameter when the previously stored protection     */
/*               key is 0.                                           */
/*                                                                   */
/*********************************************************************/
#include <c_api5.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <tpf/sysapi.h>
#include <tpf/tpfapi.h>

#define  MSG_SIZE  255
char     msg[MSG_SIZE];

extern "C" {

/********************************************************************/
/* stpoc_func                                                       */
/********************************************************************/
void stpoc_func(void)
{

  typedef struct
  {
    int  testNum;
    char testText[30];
  } testCase;

  int frames, rc;
  char * token = "STPOC";
  testCase * testCase_p;
  char stpoc_save[9];

  prtmsg("stpoc: entering stpoc_func test\n");

  /******************************************************************/
  // if (NO)ERR is specified, process valid test case
  /******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    prtmsg("stpoc: processing valid case\n");

    frames = 2;

    /* get storage from system heap */
    testCase_p = (testCase *) tpf_gsysc(frames, token, NULL, GSYSC_1MB);
    if (testCase_p == 0)
    {
       prterr("stpoc: structure NOT ALLOCATED");
       exit(0);
    }
    else
    {
       /* force program to run in key 0 */
       cinfc(CINFC_WRITE,CINFC_CMMPAT);

       /* turn on storage protection override */
       tpf_stpoc(TPF_STPOC_ON, stpoc_save);

       /* initialize the structure in system heap */
       testCase_p->testNum = 1;
       strcpy(testCase_p->testText, "Test TPF_STPOC_RESTORE option with key 0");

       /* display structure contents stored in system heap */
       sprintf(msg,"stpoc: Test Case %d: %s \n", testCase_p->testNum, testCase_p->testText);
       prtmsg(msg);

       /* turn off storage protection override */
       tpf_stpoc(TPF_STPOC_RESTORE, stpoc_save);

       /* try writing to system heap again to ensure key 0 was restored */
       strcpy(testCase_p->testText, "Key 0 was successfully restored");
       sprintf(msg,"stpoc: Test Case %d: %s \n", testCase_p->testNum, testCase_p->testText);
       prtmsg(msg);

       /* restore protection key */
       keyrc();

       /* release storage from system heap */
       rc = tpf_rsysc((void *)testCase_p, frames, token);
       if (rc == RSYSC_ERROR)
          serrc_op(SERRC_RETURN, 0x2222, "ERROR RELEASING STORAGE", NULL);

       prtmsg("stpoc: exiting VALID CASE test\n");
    }
  }
  else

  /******************************************************************/
  //  if ERR specified print a message.
  /******************************************************************/
  {
    prtmsg("stpoc: No error test cases for tpf_stpoc\n");
  }
  prtmsg("stpoc: exiting stpoc_func test\n");
  return;
  } /* stpoc_func() */

}/* extern C */
