/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20070713  F15945  Initial version. Add drv_zdcrt_func() to        */
/*                   API5 driver.                                    */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name: qmac80.cpp                                          */
/*                                                                   */
/* Contents:     drv_zdcrt_func()                                    */
/*                                                                   */
/* Abstract:     Member of the QMAC DLM which comprises the          */
/*               z/TPF API driver.                                   */
/*                                                                   */
/* Description:  This program issues cretcs to QMAP in order to      */
/*               create entries in the CRET table                    */   
/*                                                                   */
/*********************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <tpf/sysapi.h>
#include <tpf/tpfapi.h>
#include <tpf/c_syseq.h>        /* For _CC1052 */ 
#include <qmac80.h>

#define  NUM_CRET_ENTRIES  4
boolean firstCRETentryFlag = TRUE;
boolean noMoreCRETentries = FALSE;
 
extern "C" 
{
  void set_cret_parms(int, struct cretc_parms *);
  void printCRETentries(struct cretc_parms *);
  
  void drv_zdcrt_func(void)
  {
    struct cretc_parms * cretcs[NUM_CRET_ENTRIES];
    char *system_state = 0;                  
    struct wtopc_header msg_hdr;
  
    /******************************************************************/
    // if (NO)ERR is specified, process valid test case
    /******************************************************************/
    if (ecbptr()->ebx000 == FALSE)
    {
      system_state = (char *) cinfc (CINFC_READ, CINFC_CMMSTI);
 
      for(int i=0; i<NUM_CRET_ENTRIES; i++)
      {
        if (!levtest(D1))
        {
          cretcs[i] = (struct cretc_parms *) getcc(D1,GETCC_TYPE,L0);
          gettimeofday(&(cretcs[i]->StartTime), NULL);
          set_cret_parms(i, cretcs[i]);

          //Check for 1052 state
          if ((*system_state == _CC1052)&&(cretcs[i]->state!=CRETC_1052))
          {
      		wtopc_insert_header(&msg_hdr, "QMAC", 10, 'E', WTOPC_SYS_TIME);
      		wtopc("CRETC ISSUED IN 1052 STATE WITHOUT CRETC_1052 PARAMETER",WTOPC_EBROUT, WTOPC_CHAIN_END, &msg_hdr);
   	        relcc(D1);   //release working storage block if not using it
          }
          else
          {   
            if(i==(NUM_CRET_ENTRIES-1))
              noMoreCRETentries = TRUE;
            printCRETentries(cretcs[i]);
            cretc_level((cretcs[i]->units + cretcs[i]->state),QMAP,cretcs[i]->desiredElapsedTime,"INIT",D1);
          	firstCRETentryFlag = FALSE;
          }  
        }
        else
        {
          wtopc_insert_header(&msg_hdr, "QMAC", 49, 'E', WTOPC_SYS_TIME);
    	  wtopc("ECB DATA LEVEL 1 OCCUPIED",WTOPC_EBROUT, WTOPC_CHAIN_END, &msg_hdr);

        }  
      }
    }
    else
    /******************************************************************/
    //  if ERR specified print a message.
    /******************************************************************/
    { 
      wtopc_insert_header(&msg_hdr, "QMAC", 77, 'E', WTOPC_SYS_TIME);
      wtopc("NO ERROR TEST CASES FOR CRET API\n", WTOPC_EBROUT, WTOPC_CHAIN_END, &msg_hdr);
    }
    return;
  } 
  
  void set_cret_parms(int i, struct cretc_parms * cretc_ptr)
  {
  	switch(i)
  	{
  	  case 0:
  	    cretc_ptr->desiredElapsedTime=45;
        cretc_ptr->units=CRETC_SECONDS;
        cretc_ptr->state=0;
        break;
      case 1:
        cretc_ptr->desiredElapsedTime=3;
        cretc_ptr->units=CRETC_MINUTES;
        cretc_ptr->state=0;
        break;
      case 2:
        cretc_ptr->desiredElapsedTime=75;
        cretc_ptr->units=CRETC_SECONDS;
        cretc_ptr->state=CRETC_1052;
        break;
      case 3:
        cretc_ptr->desiredElapsedTime=90;
        cretc_ptr->units=CRETC_SECONDS;
        cretc_ptr->state=CRETC_1052;   
  	}
  }
  
  void printCRETentries(struct cretc_parms * cretc_ptr)
  {
    char time_units[8];
    char msgbuffer[50];           // for printing messages
  	int units;
    struct wtopc_header msg_hdr;
  	
  	units = cretc_ptr->units;
    if(units==CRETC_SECONDS)
      sprintf(time_units,"seconds");

    if(units==CRETC_MINUTES)
      sprintf(time_units,"minutes");
      
    if(firstCRETentryFlag)
    {
      wtopc_insert_header(&msg_hdr, "QMAC", 44, 'I', WTOPC_SYS_TIME);
      wtopc("Creating CRET Table entries",WTOPC_EBROUT, WTOPC_CHAIN, &msg_hdr);
    }
  	
    sprintf(msgbuffer,"                   QMAP ECB should be created in %d %s", cretc_ptr->desiredElapsedTime,time_units);
    wtopc(msgbuffer, WTOPC_EBROUT, WTOPC_CHAIN, WTOPC_NO_HEADER);
    
    if(noMoreCRETentries)
      wtopc("                   *** END OF DISPLAY ***",WTOPC_EBROUT, WTOPC_CHAIN_END, WTOPC_NO_HEADER);
  }
}/* extern C */
