/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20070713  f15945  Initial version. Add pausc_func() to            */
/*                   API5 driver.                                    */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name: QMAC (qmac85.cpp)                                   */
/*                                                                   */
/* Contents:     pausc_func()                                        */
/*                                                                   */
/* Abstract:     Member of the QMAC DLM which comprises the          */
/*               z/TPF API driver. This program is used to test the  */
/*               PAUSC CTL-574 system error for CP User Enhancements */             
/*                                                                   */
/* Description:  This program swisc_creates QMAO() and then issues a */   
/*               PAUSC_BEGIN.  QMAO should not give up control and   */
/*               take the new 574 system error from CP User Enhance- */
/*               ments for case-1 and the existing 583 for case-2    */
/*********************************************************************/
#include <tpf/sysapi.h>
#include <tpf/tpfapi.h>
#include <tpf/c_dctist.h>
#include <stdlib.h>
#include <c_api5.h>

extern "C" 
{
  void pausc_func(void)
  {
    struct swisc_create_input swiscParameters;     // parameters for swisc_create  
    struct wtopc_header msg_hdr;
    struct ev0bk eb;
    api5parms *api5parms_p;

    /* Get the total active I-Streams defined on the system */ 
    struct dctist *ist = (struct dctist *) cinfc_fast(CINFC_CMMIST);

    unsigned short curIStream;      /* Current istream for use during driver invocation */
    unsigned short totalIStreams;   /* # of istreams active during IPL */

  	int rc;
  	int *caseNum;
  	
    memcpy(&(api5parms_p),&(ecbptr()->ebx008),sizeof(api5parms_p));
  	
    /******************************************************************/
    // if (NO)ERR is specified, process valid test case
    /******************************************************************/
    if (ecbptr()->ebx000 == FALSE)
    {  
 	  //Check that a valid test case number was specified
      if((api5parms_p->icase == 1) || (api5parms_p->icase == 2))
  	  {
   	    //Check that there is more than one i-stream available
  	    if((totalIStreams = ist->istactis)>1)
  	    {
  	      //Check that the command was issued from the main i-stream	
  	      //PAUSC can only be run on the main i-stream
          if((curIStream = ecbptr()->ce1isn) == 1)  	
          { 
             wtopc_insert_header(&msg_hdr, "QMAC", 65, 'I', WTOPC_SYS_TIME);
             wtopc("SWISC-ing QMAO ECB", WTOPC_EBROUT, WTOPC_CHAIN_END, &msg_hdr);
 
        	 swiscParameters.istream = 2;
             swiscParameters.cpu_list = SWISC_LIST_READY;   // Launch Immediately
             swiscParameters.bypass = SWISC_BYPASS_NO;      
             swiscParameters.program = "QMAO";              
             swiscParameters.ebw_data = NULL;
             swiscParameters.ebw_data_length = 0;
             swiscParameters.data_level = D2; 

             //Pass test case number to QMAO
             caseNum = (int *) getcc(D2, GETCC_TYPE, L0);
             *caseNum = api5parms_p->icase;
             swisc_create(&swiscParameters);                // Launch new ECB

             eb.evnpstinf.evnbkc1 = 1;   //Only one postc required to satisfy the event
             strcpy (eb.evnbkn, "pauscOK");
   
             //Create the event
             rc = evntc(&eb, EVENT_CNT, 'Y', 15, EVNTC_NORM);
             if (rc)
             {
               wtopc_insert_header(&msg_hdr, "QMAC", 31, 'E', WTOPC_SYS_TIME);
               wtopc("evntc() ERROR: THE EVENT NAME ALREADY EXISTS", WTOPC_EBROUT, WTOPC_CHAIN_END, &msg_hdr);
               return;
             }
       
             //Now wait on it
             rc = evnwc(&eb, EVENT_CNT);
             if (rc) 
             {
               wtopc_insert_header(&msg_hdr, "QMAC", 32, 'E', WTOPC_SYS_TIME);
               wtopc("evnwc() ERROR. EXITS FROM HERE", WTOPC_EBROUT, WTOPC_CHAIN_END, &msg_hdr);
               return;
             }
             
             //After the postc from QMAO
             pausc(PAUSC_BEGIN);
           }  
           else
           {
             wtopc_insert_header(&msg_hdr, "QMAC", 80, 'E', WTOPC_SYS_TIME);
             wtopc("COMMAND MUST BE ISSUED FROM THE MAIN I-STREAM", WTOPC_EBROUT, WTOPC_CHAIN_END, &msg_hdr);
           }
  	    }   
        else
        {
          wtopc_insert_header(&msg_hdr, "QMAC", 42, 'E', WTOPC_SYS_TIME);
          wtopc("SYSTEM HAS ONLY ONE I-STREAM...EXITING", WTOPC_EBROUT, WTOPC_CHAIN_END, &msg_hdr);
        } 
  	  }
  	  else  
      {
      	wtopc_insert_header(&msg_hdr, "QMAC", 88, 'E', WTOPC_SYS_TIME);
        wtopc("INVALID PAUSC TEST CASE NUMBER", WTOPC_EBROUT, WTOPC_CHAIN_END, &msg_hdr);
  	  }
    }  	
    else
    /******************************************************************/
    //  if ERR specified print a message.
    /******************************************************************/
    { 
      wtopc_insert_header(&msg_hdr, "QMAC", 78, 'E', WTOPC_SYS_TIME);
      wtopc("NO ERROR TEST CASES FOR PAUSC API", WTOPC_EBROUT, WTOPC_CHAIN_END, &msg_hdr);
    }
    
    exit(0);
  } 
}/* extern C */
