/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2005                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20041001  Initial Version                                         */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name:  qmac90.cpp                                         */
/*                                                                   */
/* Abstract:      Member of the QMAC DLM which comprises the         */
/*                z/TPF API driver.                                  */
/*                                                                   */
/* Description:   This segment contains the following test cases:    */
/*                - swisc_enter                                      */
/*                - tpf_genlc                                        */
/*                                                                   */
/*********************************************************************/
#include <c_api5.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <tpf/tpfapi.h>
#include <tpf/sysapi.h>
#include <tpf/c_eb0eb.h>
#include <tpf/c_dctist.h>
#include <tpf/c_pi1dt.h> /* genlc_func() */
#include <errno.h>


extern "C" {

void entrc_func(void)
{
  prtmsg("entrc: Entering entrc_func test.\n");

  /*******************************************************************/
  /* if (NO)ERR s specified...                                       */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    prtmsg("entrc: Processing entrc_func normal cases.\n");

    /*****************************************************************/
    /*****************************************************************/
    /* Test Case 1:      Create an ECB to test swisc_enter           */
    /* Test Result:      Normal exit with no errors.                 */
    /* Error Msg Output: None                                        */
    /*****************************************************************/
    {
      struct tpf_cresc_input param;
      char *prog = "QMAH";

      struct {
        char verbose;
        short int iStream;
        unsigned short isTotal;
        char data;
      } args;

      struct dctist *istdata;

      prtmsg("entrc: 1-Create ECB for swisc_enter test.\n");

      istdata = (struct dctist *) cinfc(CINFC_READ, CINFC_CMMIST);

      /* Populate ECB arguments */
      args.verbose = ecbptr()->ebcm01;
      args.iStream = 0;
      args.isTotal = istdata->istactis;
      args.data = 'T';

      /* Populate the parameters for creating the ECB                */
      param.program = prog;
      param.istream = 1;
      param.timeout = 0;
      param.wait = TPF_CRESC_WAIT_YES;
      param.data = &args;
      param.data_length = sizeof(args);

      /* Create the ECB and wait                                     */
      tpf_cresc(&param);

      /* Don't care about the child ECB return code                  */
    }
  }

  /*******************************************************************/
  /* if ERR is specified...                                          */
  /*******************************************************************/
  else
  {
    prtmsg("entrc: No entrc_func error cases.\n");
  }

  prtmsg("entrc: Completed entrc_func test.\n");
  return;
}


/*********************************************************************/
/*********************************************************************/
/* Function Name:   genlc_func                                       */
/* Number of Cases: 7 Normal Cases                                   */
/*                  0 Error Cases                                    */
/*********************************************************************/
void genlc_func(void)
{
  struct pi1dt *pidt = NULL;     /* processor id table               */
  short int pinocp;              /* number of genned processors      */
  short int picidn;              /* processor ordinal number         */
  unsigned char piapc;           /* number of active processors      */

  char buffer[256];              /* output buffer                    */

  int i, j;                      /* junk incrementors                */


  prtmsg("genlc: Entering genlc_func test.\n");


  /*******************************************************************/
  /* Get the number of processors in the complex.                    */
  /*******************************************************************/
  pidt = (struct pi1dt *) cinfc_fast(CINFC_CMMPID);
  pinocp = pidt->pi1pids.pi1nocp;
  picidn = pidt->pi1pids.pi1cidn;
  piapc = pidt->pi1pids.pi1apc;


  sprintf(buffer, "genlc: Currently running on processor %d.\n",
          picidn);
  prtmsg(buffer);

  sprintf(buffer, "genlc: There are %d active processors.\n", piapc);
  prtmsg(buffer);

  sprintf(buffer, "genlc: There are %d processors in the complex.\n",
          pinocp);
  prtmsg(buffer);


  /*******************************************************************/
  /* if (NO)ERR is specified...                                      */
  /*******************************************************************/
  if (ecbptr()->ebx000 == FALSE)
  {
    prtmsg("genlc: Processing genlc_func normal cases.\n");


    /*****************************************************************/
    /*****************************************************************/
    /* Test Case 1:      Generate event data list using ALL          */
    /*                   processor ordinal numbers.                  */
    /* Test Result:      Normal exit with no errors.                 */
    /* Error Msg Output: None                                        */
    /*****************************************************************/
    {
      struct tpf_ev0bk_list_data *event_blk = NULL;

      prtmsg("genlc: 1-Generate event list with all proc ords.\n");

      /* Allocate space for the event data list.                     */
      event_blk = (struct tpf_ev0bk_list_data *)
        getcc(D1, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L4, 0x00);

      if (levtest(D1))
      {
        /*************************************************************/
        /* Fill the event data list with all processor ordinal       */
        /* ordinal numbers, except the current.                      */
        /*************************************************************/
        prtmsg("genlc: 1-Creating data list with all proc ords.\n");
        tpf_genlc(event_blk, TPF_GENLC_BLOCK, TPF_GENLC_CREATE,
                  TPF_GENLC_PROCESSOR, TPF_GENLC_INCLUDE_ALL,
                  0, 0, 0, 0);

        sprintf(buffer, "genlc: 1-Data list contains %d items.\n",
                event_blk->evnbklc);
        prtmsg(buffer);

        /*************************************************************/
        /* Verify the data list                                      */
        /*************************************************************/
        if (event_blk->evnbklc == (pinocp - 1))
        {
          /* Locate the Current Processor in the data list           */
          j = 0;
          for (i = 0; i < event_blk->evnbklc; i++)
          {
            if (TPF_EVENT_LIST_ITEM(event_blk, i)->evnbklid == picidn)
            {
              j++;
            }
          }

          if (j == 0)
          {
            prtmsg("genlc: 1-Successfully Completed.\n");
          }
          else
          {
            prterr("genlc: 1-Current processor in event data list.");
          }
        }
        else
        {
          prterr("genlc: 1-Event data list size incorrect.");
        }

        /* Release the working storage block on Data Level D1.       */
        relcc(D1);
      }
      else
      {
        prterr("genlc: 1-getcc() failed - not a genlc error.");
      }
    }


    /*****************************************************************/
    /*****************************************************************/
    /* Test Case 2:      Generate evemt data list using ACTIVE       */
    /*                   processor ordinal numbers.                  */
    /* Test Result:      Normal exit with no errors.                 */
    /* Error Msg Output: None                                        */
    /*****************************************************************/
    {
      struct tpf_ev0bk_list_data *event_blk = NULL;

      prtmsg("genlc: 2-Generate event list with active proc ords.\n");

      /* Allocate space for the event data list.                     */
      event_blk = (struct tpf_ev0bk_list_data *)
        getcc(D1, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L4, 0x00);

      if (levtest(D1))
      {
        /*************************************************************/
        /* Fill the event data list with active processor            */
        /* ordinal numbers, except the current.                      */
        /*************************************************************/
        prtmsg("genlc: 2-Creating data list with active proc ords.\n");
        tpf_genlc(event_blk, TPF_GENLC_BLOCK, TPF_GENLC_CREATE,
                  TPF_GENLC_PROCESSOR, TPF_GENLC_INCLUDE_ACTIVE,
                  0, 0, 0, 0);

        sprintf(buffer, "genlc: 2-Data list contains %d items.\n",
                event_blk->evnbklc);
        prtmsg(buffer);

        /*************************************************************/
        /* Verify the data list                                      */
        /*************************************************************/
        if (event_blk->evnbklc == (piapc - 1))
        {
          j = 0;
          for (i = 0; i < event_blk->evnbklc; i++)
          {
            if (TPF_EVENT_LIST_ITEM(event_blk, i)->evnbklid == picidn)
            {
              j++;
            }
          }

          if (j == 0)
          {
            prtmsg("genlc: 2-Successfully Completed.\n");
          }
          else
          {
            prterr("genlc: 2-Current processor in event data list.");
          }
        }
        else
        {
          prterr("genlc: 2-Event data list size incorrect.");
        }

        /* Release the working storage block on Data Level D1.       */
        relcc(D1);
      }
      else
      {
        prterr("genlc: 2-getcc() failed - not a genlc error.");
      }
    }

    /*****************************************************************/
    /*****************************************************************/
    /* Test Case 3:      Generate event data list using INACTIVE     */
    /*                   processor ordinal numbers.                  */
    /* Test Result:      Normal exit with no errors.                 */
    /* Error Msg Output: None                                        */
    /*****************************************************************/
    {
      struct tpf_ev0bk_list_data *event_blk = NULL;

      prtmsg("genlc: 3-Generate event list with inactive "
             "proc ords.\n");

      /* Allocate space for the event data list.                     */
      event_blk = (struct tpf_ev0bk_list_data *)
        getcc(D1, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L4, 0x00);

      if (levtest(D1))
      {
        /*************************************************************/
        /* Fill the event data list with inactive processor          */
        /* ordinal numbers, except the current.                      */
        /*************************************************************/
        prtmsg("genlc: 3-Creating list with inactive proc ords.\n");
        tpf_genlc(event_blk, TPF_GENLC_BLOCK, TPF_GENLC_CREATE,
                  TPF_GENLC_PROCESSOR, TPF_GENLC_INCLUDE_INACTIVE,
                  0, 0, 0, 0);

        sprintf(buffer, "genlc: 3-Data list contains %d items.\n",
                event_blk->evnbklc);
        prtmsg(buffer);

        /*************************************************************/
        /* Verify the data list                                      */
        /*************************************************************/
        if (event_blk->evnbklc == (pinocp - piapc))
        {
          j = 0;
          for (i = 0; i < event_blk->evnbklc; i++)
          {
            if (TPF_EVENT_LIST_ITEM(event_blk, i)->evnbklid == picidn)
            {
              j++;
            }
          }

          if (j == 0)
          {
            prtmsg("genlc: 3-Successfully Completed.\n");
          }
          else
          {
            prterr("genlc: 3-Current processor in event data list.");
          }
        }
        else
        {
          prterr("genlc: 3-Event data list size incorrect.");
        }

        /* Release the working storage block on Data Level D1.       */
        relcc(D1);
      }
      else
      {
        prterr("genlc: 3-getcc() failed - not a genlc error.");
      }
    }


    /*****************************************************************/
    /*****************************************************************/
    /* Test Case 4:      Generate event data list by adding the      */
    /*                   ACTIVE and INACTIVE processor ordinals.     */
    /* Test Result:      Normal exit with no errors.                 */
    /* Error Msg Output: None                                        */
    /*****************************************************************/
    {
      struct tpf_ev0bk_list_data *event_blk = NULL;

      prtmsg("genlc: 4-Add active and inactive proc ords to list.\n");

      /* Allocate space for the event data list.                     */
      event_blk = (struct tpf_ev0bk_list_data *)
        getcc(D1, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L4, 0x00);

      if (levtest(D1))
      {
        /*************************************************************/
        /* Fill the event data list with active processor            */
        /* ordinal numbers, except the current.                      */
        /*************************************************************/
        prtmsg("genlc: 4-Creating data list with active proc ords.\n");
        tpf_genlc(event_blk, TPF_GENLC_BLOCK, TPF_GENLC_CREATE,
                  TPF_GENLC_PROCESSOR, TPF_GENLC_INCLUDE_ACTIVE,
                  0, 0, 0, 0);

        /*************************************************************/
        /* Fill the event data list with inactive processor          */
        /* ordinal numbers, except the current.                      */
        /*************************************************************/
        prtmsg("genlc: 4-Adding inactive proc ords to list.\n");
        tpf_genlc(event_blk, TPF_GENLC_BLOCK, TPF_GENLC_MODIFY,
                  TPF_GENLC_PROCESSOR, TPF_GENLC_INCLUDE_INACTIVE,
                  0, 0, 0, 0);

        sprintf(buffer, "genlc: 4-Data list contains %d items.\n",
                event_blk->evnbklc);
        prtmsg(buffer);

        /*************************************************************/
        /* Verify the data list                                      */
        /*************************************************************/
        if (event_blk->evnbklc == (pinocp - 1))
        {
          /* Locate the Current Processor in the data list           */
          j = 0;
          for (i = 0; i < event_blk->evnbklc; i++)
          {
            if (TPF_EVENT_LIST_ITEM(event_blk, i)->evnbklid == picidn)
            {
              j++;
            }
          }

          if (j == 0)
          {
            prtmsg("genlc: 4-Successfully Completed.\n");
          }
          else
          {
            prterr("genlc: 4-Current processor in event data list.");
          }
        }
        else
        {
          prterr("genlc: 4-Event data list size incorrect.");
        }

        /* Release the working storage block on Data Level D1.       */
        relcc(D1);
      }
      else
      {
        prterr("genlc: 4-getcc() failed - not a genlc error.");
      }
    }


    /*****************************************************************/
    /*****************************************************************/
    /* Test Case 5:      Generate event data list by removing all    */
    /*                   active and inactive processor ordinals from */
    /*                   a existing data list.                       */
    /* Test Result:      Normal exit with no errors.                 */
    /* Error Msg Output: None                                        */
    /*****************************************************************/
    {
      struct tpf_ev0bk_list_data *event_blk = NULL;

      prtmsg("genlc: 5-Remove all proc ords from data list.\n");

      /* Allocate space for the event data list.                     */
      event_blk = (struct tpf_ev0bk_list_data *)
        getcc(D1, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L4, 0x00);

      if (levtest(D1))
      {
        /*************************************************************/
        /* Fill the event data list with all processor               */
        /* ordinal numbers, except the current.                      */
        /*************************************************************/
        prtmsg("genlc: 5-Creating data list with all proc ords.\n");
        tpf_genlc(event_blk, TPF_GENLC_BLOCK, TPF_GENLC_CREATE,
                  TPF_GENLC_PROCESSOR, TPF_GENLC_INCLUDE_ALL,
                  0, 0, 0, 0);

        /*************************************************************/
        /* Modify the event data list by removing the active         */
        /* processor ordinal numbers, except the current.            */
        /*************************************************************/
        prtmsg("genlc: 5-Removing active proc ords to data list.\n");
        tpf_genlc(event_blk, TPF_GENLC_BLOCK, TPF_GENLC_MODIFY,
                  TPF_GENLC_PROCESSOR, TPF_GENLC_EXCLUDE_ACTIVE,
                  0, 0, 0, 0);

        /*************************************************************/
        /* Modify the event data list by removing the inactive       */
        /* processor ordinal numbers, except the current.            */
        /*************************************************************/
        prtmsg("genlc: 5-Removing inactive proc ords to data list.\n");
        tpf_genlc(event_blk, TPF_GENLC_BLOCK, TPF_GENLC_MODIFY,
                  TPF_GENLC_PROCESSOR, TPF_GENLC_EXCLUDE_INACTIVE,
                  0, 0, 0, 0);

        sprintf(buffer, "genlc: 5-Data list contains %d items.\n",
                event_blk->evnbklc);
        prtmsg(buffer);

        /*************************************************************/
        /* Verify the data list                                      */
        /*************************************************************/
        if (event_blk->evnbklc == 0)
        {
          prtmsg("genlc: 5-Successfully Completed.\n");
        }
        else
        {
          prterr("genlc: 5-Event data list size incorrect.");
        }

        /* Release the working storage block on Data Level D1.       */
        relcc(D1);
      }
      else
      {
        prterr("genlc: 5-getcc() failed - not a genlc error.");
      }
    }


    /*****************************************************************/
    /*****************************************************************/
    /* Test Case 6:      Generate a user-defined data list using ALL */
    /*                   processor ordinal numbers.                  */
    /* Test Result:      Normal exit with no errors.                 */
    /* Error Msg Output: None                                        */
    /*****************************************************************/
    {
      struct userlist
      {
        unsigned short count;
        unsigned short size;
        tpf_ev0bk_list_item items[1];
      } *event_blk = NULL;

      prtmsg("genlc: 6-Generate user data list with all proc ords.\n");

      /* Allocate space for the event data list.                     */
      event_blk = (struct userlist *)
        getcc(D1, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L4, 0x00);

      if (levtest(D1))
      {
        /*************************************************************/
        /* Fill the event data list with all processor ordinal       */
        /* ordinal numbers, except the current.                      */
        /*************************************************************/
        tpf_genlc(event_blk, TPF_GENLC_AREA, TPF_GENLC_CREATE,
                  TPF_GENLC_PROCESSOR, TPF_GENLC_INCLUDE_ALL,
                  0, 0, 0, 0);

        /*************************************************************/
        /* Verify the data list                                      */
        /*************************************************************/
        if (event_blk->count == (pinocp - 1))
        {
          j = 0;
          for (i = 0; i < event_blk->count; i++)
          {
            if (event_blk->items[i].evnbklid == picidn)
            {
              j++;
            }
          }

          if (j == 0)
          {
            prtmsg("genlc: 6-Successfully Completed.\n");
          }
          else
          {
            prterr("genlc: 6-Current processor in data list.");
          }
        }
        else
        {
          prterr("genlc: 6-Event data list size incorrect.");
        }

        /* Release the working storage block on Data Level D1.       */
        relcc(D1);
      }
      else
      {
        prterr("genlc: 6-getcc() failed - not a genlc error.");
      }
    }


    /*****************************************************************/
    /*****************************************************************/
    /* Test Case 7:      Generate data list using user-defined       */
    /*                   input data.                                 */
    /* Test Result:      Normal exit with no errors.                 */
    /* Error Msg Output: None                                        */
    /*****************************************************************/
    {
      struct
      {
        unsigned char item[10];
      } input_data;

      struct userlist
      {
        unsigned short count;
        unsigned short size;
        tpf_ev0bk_list_item items[10];
      } *event_blk = NULL;

      prtmsg("genlc: 7-Generate data list with user data.\n");

      /* The input data                                              */
      input_data.item[0] = 1;
      input_data.item[1] = 2;
      input_data.item[2] = 3;

      /* Allocate space for the data list.                     */
      event_blk = (struct userlist *)
        getcc(D1, (enum t_getfmt) (GETCC_TYPE+GETCC_FILL), L4, 0x00);

      if (levtest(D1))
      {
        /*************************************************************/
        /* Fill the data list with the user-defined data.            */
        /* (3 data items, of size 1-byte)                            */
        /*************************************************************/
        tpf_genlc(event_blk, TPF_GENLC_AREA, TPF_GENLC_CREATE,
                  TPF_GENLC_POINTER, TPF_GENLC_INCLUDE_ALL,
                  &input_data, 3, 1, 0);

        /*************************************************************/
        /* Verify the data list                                      */
        /*************************************************************/
        if (event_blk->count == 3)
        {
          if ((event_blk->items[0].evnbklid == 1) &&
              (event_blk->items[1].evnbklid == 2) &&
              (event_blk->items[2].evnbklid == 3))
          {
            prtmsg("genlc: 7-Successfully Completed.\n");
          }
          else
          {
            prterr("genlc: 7-Data list does not match input.");
          }
        }
        else
        {
          prterr("genlc: 7-Data list size is incorrect.");
        }

        /* Release the working storage block on Data Level D1.       */
        relcc(D1);
      }
      else
      {
        prterr("genlc: 7-getcc() failed - not a genlc error.");
      }
    }

    /*****************************************************************/
    /* End of Normal Tests                                           */
    /*****************************************************************/
  }

  /*******************************************************************/
  /* if ERR is specified...                                          */
  /*******************************************************************/
  else
  {
    prtmsg("genlc: No genlc_func error cases.\n");
  }

  prtmsg("genlc: Completed genlc_func test.\n");
  return;
}

}
