/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2005                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20041001  Initial Version                                         */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name:  QMAD.cpp                                           */
/*                                                                   */
/* Abstract:      This program is used for error dump test cases of  */
/*                the following functions:                           */
/*                  attac - attac_func():     Test Case #0           */
/*                  crosc - crosc_func():     Test Case #1           */
/*                  freec - freec_func():     Test Case #2           */
/*                  crexc - crexc_func():     Test Case #3           */
/*                  cremc - cremc_func():     Test Case #4           */
/*                  unhka - unhka_func():     Test Case #5           */
/*                                                                   */
/*********************************************************************/
#include <c_api5.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <tpf/tpfapi.h>
#include <tpf/c_eb0eb.h>
#include <tpf/sysapi.h>               /* for crosc_entrc()           */
#include <tpf/tpfio.h>                /* for FACE and FACS programs  */
#include <tpf/c_uatbc.h>              /* for uatbc                   */
#include <tpf/c_ms0ut.h>              /* used in crosc cases         */

extern "C" void QMAD(){
  int parm[3];                        /* array holds 3 parameters    */
  parm[0] = ((int)(ecbptr()->ebw000));/* holds the function number   */
  parm[1] = ((int)(ecbptr()->ebw001));/* holds the test case number  */
  parm[2] = ((int)(ecbptr()->ebw002));/* holds 'print'/'noprint' info*/

  char  macro_tested[] = "CREXC";   /*                       - crexc */
  int freec_rc = 0x11111111;        /* rc out of heap        - freec */
  struct TPF_regs regs;             /* holds parameter list  - crosc */
  unsigned short    save_dbi;       /* saves ECB dbi setting - crosc */
  unsigned short    save_ssu;       /* saves ECB ssu setting - crosc */
  unsigned short    new_dbi;        /* subsystem to cross to - crosc */
  struct   uatbc_area uatbc_parm;   /*                       - crosc */
  struct   ms0ut    *ssu_slot_ptr;  /*                       - crosc */
  unsigned int      ss_ord = 0;     /* subsystem oridnal #   - crosc */
  char     save_area[]= "GARBAGE "; /* initialization        - unhka */
  char buffer[80];                  /* buffer for prtmsg()   - unhka */
  ecbptr()->ebcm01 = parm[2];       /* sets the print bit to on/off  */

  /*******************************************************************/
  /* Find out what function is being tested                          */
  /*******************************************************************/
  switch(parm[0])
  {
    /*****************************************************************/
    /* Function '0':  attac_func()                                   */
    /*****************************************************************/
    case '0':
      /***************************************************************/
      /* Find out what test case is being tested                     */
      /***************************************************************/
      switch(parm[1])
      {
        /*************************************************************/
        /* Test Case 1:       No storage block was previously        */
        /*                    detached on the referenced level.      */
        /* Test Result:       System error with exit.                */
        /* Error Msg Output:  "ATTAC - SPECIFIED LEVEL HOLDING BLOCK"*/
        /*************************************************************/
        case '1':
          prtmsg("attac: 1-Attaching Datalevel D3 without previously having detached it.\n");
          getcc(D3, (enum t_getfmt)(GETCC_TYPE), L2);

          if(levtest(D3))          /* if the DataLevel is occupied.. */
          {
            prtmsg("attac: 1-Should get a CTL-D1 soon.\n");
            attac(D3);
            prterr("attac: 1-Test Failed, D3 was able to be attached.");

            /* Release the working storage block on Data Level D3.   */
            relcc(D3);
          }
          else
            prterr("attac: 1-getcc() failed - not an attac error.");
        break;

        /*************************************************************/
        /* Test Case 2:       There is already a block on the        */
        /*                    specified level.                       */
        /* Test Result:       System error with exit.                */
        /* Error Msg Output:  "ATTAC - SPECIFIED LEVEL HOLDING BLOCK"*/
        /*************************************************************/
        case '2':
          prtmsg("attac: 2-Attaching Datalevel D4 after it had already been attached.\n");
          getcc(D4, (enum t_getfmt)(GETCC_TYPE), L2);

          if(levtest(D4))          /* if the DataLevel is occupied.. */
          {
            prtmsg("attac: 2-Should get a CTL-D1 soon.\n");
            detac(D4);
            if(!levtest(D4))
            {
              attac(D4);
              attac(D4);
              prterr("attac: 2-Test Failed, D4 was able to be attached.");
            }
            else
              prterr("attac: 2-Unable to detac D4 - not an attac error.");

            /* Release the working storage block on Data Level D4.   */
            relcc(D4);
          }
          else
            prterr("attac: 2-getcc() failed - not an attac error.");
        break;

        /*************************************************************/
        /* Test Case 3:       The block has not been detached yet.   */
        /*                    A different block is trying to be      */
        /*                    attached than was previously detached. */
        /* Test Result:       System error with exit.                */
        /* Error Msg Output:  "ATTAC - NO BLOCKS DETACHED FROM LEVEL"*/
        /*************************************************************/
        case '3':
          prtmsg("attac: 3-Attaching Datalevel D6 after Datalevel D5 has been detached.\n");
          getcc(D5, (enum t_getfmt)(GETCC_TYPE), L2);

          if(levtest(D5))          /* if the DataLevel is occupied.. */
          {
            prtmsg("attac: 3-Should get a CTL-D1 soon.\n");
            detac(D5);
            if(!levtest(D5))
            {
              attac(D6);
              prterr("attac: 3-Test Failed, D6 was able to be attached.");
            }
            else
              prterr("attac: 3-Unable to detac D5 - not an attac error.");

            /* Release the working storage block on Data Level D5.   */
            relcc(D5);
          }
          else
            prterr("attac: 3-getcc() failed - not an attac error.");
        break;

        /*************************************************************/
        /* Test Case 4:       Block was attached without having      */
        /*                    previously made a call to getcc() and  */
        /*                    without having previously detached it. */
        /* Test Result:       System error with exit.                */
        /* Error Msg Output:  "ATTAC - NO BLOCKS DETACHED FROM ECB"  */
        /*************************************************************/
        case '4':
          prtmsg("attac: 4-Attaching Datalevel D7 without having previously detached it.\n");

          prtmsg("attac: 4-Should get a CTL-D1 soon.\n");
          attac(D7);
          prterr("attac: 4-Test Failed, D7 was able to be attached.");
        break;

        /*************************************************************/
        /* Test Case 5:       Attach while holding a core block.     */
        /* Test Result:       System error with exit.                */
        /* Error Msg Output:  "ATTAC - SPECIFIED LEVEL HOLDING BLOCK"*/
        /*************************************************************/
        case '5':
          prtmsg("attac: 5-Attempting to Attach DataLevel D8 while holding core block.\n");
          getcc(D8, (enum t_getfmt)(GETCC_TYPE), L2);

          if(levtest(D8))          /* if the DataLevel is occupied.. */
          {
            prtmsg("attac: 5-Should get a CTL-D1 soon.\n");
            detac(D8);
            if(!levtest(D8))
            {
              getcc(D8, (enum t_getfmt)(GETCC_TYPE), L2);
              attac(D8);
              prterr("attac: 5-Test Failed, D8 was able to be attached.");
            }
            else
              prterr("attac: 5-Unable to detac D8 - not an attac error.");

            /* Release the working storage block on Data Level D8.   */
            relcc(D8);
          }
          else
            prterr("attac: 5-getcc() failed - not an attac error.");
        break;

        /*************************************************************/
        /* Default Case:                                             */
        /*************************************************************/
        default:
          prterr("attac: An invalid function case was entered.");
        break;
      }/*end test case switch                                        */
    break;

    /*****************************************************************/
    /* Function '1':  crosc_func()                                   */
    /*****************************************************************/
    case '1':
      /***************************************************************/
      /* Save original DBI and SSU.  Even though the DBI and SSU are */
      /* saved, you will still need to go through cebic_save() so the*/
      /* reference to globals will be correct.                       */
      /***************************************************************/
      save_dbi = ecbptr()->ce1dbi;
      save_ssu = ecbptr()->ce1ssu;

      uatbc_parm.uat_call = UAT_RSSI;
      uatbc_parm.uat_parm[0] = (char)ss_ord;

      if (uatbc(&uatbc_parm) == UAT_SUCCESS)
      {
        ss_ord++;                        /* go to the next subsystem */
        uatbc_parm.uat_parm[0] = (char)ss_ord;

        /*************************************************************/
        /* If there is more than one subsystem running, alert the    */
        /* user that crosc_entrc() will cross subsystems to enter    */
        /* the program... otherwise, crosc_entrc() will simply enter */
        /* the program in the current subsystem.                     */
        /*************************************************************/
        if(uatbc(&uatbc_parm) == UAT_SUCCESS)
        {
          prtmsg("crosc: There are at least 2 subsystems running.\n");
          prtmsg("crosc: crosc_entrc() will enter program in a different subsystem.\n");
        }
        else
        {
          prtmsg("crosc: There is only 1 subsystem running.\n");
          prtmsg("crosc: crosc_entrc() will enter program in the current subsystem\n");
        }

        /* pointer to subsystem user slot                            */
        ssu_slot_ptr = (ms0ut*) uatbc_parm.uat_ssu_ptr;
        new_dbi = *(unsigned short *) ssu_slot_ptr->mu0dpi;

        /*************************************************************/
        /* Find out what test case is being tested                   */
        /*************************************************************/
        switch(parm[1])
        {
          /***********************************************************/
          /* Test Case 1:       crosc to FACE program on a different */
          /*                    subsystem.                           */
          /* Test Result:       Ctl-66 dump                          */
          /* Error Msg Output:  None                                 */
          /***********************************************************/
          case '1':
            if(uatbc(&uatbc_parm) == UAT_SUCCESS)
              prtmsg("crosc: 1-Enter/Return from FACE program in different SS.\n");
            else
              prtmsg("crosc: 1-Enter/Return from FACE program in current SS.\n");
            regs.r0 = 235;
            regs.r6 = (long)"#PROG1  ";
            regs.r7 = (long)&ecbptr()->ce1fa6;

            /*********************************************************/
            /* crosc to FACE program results in a system dump with   */
            /* exit                                                  */
            /*********************************************************/
            prtmsg("crosc: 1-Should get a CTL-64905 soon.\n");

            cebic_save();
            cebic_goto_dbi(new_dbi);
            crosc_entrc("FACE", CROSC_DBI, &regs);
            cebic_restore();

            prterr("crosc: 1-Test Failed, crosc_entrc was able to enter the FACE program.");
          break;

          /***********************************************************/
          /* Test Case 2:       crosc to FACS program on a different */
          /*                    different subsystem.                 */
          /* Test Result:       Ctl-66 dump                          */
          /* Error Msg Output:  None                                 */
          /***********************************************************/
          case '2':
            if(uatbc(&uatbc_parm) == UAT_SUCCESS)
              prtmsg("crosc: 2-Enter/Return from FACS program in different SS.\n");
            else
              prtmsg("crosc: 2-Enter/Return from FACS program in current SS.\n");
            regs.r0 = 235;
            regs.r6 = (long)"#PROG1  ";
            regs.r7 = (long)&ecbptr()->ce1fa6;

            /*******************************************************/
            /* crosc to FACS program results in a system dump with */
            /* exit                                                */
            /*******************************************************/
            prtmsg("crosc: 2-Should get a CTL-64905 soon.\n");

            cebic_save();
            cebic_goto_dbi(new_dbi);
            crosc_entrc("FACS", CROSC_DBI, &regs);
            cebic_restore();

            prterr("crosc: 2-Test Failed, crosc_entrc was able to enter the FACS program.");
          break;

          /***********************************************************/
          /* Other Possible Test Cases:                              */
          /*                                                         */
          /* Test Case x:       crosc to a program in a subsystem    */
          /*                    that does not exist.                 */
          /* Test Result:       OPR-000230 or 000231 dump with exit  */
          /*                                                         */
          /*                                                         */
          /* Test Case y:       crosc to a program in a subsystem    */
          /*                    that is not genned.                  */
          /* Test Result:       OPR-000230 or 000231 dump with exit  */
          /*                                                         */
          /*                                                         */
          /* Test Case z:       crosc to a program in a subsystem    */
          /*                    that is not active.                  */
          /* Test Result:       OPR-000230 or 000231 dump with exit  */
          /***********************************************************/

          /*********************************************************/
          /* Default Case:                                         */
          /*********************************************************/
          default:
            prterr("crosc: An invalid function case was entered.");
          break;
        }/*end switch                                              */
      }
      else
        prterr("crosc: There are no subsystems active.");
    break;/* end of crosc_func() case                                */

    /*****************************************************************/
    /* Function '2':  freec_func()                                   */
    /*****************************************************************/
    case '2':
      /***************************************************************/
      /* Find out what test case is being tested                     */
      /***************************************************************/
      switch(parm[1])
      {
        /*************************************************************/
        /* Test Case 1:       Attempt to release storage with a      */
        /*                    block address that is not in the       */
        /*                    system heap.                           */
        /* Test Result:       Ctl-6F1 dump                           */
        /* Error Msg Output:  FREEC - BLOCK ADDRESS NOT IN HEAP AREA */
        /*************************************************************/
        case '1':
          prtmsg("freec: 1-Attempting to release invalid ECB heap address.\n");
          prtmsg("freec: 1-Should get a CTL-6F1 soon.\n");
          free((void *)(long) freec_rc);         /* rc is not in heap area */

          prterr("freec: 1-Test Failed, freec_entrc was able to free memory.");
        break;
        /*************************************************************/
        /* Default Case:                                             */
        /*************************************************************/
        default:
          prterr("freec: An invalid function case was entered.");
        break;
      }/*end switch                                                  */
    break;/* end of freec_func() case                                */


    /*****************************************************************/
    /* Function '3':  crexc_func()                                   */
    /*****************************************************************/
    case '3':
      prtmsg("QMAD: CREXC - Entering QMAD.\n");

      /***************************************************************/
      /* Find out what test case is being tested                     */
      /***************************************************************/
      switch(parm[1])
      {
        case '1':
        /********************************/
        /* Checks the parameters passed */
        /********************************/
        if (!memcmp(&macro_tested, &ecbptr()->ebw003, strlen(macro_tested)))
           prtmsg("QMAD: Msg from CREXC - The parameters passed are CORRECT.\n");
        else
           prterr("QMAD: Msg from CREXC - The parameters passed are INCORRECT.");
        break;

        /*************************************************************/
        /* Default Case:                                             */
        /*************************************************************/
        default:
          prterr("crexc: An invalid function case was entered.");
        break;
      }/*end switch                                                  */

      prtmsg("QMAD: CREXC - Exiting QMAD.\n");
    break;/* end of crexc_func() case                                */


    /*****************************************************************/
    /* Function '4':  cremc_func()                                   */
    /*****************************************************************/
    case '4':
      prtmsg("QMAD: CREMC - Entering QMAD.\n");

      /***************************************************************/
      /* Find out what test case is being tested                     */
      /***************************************************************/
      switch(parm[1])
      {
        case '1':
          prtmsg("QMAD: Msg from CREMC - This msg should come before the msg from CREXC.\n");
        break;
        /*************************************************************/
        /* Default Case:                                             */
        /*************************************************************/
        default:
          prterr("cremc: An invalid function case was entered.");
        break;
      }/*end switch                                                  */

      prtmsg("QMAD: CREMC - Exiting QMAD.\n");
    break;/* end of cremc_func() case                                */

    /*****************************************************************/
    /* Function '5':  unhka_func()                                   */
    /*****************************************************************/
    case '5':
      /***************************************************************/
      /* Find out what test case is being tested                     */
      /***************************************************************/
      switch(parm[1])
      {
        /*************************************************************/
        /* Test Case 1:       Attempt to unhook/rehook core block.   */
        /*                                                           */
        /* Test Result:       CTL-000007 Dump.                       */
        /* Error Msg Output:  None                                   */
        /*************************************************************/
        case '1':
          prtmsg("unhka: 1-Attempting to unhook core block.\n");

          /***********************************************************/
          /* If DataLevel D4 is occupied, release the core block.    */
          /* Then print out a message that states whether D4 is      */
          /* occupied or not.                                        */
          /***********************************************************/
          if(levtest(D4))
            relcc(D4);

          /***********************************************************/
          /* Get a common block on DataLevel D4. (L2: 1055 bytes)    */
          /***********************************************************/
          getcc(D4, (enum t_getfmt) (GETCC_TYPE), L2);

          if(levtest(D4))           /* if DataLevel D4 is occupied.. */
          {
            prtmsg("unhka: 1-Data level D4 now has a core block.\n");

            sprintf((char*)ecbptr()->ce1cr4, "unhka: 1-8-byte field before unhka() of D4 cbrw : %s\n", save_area);
            prtmsg((const char*)ecbptr()->ce1cr4);
            sprintf((char*)ecbptr()->ce1cr4, "unhka: 1-Address of cbrw on data level D4: %i\n",(int) ecbptr()->ce1cr4);
            prtmsg((const char*)ecbptr()->ce1cr4);
          }
          else
            prterr("unhka: 1-Data level D4 was unable to get a core block.\n");

          /***********************************************************/
          /* Next, unhook the common block into an 8-byte field.     */
          /***********************************************************/
          unhka(D4, UNHKA_UNPROTECTED, save_area);
          prtmsg("unhka: 1-8-byte field now contains the info from the D4 cbrw.\n");

          if(levtest(D4))           /* if DataLevel D4 is occupied.. */
            prterr("unhka: 1-Data level D4 should have had its core block unhooked.\n");
          else
            prtmsg("unhka: 1-Data level D4 is now unhooked and does not contain a core block.\n");

          /***********************************************************/
          /* If DataLevel D5 is occupied, release the core block.    */
          /* Then print out a message that states whether D5 is      */
          /* occupied or not.                                        */
          /***********************************************************/
          if(levtest(D5))
            relcc(D5);

          if(levtest(D5))           /* if DataLevel D5 is occupied.. */
            prterr("unhka: 1-Data level D5 should not have a core block.\n");
          else
          {
            prtmsg("unhka: 1-Data level D5 does not have a core block.\n");

            sprintf(buffer, "unhka: 1-Address of cbrw on data level D5 before rehka(): %i\n", (int)ecbptr()->ce1cr5);
            prtmsg(buffer);
          }

          /***********************************************************/
          /* Get a common block on DataLevel D5. (L2: 1055 bytes)    */
          /***********************************************************/
          getcc(D5, (enum t_getfmt) (GETCC_TYPE), L2);

          /***********************************************************/
          /* Rehook Common Block that was previously unhooked from   */
          /* data level D4 to data level D5.                         */
          /*                                                         */
          /* The following rehka() should give an "INVALID GLOBAL    */
          /* AREA SPECIFIED " error (OPR-I000074). A global area that*/
          /* is not valid was specified. Taking REHKA_UNPROTECTED as */
          /* an argument rather than REHKA_GLOBAL would fix this.    */
          /***********************************************************/
          prtmsg("unhka: 1-Should get a OPR-74 soon.\n");
          rehka(D5, REHKA_GLOBAL, save_area);

          prterr("unhka: 1-Test Failed, A global area that is not valid was able to be rehooked.");

          /***********************************************************/
          /* Release the working storage block. relcc(D4) gives you  */
          /* a CTL-0000I7 dump because an unhka() was done on D4, so */
          /* I have used crusa(1, D4).                               */
          /***********************************************************/
          crusa(1, D4);
          relcc(D5);
        break;
        /*************************************************************/
        /* Default Case:                                             */
        /*************************************************************/
        default:
          prterr("unhka: An invalid function case was entered.");
        break;
      }/*end switch                                                  */
    break;/* end of unhka_func() case                                */


    /*****************************************************************/
    /* Default Case:  if an invalid function was somehow entered...  */
    /*****************************************************************/
    default:
      prterr("An invalid function was entered.");
    break;/* end of default case                                     */
  }/*end function switch                                             */

  exit(0);
}/*end extern "C"                                                    */
