/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2005                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20041001  Initial Version                                         */
/* 20041020  f4523- Add a test case to test swisc_entrc() using      */
/*                  iStream parameter = 0.                           */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/* Load module name: QMAH                                            */
/* Function Name:    QMAH()                                          */
/*                                                                   */
/* Test cases 1-3: Called by corhc_func in qmac20.cpp to test the    */
/*                 exclusive control over a resource.                */
/*                                                                   */
/* Time Slice test cases (4-9) was moved to TMSL driver  2004/01/15  */
/*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tpf/tpfapi.h>
#include <tpf/sysapi.h>
#include <tpf/c_eb0eb.h>
#include <unistd.h>                                                             //C64CF
#include <tpf/tpfglbl.h>
#include <tpf/c_globz.h>
#include <c_api5.h>

struct para {
   unsigned int     case_No;
   char             printmsg;
   unsigned int     ECB_No;
   unsigned int     num_ECBs;
   char             macro_name[6];
   void*            addr;
   unsigned short   loop_time;
   unsigned short   skip_hwm;
};

#define   MSG_SIZE            255
#define   ISTREAM_ZERO        88                                                //f4523

extern "C" {

void swisc_enter_test();

void QMAH() {

   /* Variables used in corhc_func */
   void *correct_pointer;
   struct para * parameters;
   char msg[MSG_SIZE];
   struct ev0bk qb = { "cremcOK" };

   /* If called from a tpf_cresc, then called from QMAC90 */
   if (levtest(D0))
   {
      swisc_enter_test();
      exit(0);
   }

   /* Gets parameters passed */
   parameters = (struct para*) &ecbptr()->ebw000;
   ecbptr()->ebcm01 = parameters->printmsg;

   /* What's the case_No of this ECB? */
   switch (parameters->case_No) {

      case 1:
      case 2:
      case 3:

        correct_pointer = glob(_q02gcvf);

        /* Verifies the string passed is correct */
        if ( parameters->ECB_No != parameters->case_No
                   || strcmp(parameters->macro_name,"corhc")
                   || parameters->addr != correct_pointer)  {
            sprintf(msg, "qmah: ECB#%d: parameters passed by CREMC is NOT correct;"
                         " Exits from here."
                       , parameters->ECB_No);
            prterr(msg);
            exit(EXIT_FAILURE);
        }

        sprintf(msg,"qmah: ECB#%d entering.\n", parameters->ECB_No);
        prtmsg(msg);

        if (parameters->case_No == 3)  {
           /********************************/
           /* post "cremcOK" with no error */
           /********************************/
           postc(&qb, EVENT_CNT, 0);
           sprintf(msg,"qmah: ECB#%d postc(\"cremcOK\").\n", parameters->ECB_No);
           prtmsg(msg);
        }

        /**************************************************************/
        /* Try to Hold on the same resource and print msg if succeed  */
        /* Control is not returned to the issuing ECB until exclusive */
        /* control over the resource is obtained.                     */
        /**************************************************************/
        sprintf(msg,"qmah: ECB#%d- Trying to hold on global record"
               " _q02gcvf, Invokes corhc().\n", parameters->ECB_No);
        prtmsg(msg);
        corhc(correct_pointer);
        sprintf(msg,"qmah: ECB#%d- Holds on global record _q02gcvf"
                    " successfully.\n", parameters->ECB_No);
        prtmsg(msg);

        /*****************************************************************/
        /* Delays processing of an entry in order to allow processing of */
        /* other entries. The current entry is placed on the input list  */
        /*****************************************************************/
        dlayc();
        sprintf(msg,"qmah: ECB#%d- Invokes dlayc()\n", parameters->ECB_No);
        prtmsg(msg);

        /********************************************************/
        /* Try to release the resource and print msg if succeed */
        /********************************************************/
        sprintf(msg,"qmah: ECB#%d- Release on global record"
                    " _q02gcvf, Invokes coruc().\n", parameters->ECB_No);
        prtmsg(msg);

        coruc(correct_pointer);

        sprintf(msg,"qmah: ECB#%d- Releases global record _q02gcvf"
                    " successfully.\n", parameters->ECB_No);
        prtmsg(msg);

        sprintf(msg,"qmah - ECB#%d exiting.\n", parameters->ECB_No);
        prtmsg(msg);
        exit(0);
        break;
   }

   sprintf(msg, "qmah: ECB#%d- The execution of the driver should not reach here."
              , parameters->ECB_No);
   prterr(msg);
   exit(0);
}



/* Parameters are passed to this program via data level 0 as part of a      */
/* tpf_cresc call from QMAC90.                                              */

void swisc_enter_test()
{
   struct swisc_enter_input param;
   char *prog = "QMAH";  /* this program */

   /* input data stored in D0 from QMAC90 */
   struct _args {
     char verbose;
     short int iStream;
     unsigned short isTotal;
     char data;
   } *args = NULL;

   char buffer[256];  /* temporary for holding string output */

   args = (struct _args *) ECB_CBRW_CA(D0);
   ecbptr()->ebcm01 = args->verbose;

   sprintf(buffer, "entrc: Running on IStream %d of %d.\n",                     //f4523
           ecbptr()->ce1isn, args->isTotal);
   prtmsg(buffer);

   /* Verify that the ECB was created correctly.                            */
   if (args->data != 'T')
   {
      prterr("entrc: 1-ECB was not created correctly.");
   }
   /* Test case 2 finished? This block should be put before the match comparison block */
   else if (ecbptr()->ebw080 == ISTREAM_ZERO)                                   //f4523
   {
      prtmsg("entrc: 2-Completed the test using iStream parameter 0.\n");       //f4523
   }
   /* Check if current IStream matches the expected IStream.                */
   else if ((args->iStream) && (args->iStream != ecbptr()->ce1isn))
   {
      prterr("entrc: 1-ECB not running on expected IStream.");
   }
   /* Test case 1 finished?                                                             */
   else if (ecbptr()->ce1isn >= args->isTotal)
   {
      prtmsg("entrc: 1-Successfully Completed.\n");

      /* Set the flag: Use zero for IStream parameter */
      ecbptr()->ebw080 = ISTREAM_ZERO;                                          //f4523

      /* Populate the creation parameters */
      param.program = prog;                                                     //f4523
      param.istream = 0;                                                        //f4523
      param.bypass = SWISC_BYPASS_NO;                                           //f4523

      prtmsg("entrc: 2-Test using iStream parameter 0, call swisc_enter().\n"); //f4523

      /* Switch IStreams using the same ECB */
      swisc_enter(&param);                                                      //f4523

      /* This message should never display */
      prterr("entrc: 2-THIS MESSAGE SHOULD NEVER DISPLAY");                     //f4523

   }
   /* Switch to the next IStream                                            */
   else
   {
     /* Set the IStream to switch to into the ECB */
     args->iStream++;

     /* Populate the creation parameters */
     param.program = prog;
     param.istream = args->iStream;
     param.bypass = SWISC_BYPASS_NO;

     /* Display where the ECB is switching to */
     sprintf(buffer, "entrc: 1-Switching to IStream %d of %d.\n",
             param.istream, args->isTotal);
     prtmsg(buffer);

     /* Switch IStreams using the same ECB */
     swisc_enter(&param);

     /* This message should never display */
     prterr("entrc: 1-THIS MESSAGE SHOULD NEVER DISPLAY");
   }


   /* The following only executes if there is an error on completion.       */
   if (levtest(D0))
   {
      relcc(D0);
   }

   exit(0);
}

}  /* end of extern "C" */

