/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2005                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20041001  Initial Version                                         */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <tpf/tpfapi.h>
#include <tpf/sysapi.h>
#include <errno.h>
#include <qmac23.h>
#include <c_api5.h>

extern "C" void QMAI()
{
   // Struct for the test case of tpf_movec_EVM_return()

   struct ev0bk qb = { "cremcOK" };
   struct parms_passed* parameters;
   char   msg[MSG_SIZE],
          *frommem;

   unsigned int rtval;

   frommem = (char*) malloc(MALLOC_SIZE);
   if (! frommem) {
      prterr("A- malloc() error: Out of storage. Exits from here.");
      exit(1);
   }
   memset(frommem, 'Y', MALLOC_SIZE);

   parameters = (struct parms_passed*) &ecbptr()->ebw000;
   ecbptr()->ebcm01 = parameters->printmsg;

   if (parameters->is_errorCase == FALSE)
   {
      prtmsg("Child ECB: Start testing NORMAL cases of tpf_movec_EVM_return().\n");

      /*************************************************/
      /* Testing NORMAL case of tpf_movec_EVM_return() */
      /*************************************************/

      // Moving text to parent's EHEAP
      prtmsg("tpf_movec_EVM_return: 11- Move data to parent's EHEAP.\n");
      rtval = tpf_movec_EVM_return( &(parameters->text),
                                    ecbp2()->ce2sva,
                                    parameters->parent_addr_eheap,
                                    parameters->ECB_SVA_addr,
                                    sizeof(parameters->text) );

      if (rtval != sizeof(parameters->text))  {
         sprintf(msg, "tpf_movec_return: 11- Only %d out of %d bytes moved.\n"
                    , rtval
                    , (int) sizeof(parameters->text));
         prterr(msg);
      }

      // Moves text to parent's stack
      prtmsg("tpf_movec_EVM_return: 12- Move data to parent's stack.\n");
      rtval = tpf_movec_EVM_return( &(parameters->text),
                                    (void*) ecbp2()->ce2sva,
                                    parameters->parent_addr_stack,
                                    (void*) parameters->ECB_SVA_addr,
                                    sizeof(parameters->text) );

      if (rtval != sizeof(parameters->text))  {
         sprintf(msg, "tpf_movec_return: 12- Only %d out of %d bytes moved.\n"
                     , rtval
                     , (int) sizeof(parameters->text));
         prterr(msg);
      }

      // Move text to parent's private EVM address space
      prtmsg("tpf_movec_EVM_return: 13- Move data to parent's private EVM.\n");
      rtval = tpf_movec_EVM_return( &(parameters->text),
                                    (void*) ecbp2()->ce2sva,
                                    parameters->parent_addr_private,
                                    (void*) parameters->ECB_SVA_addr,
                                    sizeof(parameters->text) );

      if (rtval != sizeof(parameters->text))  {
         sprintf(msg, "tpf_movec_return: 13- Only %d out of %d bytes moved.\n"
                     , rtval
                     , (int) sizeof(parameters->text));
         prterr(msg);
      }

      prtmsg("Child ECB: Completes tpf_movec_EVM_return() calls.\n");

   }
   else
   {

      prtmsg("Child ECB: Start testing ERROR cases of tpf_movec_EVM_return().\n");

      /************************************************/
      /* Testing ERROR case of tpf_movec_EVM_return() */
      /************************************************/

      // (error 1st input): From address: INVALID_EVM
      prtmsg("tpf_movec_EVM_return: 31- Testing error From address.\n");
      rtval = tpf_movec_EVM_return( (void*)INVALID_EVM,
                                    ecbp2()->ce2sva,
                                    parameters->parent_addr_eheap,
                                    parameters->ECB_SVA_addr,
                                    sizeof(parameters->text) );

      if (rtval != sizeof(parameters->text))  {
         sprintf(msg, "tpf_movec_EVM_return: 31- rc: %d, %s\n",
                       rtval,
                       strerror(errno) );
         prtmsg(msg);

         if (errno != ETPFMOVE_BADFR)
            prterr("tpf_movec_EVM_return: 31- Error errno. Should be 5050.");
      }
      else
         prterr("tpf_movec_EVM_return: 31- Function did not fail with error input.");


      // (error 3rd input): To address: INVALID_EVM
      prtmsg("tpf_movec_EVM_return: 32- Testing error To address.\n");
      rtval = tpf_movec_EVM_return( frommem,
                                    ecbp2()->ce2sva,
                                    (void*) INVALID_EVM,
                                    parameters->ECB_SVA_addr,
                                    MALLOC_SIZE );
      if (rtval != MALLOC_SIZE)  {
         sprintf(msg, "tpf_movec_EVM_return: 32- rc: %d, %s\n",
                       rtval,
                       strerror(errno) );
         prtmsg(msg);

         if (errno != ETPFMOVE_BADTO)
            prterr("tpf_movec_EVM_return: 32- Error errno %d. Should be 5051");

      }
      else
         prterr("tpf_movec_EVM_return: 32- Function did not fail with error input.");

      // (error 2nd input): Error SVA of From ECB
      rtval = tpf_movec_EVM_return( frommem,
                                    ecbp2()->ce2sva + 1,
                                    parameters->parent_addr_eheap,
                                    parameters->ECB_SVA_addr,
                                    TEXT_SIZE );

      if (rtval != TEXT_SIZE)  {
         sprintf(msg, "tpf_movec_return: 33- rc: %d, %s\n"
                    , rtval
                    , strerror(errno));
         prtmsg(msg);
      }
      else
         prterr("tpf_movec_EVM_return: 33- Function did not fail with error input.");

      // (error 4th input): Error SVA of To ECB
      rtval = tpf_movec_EVM_return( parameters->parent_addr_eheap,
                                    parameters->ECB_SVA_addr,
                                    frommem,
                                    ecbp2()->ce2sva + 1,
                                    TEXT_SIZE );

      if (rtval != TEXT_SIZE )  {
         sprintf(msg, "tpf_movec_return: 34- rc: %d, %s\n"
                    , rtval
                    , strerror(errno));
         prtmsg(msg);
      }
      else
         prterr("tpf_movec_EVM_return: 34- Function did not fail with error input.");

   }
   postc(&qb, EVENT_CNT, 0);
   prtmsg("Child ECB: Exits.\n");
   exit(0);
}
