/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2005                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20070713  Initial Version                                         */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name:  QMAO                                               */
/*                                                                   */
/* Contents:      QMAO()                                             */
/*                                                                   */
/* Abstract:      This program is used to test the PAUSC CTL-574     */
/*                system error for CP User Enhancements              */
/*                                                                   */ 
/* Description:   This program is swisc_create-d by the pausc_func() */
/*                in qmac85.cpp file.  It will loop uninterrupted    */
/*                for the specified amount of time.  CTL-10 process- */
/*                ing was modified for this segment in the API5      */
/*                control file.                                      */        
/*********************************************************************/

#include <sys/time.h>
#include <tpf/tpfapi.h>
#include <stdlib.h> 
#include <c_api5.h>

extern "C" void QMAO()
{
  int *caseNum;
  int addedTime;
  struct wtopc_header msg_hdr;
  struct timeval targetTime;
  struct timeval currentTime;
  struct ev0bk qb = { {"pauscOK"} };
   
  caseNum = (int *)ecbptr()->ce1cr0;

  if(*caseNum == 1)
  {
  	//Loop for more than 2 minutes -- this should generate the 574 system error
    addedTime = 150;  //seconds
  }
    
  if(*caseNum == 2)
  {
  	//Loop for less than 2 minutes -- this should generate the 583 system error
    addedTime = 100;  //seconds
  } 

   //add the addedTime value to the current TOD
   gettimeofday(&currentTime, NULL);
   targetTime.tv_sec = currentTime.tv_sec + addedTime;  

   postc(&qb, EVENT_CNT, 0);  //issue the postc so that the PAUSC_BEGIN time can start

   //CTL-10 processing modified so that this loop can run uninterrupted
   
   do {
     gettimeofday(&currentTime, NULL);
   } while (targetTime.tv_sec > currentTime.tv_sec);
   
 
   wtopc_insert_header(&msg_hdr, "QMAO", 1, 'I', WTOPC_SYS_TIME);
   wtopc("QMAO ECB ENDED",WTOPC_EBROUT, WTOPC_CHAIN_END, &msg_hdr);
} // end of extern "C"
